@@ -2,7 +2,7 @@
  *
  * rdf_digest.c - RDF Digest Factory / Digest implementation
  *
- * $Id: rdf_digest.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_digest.c 12178 2007-05-01 03:32:14Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -79,6 +79,8 @@ librdf_digest_register_factory(librdf_world *world, const char *name,
   librdf_digest_factory *d, *digest;
   char *name_copy;
 
+  librdf_world_open(world);
+
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for digest %s\n", name);
 #endif
@@ -133,6 +135,8 @@ librdf_get_digest_factory(librdf_world *world, const char *name)
 {
   librdf_digest_factory *factory;
 
+  librdf_world_open(world);
+
   /* return 1st digest if no particular one wanted - why? */
   if(!name) {
     factory=world->digests;
@@ -173,10 +177,12 @@ librdf_get_digest_factory(librdf_world *world, const char *name)
  * Return value: new #librdf_digest object or NULL
  **/
 librdf_digest*
-librdf_new_digest(librdf_world *world, char *name)
+librdf_new_digest(librdf_world *world, const char *name)
 {
   librdf_digest_factory* factory;
   
+  librdf_world_open(world);
+
   factory=librdf_get_digest_factory(world, name);
   if(!factory)
     return NULL;
@@ -208,6 +214,8 @@ librdf_new_digest_from_factory(librdf_world *world,
 {
   librdf_digest* d;
 
+  librdf_world_open(world);
+
   d=(librdf_digest*)LIBRDF_CALLOC(librdf_digest, 1, sizeof(librdf_digest));
   if(!d)
     return NULL;
@@ -413,7 +421,7 @@ librdf_digest_print(librdf_digest* digest, FILE* fh)
  * librdf_init_digest:
  * @world: redland world object
  *
- * INTERNAL - Initialise the librdf_digest class.
+ * INTERNAL - Initialise the digest module.
  *
  **/
 void
@@ -443,7 +451,7 @@ librdf_init_digest(librdf_world *world)
  * librdf_finish_digest:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_digest class.
+ * INTERNAL - Terminate the digest module.
  *
  **/
 void
@@ -461,32 +469,30 @@ int main(int argc, char *argv[]);
 
 struct t
 {
-  char *type;
-  char *result;
+  const char *type;
+  const char *result;
 };
 
 int
 main(int argc, char *argv[]) 
 {
   librdf_digest* d;
-  char *test_data="http://purl.org/net/dajobe/";
+  const char *test_data="http://purl.org/net/dajobe/";
   struct t test_data_answers[]={
-    {"MD5", "78f68989ea3ded775925a1a00fe49abe"},
-    {"SHA1", "0452bca820dfdaa707f0278216c598f185867993"},
-    {"RIPEMD160", "75c5de1d21e7e96400d5d9f42ccce6552bb685a5"},
+    {"MD5", "80b52def747e8748199c1a0cf66cb35c"},
+    {"SHA1", "67d6a7b73504ce5c6b47fd6db9b7c4939bfe5174"},
+    {"RIPEMD160", "83ce259f0c23642a95fc92fade583e6d8e15784d"},
     {NULL, NULL},
   };
+  int failures=0;
 
   int i;
   struct t *answer=NULL;
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   librdf_world *world;
   
   world=librdf_new_world();
   
-  /* initialise digest module */
-  librdf_init_digest(world);
-  
   for(i=0; ((answer= &test_data_answers[i]) && answer->type != NULL) ; i++) {
     char *s;
     
@@ -512,9 +518,10 @@ main(int argc, char *argv[])
     fputc('\n', stdout);
 
     s=librdf_digest_to_string(d);
-    if(strcmp(s, answer->result))
+    if(strcmp(s, answer->result)) {
       fprintf(stderr, "%s: %s digest is wrong - expected: %s\n", program, answer->type, answer->result);
-    else
+      failures++;
+    } else
       fprintf(stderr, "%s: %s digest is correct\n", program, answer->type);
     LIBRDF_FREE(cstring, s);
     
@@ -523,13 +530,10 @@ main(int argc, char *argv[])
   }
   
   
-  /* finish digest module */
-  librdf_finish_digest(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
-  return(0);
+  return failures;
 }
 
 #endif
