@@ -2,7 +2,7 @@
  *
  * rdf_hash.c - RDF Hash interface - set of (key: value) pairs with dups
  *
- * $Id: rdf_hash.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_hash.c 12195 2007-05-02 05:48:23Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -75,7 +75,7 @@ static void librdf_hash_keys_iterator_finished(void* iterator);
 /**
  * librdf_init_hash:
  *
- * INTERNAL - Initialise the librdf_hash module.
+ * INTERNAL - Initialise the hash module.
  *
  * Initialises and registers all
  * compiled hash modules.  Must be called before using any of the hash
@@ -99,7 +99,7 @@ librdf_init_hash(librdf_world *world)
  * librdf_finish_hash:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_hash module.
+ * INTERNAL - Terminate the hash module.
  *
  **/
 void
@@ -165,6 +165,8 @@ librdf_new_hash_datum(librdf_world *world, void *data, size_t size)
 {
   librdf_hash_datum *datum;
 
+  librdf_world_open(world);
+
   /* get one from free list, or allocate new one */ 
   if((datum=world->hash_datums_list)) {
     world->hash_datums_list=datum->next;
@@ -217,6 +219,8 @@ librdf_hash_register_factory(librdf_world *world, const char *name,
   librdf_hash_factory *hash, *h;
   char *name_copy;
   
+  librdf_world_open(world);
+
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for hash %s\n", name);
 #endif
@@ -273,6 +277,8 @@ librdf_get_hash_factory(librdf_world *world, const char *name)
 {
   librdf_hash_factory *factory;
 
+  librdf_world_open(world);
+
   /* return 1st hash if no particular one wanted - why? */
   if(!name) {
     factory=world->hashes;
@@ -310,6 +316,8 @@ librdf_new_hash(librdf_world *world, const char* name)
 {
   librdf_hash_factory *factory;
 
+  librdf_world_open(world);
+
   factory=librdf_get_hash_factory(world, name);
   if(!factory)
     return NULL;
@@ -328,11 +336,13 @@ librdf_new_hash(librdf_world *world, const char* name)
  * Return value: a new #librdf_hash object or NULL on failure
  */
 librdf_hash*
-librdf_new_hash_from_factory (librdf_world *world,
-                              librdf_hash_factory* factory)
+librdf_new_hash_from_factory(librdf_world *world,
+                             librdf_hash_factory* factory)
 {
   librdf_hash* h;
 
+  librdf_world_open(world);
+
   h=(librdf_hash*)LIBRDF_CALLOC(librdf_hash, sizeof(librdf_hash), 1);
   if(!h)
     return NULL;
@@ -373,6 +383,8 @@ librdf_new_hash_from_string(librdf_world *world, const char *name,
 {
   librdf_hash* hash;
 
+  librdf_world_open(world);
+
   hash=librdf_new_hash(world, name);
   if(!hash)
     return NULL;
@@ -402,6 +414,8 @@ librdf_new_hash_from_array_of_strings(librdf_world *world, const char *name,
 {
   librdf_hash* hash;
 
+  librdf_world_open(world);
+
   hash=librdf_new_hash(world, name);
   if(!hash)
     return NULL;
@@ -501,7 +515,7 @@ librdf_free_hash(librdf_hash* hash)
  * Return value: non 0 on failure
  **/
 int
-librdf_hash_open(librdf_hash* hash, char *identifier,
+librdf_hash_open(librdf_hash* hash, const char *identifier,
                  int mode, int is_writable, int is_new,
                  librdf_hash* options) 
 {
@@ -1188,13 +1202,13 @@ librdf_hash_print_keys(librdf_hash* hash, FILE *fh)
  *
  **/
 void
-librdf_hash_print_values(librdf_hash* hash, char *key_string, FILE *fh)
+librdf_hash_print_values(librdf_hash* hash, const char *key_string, FILE *fh)
 {
   librdf_hash_datum *key, *value;
   librdf_iterator* iterator;
   int first=1;
   
-  key=librdf_new_hash_datum(hash->world, key_string, strlen(key_string));
+  key=librdf_new_hash_datum(hash->world, (char*)key_string, strlen(key_string));
   if(!key)
     return;
   
@@ -1571,6 +1585,100 @@ librdf_hash_put_strings(librdf_hash* hash, const char *key, const char *value)
   return librdf_hash_put(hash, &key_hd, &value_hd);
 }
 
+
+/**
+ * librdf_hash_interpret_template:
+ * @template_string: template string to interprate
+ * @dictionary: dictionary of key/values to substitute
+ * @prefix: prefix to mark a key in the template
+ * @suffix: suffix to mark a key in the template
+ * 
+ * Interpret keys in a template string to their value in a dictionary.
+ * 
+ * Can be used to do variable substitution for a string where
+ * the syntax that marks the variable is defined by the @prefix
+ * and @suffix strings, and the variables are stored in the @dictionary
+ * hash table.
+ *
+ * Return value: Newly allocated string, or NULL on failure
+ **/
+unsigned char*
+librdf_hash_interpret_template(const unsigned char* template_string,
+                               librdf_hash* dictionary,
+                               const unsigned char* prefix, 
+                               const unsigned char* suffix) 
+{
+  raptor_stringbuffer* sb;
+  unsigned char* result=NULL;
+  size_t len;
+  size_t prefix_len=strlen((const char*)prefix);
+  size_t suffix_len=strlen((const char*)suffix);
+  
+  sb=raptor_new_stringbuffer();
+  if(!sb)
+    return NULL;
+  
+  len=strlen((const char*)template_string);
+  
+  while(*template_string) {
+    unsigned char* p;
+    unsigned char* s;
+    librdf_hash_datum key; /* static */
+    librdf_hash_datum *hd_value;
+    size_t len2;
+    
+    p=(unsigned char*)strstr((const char*)template_string, (const char*)prefix);
+    if(!p) {
+      /* No more prefixes found so append rest of template */
+      raptor_stringbuffer_append_counted_string(sb, template_string, len, 1);
+      break;
+    }
+    len2=p-template_string;
+    if(len2)
+      raptor_stringbuffer_append_counted_string(sb, template_string, len2, 1);
+
+    template_string += len2 + prefix_len;  len -= len2 + prefix_len;
+    
+    /* key starts here */
+    key.data=(void*)template_string;
+    
+    s=(unsigned char*)strstr((const char*)template_string, (const char*)suffix);
+    if(!s)
+      /* template ended without a closing key suffix so just give up */
+      break;
+
+    /* now have key */
+    len2= s - (unsigned char*)key.data;
+    key.size= len2;
+
+    /* move past key and suffix */
+    template_string += len2 + suffix_len;  len -= len2 + suffix_len;
+
+    hd_value=librdf_hash_get_one(dictionary, &key);
+    /* append value if there is one */
+    if(hd_value) {
+      raptor_stringbuffer_append_counted_string(sb,
+                                                (const unsigned char*)hd_value->data, 
+                                                hd_value->size, 1);
+      librdf_free_hash_datum(hd_value);
+    }
+    
+  }
+
+  /* Generate a string result */
+  len=raptor_stringbuffer_length(sb);
+  if(len) {
+    result=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
+    raptor_stringbuffer_copy_to_string(sb, result, len);
+  }
+  
+  raptor_free_stringbuffer(sb);
+  return result;
+}
+
+
+
+
 #endif
 
 
@@ -1587,15 +1695,15 @@ int
 main(int argc, char *argv[]) 
 {
   librdf_hash *h, *h2, *ch;
-  char *test_hash_types[]={"bdb", "memory", NULL};
-  char *test_hash_values[]={"colour","yellow", /* Made in UK, can you guess? */
+  const char *test_hash_types[]={"bdb", "memory", NULL};
+  const char *test_hash_values[]={"colour","yellow", /* Made in UK, can you guess? */
 			    "age", "new",
 			    "size", "large",
                             "colour", "green",
 			    "fruit", "banana",
                             "colour", "yellow",
 			    NULL, NULL};
-  char *test_duplicate_key="colour";
+  const char *test_duplicate_key="colour";
   const char *test_hash_array[]={"shape", "cube",
                                  "sides", "6", /* for testing get as long */
                                  "3d", "yes", /* testing bool */
@@ -1604,19 +1712,20 @@ main(int argc, char *argv[])
                                  "creator", "rubik",
                                  NULL};
   const char * const test_hash_string="field1='value1', field2='\\'value2', field3='\\\\', field4='\\\\\\'', field5 = 'a' ";
-  char *test_hash_delete_key="size";
+  const char *test_hash_delete_key="size";
+  const unsigned char* template_string=(const unsigned char*)"the shape is %{shape} and the sides are %{sides} created by %{rubik}";
+  const unsigned char* template_expected=(const unsigned char*)"the shape is cube and the sides are 6 created by ";
   int i,j;
-  char *type;
+  const char *type;
   librdf_hash_datum hd_key, hd_value; /* on stack */
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   int b;
   long l;
+  unsigned char* template_result;
   librdf_world *world;
   
   world=librdf_new_world();
-  
-  /* initialise hash module */
-  librdf_init_hash(world);
+  librdf_world_open(world);
   
   if(argc ==2) {
     type=argv[1];
@@ -1654,8 +1763,8 @@ main(int argc, char *argv[])
     
     
     for(j=0; test_hash_values[j]; j+=2) {
-      hd_key.data=test_hash_values[j];
-      hd_value.data=test_hash_values[j+1];
+      hd_key.data=(char*)test_hash_values[j];
+      hd_value.data=(char*)test_hash_values[j+1];
       fprintf(stdout, "%s: Adding key/value pair: %s=%s\n", program,
 	      (char*)hd_key.data, (char*)hd_value.data);
       
@@ -1670,7 +1779,7 @@ main(int argc, char *argv[])
     }
     
     fprintf(stdout, "%s: Deleting key '%s'\n", program, test_hash_delete_key);
-    hd_key.data=test_hash_delete_key;
+    hd_key.data=(char*)test_hash_delete_key;
     hd_key.size=strlen((char*)hd_key.data);
     librdf_hash_delete_all(h, &hd_key);
     
@@ -1772,7 +1881,6 @@ main(int argc, char *argv[])
   /* librdf_hash_close(h2); */
   librdf_free_hash(h2);
 
-
   h2=librdf_new_hash(world, NULL);
   fprintf(stdout, "%s: Initialising hash from string >>%s<<\n", program, 
           test_hash_string);
@@ -1785,10 +1893,27 @@ main(int argc, char *argv[])
   librdf_free_hash(h2);
 
    
-  /* finish hash module */
-  librdf_finish_hash(world);
+  fprintf(stdout, "%s: Subtituting into template >>%s<<\n", program, 
+          template_string);
+  h2=librdf_new_hash(world, NULL);
+  librdf_hash_from_array_of_strings(h2, test_hash_array);
 
-  LIBRDF_FREE(librdf_world, world);
+  template_result=librdf_hash_interpret_template(template_string, h2, 
+                                                 (const unsigned char*)"%{", 
+                                                 (const unsigned char*)"}");
+  if(strcmp((const char*)template_result, (const char*)template_expected)) {
+    fprintf(stdout, "%s: Templating failed. Result was >>%s<< but expected >>%s<<\n", program, 
+            template_result, template_expected);
+    exit(1);
+  } else
+    fprintf(stdout, "%s: resulting in >>%s<<\n", program, template_result);
+
+  LIBRDF_FREE(cstring, template_result);
+
+  librdf_free_hash(h2);
+
+   
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
