@@ -2,7 +2,7 @@
  *
  * rdf_init.c - Redland library initialisation / termination
  *
- * $Id: rdf_init.c 10718 2006-03-28 07:26:14Z dajobe $
+ * $Id: rdf_init.c 11933 2007-02-25 20:02:09Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -34,12 +34,16 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #ifdef WITH_THREADS
 #include <pthread.h>
 #endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h> /* for getpid() */
+#endif
 
 /* for gettimeofday */
 #if TIME_WITH_SYS_TIME
@@ -111,18 +115,32 @@ const unsigned int librdf_version_decimal = LIBRDF_VERSION_DECIMAL;
 /**
  * librdf_new_world:
  *
- * Create a new redland execution environment.
+ * Create a new Redland execution environment.
+ *
+ * Once this constructor is called to build a #librdf_world object
+ * several functions may be called to set some parameters such as
+ * librdf_world_set_error(), librdf_world_set_warning(),
+ * librdf_world_set_logger(), librdf_world_set_digest(),
+ * librdf_world_set_feature().
+ *
+ * The world object needs initializing using librdf_world_open()
+ * whether or not the above functions are called.  It will be
+ * automatically called by all object constructors in Redland 1.0.6
+ * or later, but for earlier versions it MUST be called before using
+ * any other part of Redland.
  *
  * Returns: a new #librdf_world or NULL on failure
  */
 librdf_world*
 librdf_new_world(void) {
-  librdf_world *world=(librdf_world*)LIBRDF_CALLOC(librdf_world, sizeof(librdf_world), 1);
+  librdf_world *world;
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval tv;
   struct timezone tz;
 #endif
 
+  world=(librdf_world*)LIBRDF_CALLOC(librdf_world, sizeof(librdf_world), 1);
+
 #ifdef HAVE_GETTIMEOFDAY
   if(!gettimeofday(&tv, &tz)) {
     world->genid_base=tv.tv_sec;
@@ -222,6 +240,9 @@ librdf_world_init_mutex(librdf_world* world)
 void
 librdf_world_open(librdf_world *world)
 {
+  if(world->opened++)
+    return;
+  
   librdf_world_init_mutex(world);
   
   /* Digests first, lots of things use these */
@@ -508,6 +529,27 @@ librdf_destroy_world(void)
 }
 
 
+/**
+ * librdf_basename:
+ * @name: path
+ * 
+ * Get the basename of a path
+ * 
+ * Return value: filename part of a pathname
+ **/
+const char*
+librdf_basename(const char *name)
+{
+  char *p;
+  if((p=strrchr(name, '/')))
+    name=p+1;
+  else if((p=strrchr(name, '\\')))
+    name=p+1;
+
+  return name;
+}
+
+
 #if defined (LIBRDF_DEBUG) && defined(LIBRDF_MEMORY_SIGN)
 void*
 librdf_sign_malloc(size_t size)
