@@ -2,7 +2,7 @@
  *
  * rdf_iterator.c - RDF Iterator Implementation
  *
- * $Id: rdf_iterator.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_iterator.c 12180 2007-05-01 03:42:32Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -66,6 +66,8 @@ librdf_new_iterator(librdf_world *world,
 {
   librdf_iterator* new_iterator;
   
+  librdf_world_open(world);
+
   new_iterator=(librdf_iterator*)LIBRDF_CALLOC(librdf_iterator, 1, 
                                                sizeof(librdf_iterator));
   if(!new_iterator)
@@ -415,6 +417,8 @@ librdf_new_empty_iterator(librdf_world *world)
 {
   librdf_iterator* new_iterator;
   
+  librdf_world_open(world);
+
   new_iterator=(librdf_iterator*)LIBRDF_CALLOC(librdf_iterator, 1, 
                                                sizeof(librdf_iterator));
   if(!new_iterator)
@@ -456,15 +460,10 @@ main(int argc, char *argv[])
   int i;
   librdf_iterator* iterator;
   int count;
-  
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
 	
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
-
-  librdf_init_hash(world);
-  librdf_init_uri(world);
-  librdf_init_node(world);
+  librdf_world_open(world);
 
   prefix_uri=librdf_new_uri(world, (const unsigned char*)NODE_URI_PREFIX);
   if(!prefix_uri) {
@@ -534,11 +533,7 @@ main(int argc, char *argv[])
 
   librdf_free_uri(prefix_uri);
   
-  librdf_finish_node(world);
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
 
   /* keep gcc -Wall happy */
   return(0);
