@@ -2,7 +2,7 @@
  *
  * rdf_model.c - RDF Graph (Model) interface
  *
- * $Id: rdf_model.c 8535 2006-01-26 05:02:23Z cmdjb $
+ * $Id: rdf_model.c 12183 2007-05-01 05:53:50Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -42,21 +42,18 @@
 #include <redland.h>
 
 #ifndef STANDALONE
-/* prototypes for helper functions */
-static void librdf_delete_model_factories(void);
-
 
 /**
  * librdf_init_model:
  * @world: redland world object
  *
- * INTERNAL - Initialise librdf_model class.
+ * INTERNAL - Initialise the model module.
  *
  **/
 void
 librdf_init_model(librdf_world *world)
 {
-  /* Always have model storage implementation available */
+  /* Always have model storage - must always be the default model */
   librdf_init_model_storage(world);
 }
 
@@ -65,36 +62,14 @@ librdf_init_model(librdf_world *world)
  * librdf_finish_model:
  * @world: redland world object
  *
- * INTERNAL - Terminate librdf_model class.
+ * INTERNAL - Terminate the model module.
  *
  **/
 void
 librdf_finish_model(librdf_world *world)
 {
-  librdf_delete_model_factories();
-}
-
-
-/* statics */
-
-/* list of storage factories */
-static librdf_model_factory* models;
-
-
-/*
- * librdf_delete_model_factories - helper function to delete all the registered model factories
- */
-static void
-librdf_delete_model_factories(void)
-{
-  librdf_model_factory *factory, *next;
-  
-  for(factory=models; factory; factory=next) {
-    next=factory->next;
-    LIBRDF_FREE(librdf_model_factory, factory->name);
-    LIBRDF_FREE(librdf_model_factory, factory);
-  }
-  models=NULL;
+   raptor_free_sequence(world->models);
+   world->models=NULL;
 }
 
 
@@ -110,25 +85,54 @@ librdf_model_supports_contexts(librdf_model* model) {
 
 /* class methods */
 
+static void
+librdf_free_model_factory(librdf_model_factory* factory)
+{
+  LIBRDF_FREE(librdf_model_factory, factory->name);
+  LIBRDF_FREE(librdf_model_factory, factory->label);
+  LIBRDF_FREE(librdf_model_factory, factory);
+}
+
+
 /**
  * librdf_model_register_factory:
  * @world: redland world object
  * @name: the model factory name
+ * @label: the storage factory label
  * @factory: pointer to function to call to register the factory
  *
  * Register a model factory.
  * 
  **/
 void
-librdf_model_register_factory(librdf_world *world, const char *name,
+librdf_model_register_factory(librdf_world *world, 
+                              const char *name, const char *label,
                               void (*factory) (librdf_model_factory*)) 
 {
   librdf_model_factory *model, *h;
   char *name_copy;
+  char *label_copy;
+  int i;
   
+  librdf_world_open(world);
+
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for model %s\n", name);
 #endif
+
+  if(!world->models)
+    world->models=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_model_factory, NULL);
+  
+  for(i=0;
+      (h=(librdf_model_factory*)raptor_sequence_get_at(world->models, i));
+      i++) {
+    if(!strcmp(h->name, name)) {
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_MODEL, NULL,
+                 "model %s already registered", h->name);
+      return;
+    }
+  }
   
   model=(librdf_model_factory*)LIBRDF_CALLOC(librdf_model_factory, 1,
                                              sizeof(librdf_model_factory));
@@ -143,30 +147,28 @@ librdf_model_register_factory(librdf_world *world, const char *name,
   strcpy(name_copy, name);
   model->name=name_copy;
         
-  for(h = models; h; h = h->next ) {
-    if(!strcmp(h->name, name_copy)) {
-      LIBRDF_FREE(cstring, name_copy);
-      librdf_log(world,
-                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_MODEL, NULL,
-                 "model %s already registered", h->name);
-      return;
-    }
+  label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
+  if(!label_copy) {
+    LIBRDF_FREE(librdf_model, model);
+    LIBRDF_FATAL1(world, LIBRDF_FROM_MODEL, "Out of memory");
   }
-  
+  strcpy(label_copy, label);
+  model->label=label_copy;
+
   /* Call the model registration function on the new object */
   (*factory)(model);
   
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG3("%s has context size %d\n", name, model->context_length);
 #endif
-  
-  model->next = models;
-  models = model;
+
+  raptor_sequence_push(world->models, model);
 }
 
 
 /**
  * librdf_get_model_factory:
+ * @world: redland world object
  * @name: the factory name or NULL for the default factory
  *
  * Get a model factory by name.
@@ -174,22 +176,27 @@ librdf_model_register_factory(librdf_world *world, const char *name,
  * Return value: the factory object or NULL if there is no such factory
  **/
 librdf_model_factory*
-librdf_get_model_factory (const char *name) 
+librdf_get_model_factory(librdf_world* world, const char *name) 
 {
   librdf_model_factory *factory;
 
+  librdf_world_open(world);
+
   /* return 1st model if no particular one wanted - why? */
   if(!name) {
-    factory=models;
+    factory=(librdf_model_factory *)raptor_sequence_get_at(world->models, 0);
     if(!factory) {
       LIBRDF_DEBUG1("No (default) models registered\n");
       return NULL;
     }
   } else {
-    for(factory=models; factory; factory=factory->next) {
-      if(!strcmp(factory->name, name)) {
+    int i;
+    
+    for(i=0;
+        (factory=(librdf_model_factory*)raptor_sequence_get_at(world->models, i));
+        i++) {
+      if(!strcmp(factory->name, name))
         break;
-      }
     }
     /* else FACTORY name not found */
     if(!factory) {
@@ -202,6 +209,37 @@ librdf_get_model_factory (const char *name)
 }
 
 
+/**
+ * librdf_model_enumerate:
+ * @world: redland world object
+ * @counter: index into the list of models
+ * @name: pointer to store the name of the model (or NULL)
+ * @label: pointer to store syntax readable label (or NULL)
+ *
+ * Get information on models.
+ * 
+ * Return value: non 0 on failure of if counter is out of range
+ **/
+int
+librdf_model_enumerate(librdf_world* world,
+                       const unsigned int counter,
+                       const char **name, const char **label)
+{
+  librdf_model_factory *factory;
+  
+  librdf_world_open(world);
+
+  factory=(librdf_model_factory*)raptor_sequence_get_at(world->models,
+                                                        counter);
+  if(!factory)
+    return 1;
+  
+  if(name)
+    *name=factory->name;
+  if(label)
+    *label=factory->label;
+  return 0;
+}
 
 
 /**
@@ -219,10 +257,12 @@ librdf_get_model_factory (const char *name)
  */
 librdf_model*
 librdf_new_model (librdf_world *world,
-                  librdf_storage *storage, char *options_string) {
+                  librdf_storage *storage, const char *options_string) {
   librdf_hash* options_hash;
   librdf_model *model;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
 
   if(!storage)
@@ -262,6 +302,8 @@ librdf_new_model_with_options(librdf_world *world,
   librdf_model *model;
   librdf_uri *uri;
   
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
 
   if(!storage)
@@ -273,7 +315,7 @@ librdf_new_model_with_options(librdf_world *world,
   
   model->world=world;
 
-  model->factory=librdf_get_model_factory("storage");
+  model->factory=librdf_get_model_factory(world, "storage");
   if(!model->factory) {
     LIBRDF_FREE(librdf_model, model);
     return NULL;
@@ -413,6 +455,9 @@ librdf_model_size(librdf_model* model)
  * shared with the model.  It must be a complete statement - all
  * of subject, predicate, object parts must be present.
  *
+ * Only statements that are legal RDF can be added: URI or blank subject,
+ * URI predicate and URI or blank or literal object (i.e. anything).
+ *
  * If the statement already exists in the model, it is not added.
  * Duplicate statements can be added when used with Redland Contexts
  * such as with #librdf_model_context_add_statement
@@ -443,6 +488,9 @@ librdf_model_add_statement(librdf_model* model, librdf_statement* statement)
  *
  * Add a stream of statements to the model.
  * 
+ * If any of the statements are illegal RDF statements they will
+ * be skipped and not added.  See #librdf_model_add_statement for the detail.
+ *
  * If any of the statements already exists in the store, they are not
  * added unless Redland contexts are being used.  See also
  * #librdf_model_context_add_statements
@@ -485,7 +533,14 @@ librdf_model_add(librdf_model* model, librdf_node* subject,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(predicate, librdf_node, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(object, librdf_node, 1);
 
-  if(!subject || !predicate || !object)
+  if(!subject ||
+     (!librdf_node_is_resource(subject) && !librdf_node_is_blank(subject)))
+    return 1;
+
+  if(!predicate || !librdf_node_is_resource(predicate))
+     return 1;
+
+  if(!object)
     return 1;
 
   statement=librdf_new_statement(model->world);
@@ -527,7 +582,7 @@ librdf_model_add_typed_literal_statement(librdf_model* model,
                                          librdf_node* subject, 
                                          librdf_node* predicate, 
                                          const unsigned char* literal,
-                                         char *xml_language,
+                                         const char *xml_language,
                                          librdf_uri *datatype_uri)
 {
   librdf_node* object;
@@ -537,7 +592,14 @@ librdf_model_add_typed_literal_statement(librdf_model* model,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(predicate, librdf_node, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(literal, string, 1);
 
-  if(!subject || !predicate || !literal)
+  if(!subject ||
+     (!librdf_node_is_resource(subject) && !librdf_node_is_blank(subject)))
+    return 1;
+
+  if(!predicate || !librdf_node_is_resource(predicate))
+     return 1;
+
+  if(!literal)
     return 1;
 
   object=librdf_new_node_from_typed_literal(model->world,
@@ -573,7 +635,7 @@ librdf_model_add_string_literal_statement(librdf_model* model,
 					  librdf_node* subject, 
 					  librdf_node* predicate, 
                                           const unsigned char* literal,
-					  char *xml_language,
+					  const char *xml_language,
                                           int is_wf_xml)
 {
   librdf_node* object;
@@ -584,7 +646,14 @@ librdf_model_add_string_literal_statement(librdf_model* model,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(predicate, librdf_node, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(literal, string, 1);
 
-  if(!subject || !predicate || !literal)
+  if(!subject ||
+     (!librdf_node_is_resource(subject) && !librdf_node_is_blank(subject)))
+    return 1;
+
+  if(!predicate || !librdf_node_is_resource(predicate))
+     return 1;
+
+  if(!literal)
     return 1;
 
   object=librdf_new_node_from_literal(model->world,
@@ -609,7 +678,7 @@ librdf_model_add_string_literal_statement(librdf_model* model,
  * Remove a known statement from the model.
  *
  * It must be a complete statement - all of subject, predicate, object
- * parts must be present.
+ * parts must be present and a legal RDF triple.
  *
  * Return value: non 0 on failure
  **/
@@ -633,16 +702,17 @@ librdf_model_remove_statement(librdf_model* model, librdf_statement* statement)
  *
  * Check for a statement in the model.
  * 
- * It must be a complete statement - all of subject, predicate, object
- * parts must be present.  Use librdf_model_find_statements to search
- * for partial statement matches.
+ * It must be a complete statement - all of subject, predicate,
+ * object parts must be present and a legal RDF triple.  Use
+ * librdf_model_find_statements to search for partial statement
+ * matches.
  *
  * WARNING: librdf_model_contains_statement may not work correctly
  * with stores using contexts.  In this case, a search using
  * librdf_model_find_statements for a non-empty list will
  * return the correct result.
  *
- * Return value: non 0 if the model contains the statement
+ * Return value: non 0 if the model contains the statement (>0 if the statement is illegal)
  **/
 int
 librdf_model_contains_statement(librdf_model* model, librdf_statement* statement)
@@ -928,7 +998,7 @@ librdf_model_add_submodel(librdf_model* model, librdf_model* sub_model)
     model->sub_models=l;
   }
   
-  if(!librdf_list_add(l, sub_model))
+  if(librdf_list_add(l, sub_model))
     return 1;
   
   return 0;
@@ -1166,7 +1236,7 @@ librdf_model_context_add_statements(librdf_model* model,
 /**
  * librdf_model_context_remove_statement:
  * @model: #librdf_model object
- * @context: #librdf_uri context
+ * @context: #librdf_node context
  * @statement: #librdf_statement statement
  *
  * Remove a statement from a model in a context.
@@ -1202,7 +1272,7 @@ librdf_model_context_remove_statement(librdf_model* model,
 /**
  * librdf_model_context_remove_statements:
  * @model: #librdf_model object
- * @context: #librdf_uri context
+ * @context: #librdf_node context
  *
  * Remove statements from a model with the given context.
  * 
@@ -1245,7 +1315,7 @@ librdf_model_context_remove_statements(librdf_model* model,
 /**
  * librdf_model_context_as_stream:
  * @model: #librdf_model object
- * @context: #librdf_uri context
+ * @context: #librdf_node context
  *
  * List all statements in a model context.
  * 
@@ -1270,7 +1340,7 @@ librdf_model_context_as_stream(librdf_model* model, librdf_node* context)
 /**
  * librdf_model_context_serialize:
  * @model: #librdf_model object
- * @context: #librdf_uri context
+ * @context: #librdf_node context
  *
  * List all statements in a model context.
  * 
@@ -1659,6 +1729,96 @@ librdf_model_contains_context(librdf_model* model, librdf_node* context) {
 }
 
 
+/**
+ * librdf_model_transaction_start:
+ * @model: the model object
+ * 
+ * Start a transaction
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+librdf_model_transaction_start(librdf_model* model) 
+{
+  if(model->factory->transaction_start)
+    return model->factory->transaction_start(model);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_model_transaction_start_with_handle:
+ * @model: the model object
+ * @handle: the transaction object
+ * 
+ * Start a transaction using an existing external transaction object.
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+librdf_model_transaction_start_with_handle(librdf_model* model, void* handle)
+{
+  if(model->factory->transaction_start_with_handle)
+    return model->factory->transaction_start_with_handle(model, handle);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_model_transaction_commit:
+ * @model: the model object
+ * 
+ * Commit a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+int
+librdf_model_transaction_commit(librdf_model* model) 
+{
+  if(model->factory->transaction_commit)
+    return model->factory->transaction_commit(model);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_model_transaction_rollback:
+ * @model: the model object
+ * 
+ * Rollback a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+int
+librdf_model_transaction_rollback(librdf_model* model) 
+{
+  if(model->factory->transaction_rollback)
+    return model->factory->transaction_rollback(model);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_model_transaction_get_handle:
+ * @model: the model object
+ * 
+ * Get the current transaction handle.
+ * 
+ * Return value: non-0 on failure 
+ **/
+void*
+librdf_model_transaction_get_handle(librdf_model* model) 
+{
+  if(model->factory->transaction_get_handle)
+    return model->factory->transaction_get_handle(model);
+  else
+    return NULL;
+}
+
 #endif
 
 
@@ -1709,17 +1869,18 @@ main(int argc, char *argv[])
   librdf_uri* uris[URI_STRING_COUNT];
   librdf_node* nodes[URI_STRING_COUNT];
   int i;
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   /* initialise dependent modules - all of them! */
   librdf_world *world=librdf_new_world();
-  librdf_iterator* iterator;
+  librdf_iterator *iterator, *second_iterator;
   librdf_node *n1, *n2;
   int count;
   int expected_count;
-#define EXPECTED_BAD_STRING_LENGTH 281
+#define EXPECTED_BAD_STRING_LENGTH 317
   librdf_uri* base_uri;
   unsigned char *string;
   size_t string_length=0;
+  int remove_count=0;
 
   librdf_world_open(world);
   
@@ -1729,7 +1890,11 @@ main(int argc, char *argv[])
     storage=librdf_new_storage(world, NULL, NULL, "contexts='yes'");
   } else {
     /* test on disk */
-    storage=librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
+    storage=librdf_new_storage(world, "sqlite", "test", "new='yes',contexts='yes'");
+    if(!storage)
+      storage=librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
+    if(!storage)
+      storage=librdf_new_storage(world, NULL, NULL, "contexts='yes'");
   }
   if(!storage) {
     fprintf(stderr, "%s: Failed to create new storage\n", program);
@@ -1757,6 +1922,18 @@ main(int argc, char *argv[])
   librdf_model_add_statement(model, statement);
   librdf_free_statement(statement);
 
+  /* make it illegal */
+  statement=librdf_new_statement(world);
+  librdf_statement_set_subject(statement, librdf_new_node_from_literal(world, (const unsigned char*)"Bad Subject", NULL, 0));
+  librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/pred"));
+  librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)"Good Object", NULL, 0));
+
+  if(!librdf_model_add_statement(model, statement)) {
+    fprintf(stderr, "%s: librdf_model_add_statement unexpectedly succeeded adding an illegal triple\n", program);
+    return(1);
+  }
+  librdf_free_statement(statement);
+
   fprintf(stderr, "%s: Printing model\n", program);
   librdf_model_print(model, stderr);
   
@@ -1946,6 +2123,94 @@ main(int argc, char *argv[])
   free(string);
   fprintf(stderr, "%s: Serialized OK\n", program);
 
+
+#define TEST_SIMILAR_COUNT 9
+
+  /* add some similar statements */
+  fprintf(stderr, "%s: Adding %d similar statements\n", program, TEST_SIMILAR_COUNT);
+  for(i=0; i < TEST_SIMILAR_COUNT; i++) {
+    char literal[6];
+    strncpy(literal, "DaveX", 6);
+    literal[4]='0'+i;
+    statement=librdf_new_statement(world);
+    librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
+    librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
+    librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
+    
+    librdf_model_add_statement(model, statement);
+    librdf_free_statement(statement);
+  }
+
+  /* targets */
+  fprintf(stderr, "%s: iterating similar statements\n", program);
+  n1=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/");
+  n2=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator");
+  iterator=librdf_model_get_targets(model, n1, n2);
+  if(!iterator) {
+    fprintf(stderr, "%s: librdf_model_get_targets failed\n", program);
+    exit(1);
+  }
+
+  fprintf(stderr, "%s: iterating similar statements again\n", program);
+  second_iterator=librdf_model_get_targets(model, n1, n2);
+
+  /* Deleting 2 statements but only 1 before counting */
+  expected_count= TEST_SIMILAR_COUNT - 1;
+  for(count=0;
+      !librdf_iterator_end(iterator); 
+      librdf_iterator_next(iterator), librdf_iterator_next(second_iterator), count++) {
+    librdf_node* n;
+    char literal[6];
+
+    if(count == 2 ) {
+      strncpy(literal, "DaveX", 6);
+      literal[4]='0'+count;
+      statement=librdf_new_statement(world);
+      librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
+      librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
+      librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
+
+      fprintf(stderr, "%s: Removing statement with literal '%s'\n",
+              program, literal);
+      
+      librdf_model_remove_statement(model, statement);
+      librdf_free_statement(statement);
+      remove_count++;
+    }
+  
+    n=(librdf_node*)librdf_iterator_get_object(iterator);
+    fputs("  ", stderr);
+    librdf_node_print(n, stderr);
+    fputs("\n", stderr);
+
+    if(count == 5 ) {
+      strncpy(literal, "DaveX", 6);
+      literal[4]='0'+count+remove_count;
+      statement=librdf_new_statement(world);
+      librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
+      librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
+      librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
+
+      fprintf(stderr, "%s: Removing statement with literal '%s'\n",
+              program, literal);
+      
+      librdf_model_remove_statement(model, statement);
+      librdf_free_statement(statement);
+      remove_count++;
+    }
+  
+  }
+  librdf_free_iterator(iterator);
+  if(count != expected_count) {
+    fprintf(stderr, "%s: librdf_model_get_targets returned %d nodes, expected %d\n", program, count, expected_count);
+    exit(1);
+  }
+  librdf_free_node(n1);
+  librdf_free_node(n2);
+  
+
+  librdf_free_iterator(second_iterator);
+
   fprintf(stderr, "%s: Freeing URIs and Nodes\n", program);
   for (i=0; i<URI_STRING_COUNT; i++) {
     librdf_free_uri(uris[i]);
