@@ -2,7 +2,7 @@
  *
  * rdf_model_storage.c - RDF Model Storage implementation
  *
- * $Id: rdf_model_storage.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_model_storage.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -633,6 +633,90 @@ librdf_model_storage_find_statements_with_options(librdf_model* model,
   return librdf_storage_find_statements_with_options(context->storage, statement, context_node, options);
 }
 
+
+/**
+ * librdf_model_storage_transaction_start:
+ * @storage: the storage object
+ * 
+ * Start a transaction
+ * 
+ * Return value: non-0 on failure
+ **/
+static int
+librdf_model_storage_transaction_start(librdf_model* model) 
+{
+  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
+  return librdf_storage_transaction_start(context->storage);
+}
+
+
+/**
+ * librdf_model_storage_transaction_start_with_handle:
+ * @storage: the storage object
+ * @handle: the transaction object
+ * 
+ * Start a transaction using an existing external transaction object.
+ * 
+ * Return value: non-0 on failure
+ **/
+static int
+librdf_model_storage_transaction_start_with_handle(librdf_model* model,
+                                                   void* handle)
+{
+  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
+  return librdf_storage_transaction_start_with_handle(context->storage, handle);
+}
+
+
+/**
+ * librdf_model_storage_transaction_commit:
+ * @storage: the storage object
+ * 
+ * Commit a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static int
+librdf_model_storage_transaction_commit(librdf_model* model) 
+{
+  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
+  return librdf_storage_transaction_commit(context->storage);
+}
+
+
+/**
+ * librdf_model_storage_transaction_rollback:
+ * @storage: the storage object
+ * 
+ * Rollback a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static int
+librdf_model_storage_transaction_rollback(librdf_model* model) 
+{
+  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
+  return librdf_storage_transaction_rollback(context->storage);
+}
+
+
+/**
+ * librdf_model_storage_transaction_get_handle:
+ * @storage: the storage object
+ * 
+ * Get the current transaction handle.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static void*
+librdf_model_storage_transaction_get_handle(librdf_model* model) 
+{
+  librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
+  return librdf_storage_transaction_get_handle(context->storage);
+}
+
+
+
 /* local function to register model_storage functions */
 
 static void
@@ -676,12 +760,26 @@ librdf_model_storage_register_factory(librdf_model_factory *factory)
   factory->get_feature        = librdf_model_storage_get_feature;
   factory->set_feature        = librdf_model_storage_set_feature;
   factory->find_statements_with_options = librdf_model_storage_find_statements_with_options;
+
+  factory->transaction_start             = librdf_model_storage_transaction_start;
+  factory->transaction_start_with_handle = librdf_model_storage_transaction_start_with_handle;
+  factory->transaction_commit            = librdf_model_storage_transaction_commit;
+  factory->transaction_rollback          = librdf_model_storage_transaction_rollback;
+  factory->transaction_get_handle        = librdf_model_storage_transaction_get_handle;
+
 }
 
 
+/**
+ * librdf_init_model_storage:
+ * @world: world object
+ * 
+ * INTERNAL - Initialise the model_storage module
+ **/
 void
 librdf_init_model_storage(librdf_world *world)
 {
-  librdf_model_register_factory(world, "storage", 
+  librdf_model_register_factory(world, 
+                                "storage", "Model backed by a Redland storage",
                                 &librdf_model_storage_register_factory);
 }
