@@ -2,7 +2,7 @@
  *
  * rdf_node.c - RDF Node (RDF URI, Literal, Blank Node) Interface
  *
- * $Id: rdf_node.c 10927 2006-05-02 20:16:37Z dajobe $
+ * $Id: rdf_node.c 12178 2007-05-01 03:32:14Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -86,7 +86,7 @@ librdf_init_node(librdf_world* world)
  * librdf_finish_node:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_node module.
+ * INTERNAL - Terminate the node module.
  *
  **/
 void
@@ -121,6 +121,8 @@ librdf_finish_node(librdf_world *world)
 librdf_node*
 librdf_new_node(librdf_world *world)
 {
+  librdf_world_open(world);
+
   return librdf_new_node_from_blank_identifier(world, (unsigned char*)NULL);
 }
 
@@ -146,6 +148,8 @@ librdf_new_node_from_uri_string_or_uri(librdf_world *world,
   librdf_hash_datum key, value; /* on stack - not allocated */
   librdf_hash_datum *old_value;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_RETURN((uri_string == NULL && uri == NULL), 
                        "both uri_string and uri are NULL", NULL);
 
@@ -240,6 +244,8 @@ librdf_node*
 librdf_new_node_from_uri_string(librdf_world *world, 
                                 const unsigned char *uri_string) 
 {
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri_string, string, NULL);
 
   return librdf_new_node_from_uri_string_or_uri(world, uri_string, NULL);
@@ -261,6 +267,8 @@ librdf_new_node_from_uri_string(librdf_world *world,
 librdf_node*
 librdf_new_node_from_uri(librdf_world *world, librdf_uri *uri) 
 {
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
 
   return librdf_new_node_from_uri_string_or_uri(world, NULL, uri);
@@ -285,6 +293,8 @@ librdf_new_node_from_uri_local_name(librdf_world *world,
   librdf_uri *new_uri;
   librdf_node* new_node;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(local_name, string, NULL);
 
@@ -320,6 +330,8 @@ librdf_new_node_from_normalised_uri_string(librdf_world *world,
   librdf_uri* new_uri;
   librdf_node* new_node;
   
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri_string, string, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(source_uri, librdf_uri, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, NULL);
@@ -358,27 +370,40 @@ librdf_new_node_from_literal(librdf_world *world,
                              const char *xml_language, 
                              int is_wf_xml) 
 {
+  size_t xml_language_len=0;
+  
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, NULL);
 
   if(xml_language && !*xml_language)
     xml_language=NULL;
 
-  if(xml_language && is_wf_xml)
-    return NULL;
-  
-  return librdf_new_node_from_typed_literal(world,
-                                            string, xml_language,
-                                            (is_wf_xml ? 
-                                             LIBRDF_RS_XMLLiteral_URI : NULL)
-                                            );
+  if(xml_language) {
+    if(is_wf_xml)
+      return NULL;
+    xml_language_len=strlen(xml_language);
+  }
+    
+
+  return librdf_new_node_from_typed_counted_literal(world,
+                                                    string, 
+                                                    strlen((const char*)string),
+                                                    xml_language,
+                                                    xml_language_len,
+                                                    (is_wf_xml ? 
+                                                     LIBRDF_RS_XMLLiteral_URI : NULL)
+                                                    );
 }
 
 
 /**
- * librdf_new_node_from_typed_literal:
+ * librdf_new_node_from_typed_counted_literal:
  * @world: redland world object
  * @value: literal string value
+ * @value_len: literal string value length
  * @xml_language: literal XML language (or NULL, empty string)
+ * @xml_language_len: literal XML language length (not used if @xml_language is NULL)
  * @datatype_uri: URI of typed literal datatype or NULL
  *
  * Constructor - create a new typed literal #librdf_node object.
@@ -390,13 +415,14 @@ librdf_new_node_from_literal(librdf_world *world,
  * Return value: new #librdf_node object or NULL on failure
  **/
 librdf_node*
-librdf_new_node_from_typed_literal(librdf_world *world, 
-                                   const unsigned char *value,
-                                   const char *xml_language, 
-                                   librdf_uri* datatype_uri) 
+librdf_new_node_from_typed_counted_literal(librdf_world *world, 
+                                           const unsigned char *value,
+                                           size_t value_len,
+                                           const char *xml_language, 
+                                           size_t xml_language_len,
+                                           librdf_uri* datatype_uri) 
 {
   librdf_node* new_node;
-  int value_len;
   unsigned char *new_value;
   char *new_xml_language;
   librdf_hash_datum key, value_hd; /* on stack - not allocated */
@@ -404,6 +430,8 @@ librdf_new_node_from_typed_literal(librdf_world *world,
   size_t size;
   unsigned char *buffer;
   
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(value, string, NULL);
 
   if(xml_language && !*xml_language)
@@ -426,7 +454,7 @@ librdf_new_node_from_typed_literal(librdf_world *world,
   new_node->type=LIBRDF_NODE_TYPE_LITERAL;
 
   /* the only time the string literal length should ever be measured */
-  new_node->value.literal.string_len = value_len = strlen((const char*)value);
+  new_node->value.literal.string_len = value_len;
   
   new_value=(unsigned char*)LIBRDF_MALLOC(cstring, value_len + 1);
   if(!new_value) {
@@ -438,7 +466,7 @@ librdf_new_node_from_typed_literal(librdf_world *world,
   new_node->value.literal.string=new_value;
   
   if(xml_language && *xml_language) {
-    new_xml_language=(char*)LIBRDF_MALLOC(cstring, strlen(xml_language) + 1);
+    new_xml_language=(char*)LIBRDF_MALLOC(cstring, xml_language_len + 1);
     if(!new_xml_language) {
       LIBRDF_FREE(cstring, new_value);
       LIBRDF_FREE(librdf_node, new_node);
@@ -447,6 +475,7 @@ librdf_new_node_from_typed_literal(librdf_world *world,
     }
     strcpy(new_xml_language, xml_language);
     new_node->value.literal.xml_language=new_xml_language;
+    new_node->value.literal.xml_language_len=xml_language_len;
   } else
     new_xml_language=NULL;
   
@@ -527,12 +556,47 @@ librdf_new_node_from_typed_literal(librdf_world *world,
 }
 
 
+/**
+ * librdf_new_node_from_typed_literal:
+ * @world: redland world object
+ * @value: literal string value
+ * @xml_language: literal XML language (or NULL, empty string)
+ * @datatype_uri: URI of typed literal datatype or NULL
+ *
+ * Constructor - create a new typed literal #librdf_node object.
+ * 
+ * Only one of @xml_language or @datatype_uri may be given.  If both
+ * are given, NULL is returned.  If @xml_language is the empty string,
+ * it is the equivalent to NULL.
+ *
+ * Return value: new #librdf_node object or NULL on failure
+ **/
+librdf_node*
+librdf_new_node_from_typed_literal(librdf_world *world, 
+                                   const unsigned char *value,
+                                   const char *xml_language, 
+                                   librdf_uri* datatype_uri) 
+{
+  size_t xml_language_len=0;
+
+  librdf_world_open(world);
+
+  if(xml_language)
+    xml_language_len=strlen(xml_language);
+  
+  return librdf_new_node_from_typed_counted_literal(world, value,
+                                                    strlen((const char*)value),
+                                                    xml_language,
+                                                    xml_language_len,
+                                                    datatype_uri);
+}
+
 /**
  * librdf_new_node_from_blank_identifier:
  * @world: redland world object
  * @identifier: blank node identifier or NULL
  *
- * Constructor - create a new literal #librdf_node object from a blank node identifier.
+ * Constructor - create a new blank node #librdf_node object from a blank node identifier.
  *
  * If no identifier string is given, creates a new internal identifier
  * and assigns it.
@@ -549,6 +613,8 @@ librdf_new_node_from_blank_identifier(librdf_world *world,
   librdf_hash_datum key, value; /* on stack - not allocated */
   librdf_hash_datum *old_value;
 
+  librdf_world_open(world);
+
 #ifdef WITH_THREADS
   pthread_mutex_lock(world->nodes_mutex);
 #endif
@@ -1095,7 +1161,7 @@ librdf_node_to_counted_string(librdf_node* node, size_t* len_p)
   case LIBRDF_NODE_TYPE_LITERAL:
     len=node->value.literal.string_len;
     if(node->value.literal.xml_language) {
-      language_len=strlen(node->value.literal.xml_language);
+      language_len=node->value.literal.xml_language_len;
       len+=1+language_len;
     }
     
@@ -1297,7 +1363,7 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
           librdf_log(node->world,
                      0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
                      "Cannot encode a URI string of %d bytes length",
-                     string_length);
+                     (int)string_length);
         return 0;
       }
 
@@ -1313,12 +1379,15 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
       string=(unsigned char*)node->value.literal.string;
       string_length=node->value.literal.string_len;
       if(node->value.literal.xml_language)
-        language_length=strlen(node->value.literal.xml_language);
+        language_length=node->value.literal.xml_language_len;
       if(node->value.literal.datatype_uri) {
         datatype_uri_string=librdf_uri_as_counted_string(node->value.literal.datatype_uri, &datatype_uri_length);
       }
       
       total_length= 6 + string_length + 1; /* +1 for \0 at end */
+      if(string_length > 0xFFFF) /* for long literal - type 'N' */
+        total_length+= 2;
+
       if(language_length)
         total_length += language_length+1;
       if(datatype_uri_length)
@@ -1327,19 +1396,11 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
       if(length && total_length > length)
         return 0;    
 
-      if(string_length > 0xFFFF) {
-        librdf_log(node->world,
-                   0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
-                   "Cannot encode a literal string of %d bytes length",
-                   string_length);
-        return 0;
-      }
-
       if(datatype_uri_length > 0xFFFF) {
         librdf_log(node->world,
                    0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
                    "Cannot encode a datatype URI string of %d bytes length",
-                   datatype_uri_length);
+                   (int)datatype_uri_length);
         return 0;
       }
 
@@ -1347,19 +1408,33 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
         librdf_log(node->world,
                    0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
                    "Cannot encode a language string of %d bytes length",
-                   language_length);
+                   (int)language_length);
         return 0;
       }
 
-      
+
       if(buffer) {
-        buffer[0]='M';
-        buffer[1]=(string_length & 0xff00) >> 8;
-        buffer[2]=(string_length & 0x00ff);
-        buffer[3]=(datatype_uri_length & 0xff00) >> 8;
-        buffer[4]=(datatype_uri_length & 0x00ff);
-        buffer[5]=(language_length & 0x00ff);
-        buffer += 6;
+        if(string_length > 0xFFFF) {
+          /* long literal type N (string length > 0x10000) */
+          buffer[0]='N';
+          buffer[1]=(string_length & 0xff000000) >> 24;
+          buffer[2]=(string_length & 0x00ff0000) >> 16;
+          buffer[3]=(string_length & 0x0000ff00) >> 8;
+          buffer[4]=(string_length & 0x000000ff);
+          buffer[5]=(datatype_uri_length & 0xff00) >> 8;
+          buffer[6]=(datatype_uri_length & 0x00ff);
+          buffer[7]=(language_length & 0x00ff);
+          buffer += 8;
+        } else {
+          /* short literal type M (string length <= 0xFFFF) */
+          buffer[0]='M';
+          buffer[1]=(string_length & 0xff00) >> 8;
+          buffer[2]=(string_length & 0x00ff);
+          buffer[3]=(datatype_uri_length & 0xff00) >> 8;
+          buffer[4]=(datatype_uri_length & 0x00ff);
+          buffer[5]=(language_length & 0x00ff);
+          buffer += 6;
+        }
         strcpy((char*)buffer, (const char*)string);
         buffer += string_length+1;
         if(datatype_uri_length) {
@@ -1368,7 +1443,8 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
         }
         if(language_length)
           strcpy((char*)buffer, (const char*)node->value.literal.xml_language);
-      }
+      } /* end if buffer */
+
       break;
       
     case LIBRDF_NODE_TYPE_BLANK:
@@ -1384,7 +1460,7 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
         librdf_log(node->world,
                    0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
                    "Cannot encode a blank node identifier string of %d bytes length",
-                   string_length);
+                   (int)string_length);
         return 0;
       }
 
@@ -1437,6 +1513,8 @@ librdf_node_decode(librdf_world *world,
   int status=0;
   librdf_node* node=NULL;
 
+  librdf_world_open(world);
+
   /* absolute minimum - first byte is type */
   if (length < 1)
     return NULL;
@@ -1470,10 +1548,12 @@ librdf_node_decode(librdf_world *world,
         total_length += language_length+1;
       }
       
-      node=librdf_new_node_from_typed_literal(world,
-                                              buffer+6,
-                                              (const char*)language,
-                                              is_wf_xml ? LIBRDF_RS_XMLLiteral_URI : NULL);
+      node=librdf_new_node_from_typed_counted_literal(world,
+                                                      buffer+6,
+                                                      string_length,
+                                                      (const char*)language,
+                                                      language_length,
+                                                      is_wf_xml ? LIBRDF_RS_XMLLiteral_URI : NULL);
     
     break;
 
@@ -1499,10 +1579,48 @@ librdf_node_decode(librdf_world *world,
       if(datatype_uri_string)
         datatype_uri=librdf_new_uri(world, datatype_uri_string);
       
-      node=librdf_new_node_from_typed_literal(world,
-                                              buffer+6,
-                                              (const char*)language,
-                                              datatype_uri);
+      node=librdf_new_node_from_typed_counted_literal(world,
+                                                      buffer+6,
+                                                      string_length,
+                                                      (const char*)language,
+                                                      language_length,
+                                                      datatype_uri);
+      if(datatype_uri)
+        librdf_free_uri(datatype_uri);
+      
+      if(status)
+        return NULL;
+      
+    break;
+
+    case 'N': /* LIBRDF_NODE_TYPE_LITERAL - redland 1.0.5+ (long literal) */
+      /* min */
+      if(length < 8)
+        return NULL;
+      
+      string_length=(buffer[1] << 24) | (buffer[2] << 16) | (buffer[3] << 8) | buffer[4];
+      datatype_uri_length=(buffer[5] << 8) | buffer[6];
+      language_length=buffer[7];
+
+      total_length= 8 + string_length + 1; /* +1 for \0 at end */
+      if(datatype_uri_length) {
+        datatype_uri_string = buffer + total_length;
+        total_length += datatype_uri_length+1;
+      }
+      if(language_length) {
+        language = buffer + total_length;
+        total_length += language_length+1;
+      }
+
+      if(datatype_uri_string)
+        datatype_uri=librdf_new_uri(world, datatype_uri_string);
+      
+      node=librdf_new_node_from_typed_counted_literal(world,
+                                                      buffer+8,
+                                                      string_length,
+                                                      (const char*)language,
+                                                      language_length,
+                                                      datatype_uri);
       if(datatype_uri)
         librdf_free_uri(datatype_uri);
       
@@ -1660,28 +1778,73 @@ librdf_node_static_iterator_create(librdf_node** nodes,
 int main(int argc, char *argv[]);
 
 
+static void
+dump_node_as_C(FILE* fh, const char* var, void *buffer, int size) {
+  int i;
+  unsigned char* p=(unsigned char*)buffer;
+  
+  fprintf(fh, "const unsigned char %s[%d] = {", var, size);
+  for (i=0; i < size; i++) {
+    if(i)
+      fputs(", ", fh);
+    fprintf(fh, "0x%02x", p[i]);
+  }
+  fputs("};\n", fh);
+}
+
+  
+static int
+check_node(const char* program, const unsigned char *expected, 
+           void *buffer, size_t size) {
+  unsigned int i;
+  for(i=0; i< size; i++) {
+    unsigned char c=((unsigned char*)buffer)[i];
+    if(c != expected[i]) {
+      fprintf(stderr, "%s: Encoding node byte %d: 0x%02x expected 0x%02x\n",
+              program, i, c, expected[i]);
+      return(1);
+    }
+  }
+  return(0);
+}
+
+
+static const char *hp_string1="http://purl.org/net/dajobe/";
+static const char *hp_string2="http://purl.org/net/dajobe/";
+static const char *lit_string="Dave Beckett";
+static const char *genid="genid42";
+static const char *datatype_lit_string="Datatyped literal value";
+static const char *datatype_uri_string="http://example.org/datatypeURI";
+
+/* Node Encoded (type R) version of hp_string1 */
+static const unsigned char hp_uri_encoded[31] = {0x52, 0x00, 0x1b, 0x68, 0x74, 0x74, 0x70, 0x3a, 0x2f, 0x2f, 0x70, 0x75, 0x72, 0x6c, 0x2e, 0x6f, 0x72, 0x67, 0x2f, 0x6e, 0x65, 0x74, 0x2f, 0x64, 0x61, 0x6a, 0x6f, 0x62, 0x65, 0x2f, 0x00};
+
+/* Node Encoded (type M) version of typed literal with literal value
+ * datatype_lit_string and datatype URI datatype_uri_string */
+static const unsigned char datatyped_literal_M_encoded[61] = {0x4d, 0x00, 0x17, 0x00, 0x1e, 0x00, 0x44, 0x61, 0x74, 0x61, 0x74, 0x79, 0x70, 0x65, 0x64, 0x20, 0x6c, 0x69, 0x74, 0x65, 0x72, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x6c, 0x75, 0x65, 0x00, 0x68, 0x74, 0x74, 0x70, 0x3a, 0x2f, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2e, 0x6f, 0x72, 0x67, 0x2f, 0x64, 0x61, 0x74, 0x61, 0x74, 0x79, 0x70, 0x65, 0x55, 0x52, 0x49, 0x00};
+
+/* Node Encoded (type N) version of big 100,000-length literal
+ * (just the first 32 bytes, the rest are 0x58 'X')
+ */
+const unsigned char big_literal_N_encoded[32] = {0x4e, 0x00, 0x01, 0x86, 0xa0, 0x00, 0x00, 0x00, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58, 0x58};
+
+
 int
 main(int argc, char *argv[]) 
 {
-  librdf_node *node, *node2, *node3, *node4, *node5, *node6, *node7;
-  char *hp_string1="http://purl.org/net/dajobe/";
-  char *hp_string2="http://purl.org/net/dajobe/";
-  char *lit_string="Dave Beckett";
-  char *genid="genid42";
+  librdf_node *node, *node2, *node3, *node4, *node5, *node6, *node7, *node8;
   librdf_uri *uri, *uri2;
   int size, size2;
   unsigned char *buffer;
   librdf_world *world;
+  size_t big_literal_length;
+  unsigned char *big_literal;
+  unsigned int i;
   
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
 	
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
-
-  librdf_init_digest(world);
-  librdf_init_hash(world);
-  librdf_init_uri(world);
-  librdf_init_node(world);
+  librdf_world_open(world);
 
   fprintf(stdout, "%s: Creating home page node from string\n", program);
   node=librdf_new_node_from_uri_string(world, (const unsigned char*)hp_string1);
@@ -1716,6 +1879,11 @@ main(int argc, char *argv[])
     fprintf(stderr, "%s: Encoding node used %d bytes, expected it to use %d\n", program, size2, size);
     return(1);
   }
+
+  if(0)
+    dump_node_as_C(stdout, "hp_uri_encoded", buffer, size);
+  if(check_node(program, hp_uri_encoded, buffer, size))
+    return(1);
   
     
   fprintf(stdout, "%s: Creating new node\n", program);
@@ -1777,9 +1945,9 @@ main(int argc, char *argv[])
   fprintf(stdout, "%s: Copied node identifier is: '%s'\n", program, buffer);
 
 
-  uri2=librdf_new_uri(world, (const unsigned char*)"http://example.org/datatypeURI");
+  uri2=librdf_new_uri(world, (const unsigned char*)datatype_uri_string);
   node6=librdf_new_node_from_typed_literal(world, 
-                                           (const unsigned char*)"Datatyped literal value",
+                                           (const unsigned char*)datatype_lit_string,
                                            NULL, uri2);
   librdf_free_uri(uri2);
 
@@ -1793,6 +1961,11 @@ main(int argc, char *argv[])
     fprintf(stderr, "%s: Encoding typed node used %d bytes, expected it to use %d\n", program, size2, size);
     return(1);
   }
+
+  if(0)
+    dump_node_as_C(stdout, "datatyped_literal_M_encoded", buffer, size);
+  if(check_node(program, datatyped_literal_M_encoded, buffer, size))
+    return(1);
   
   fprintf(stdout, "%s: Decoding typed node from buffer\n", program);
   if(!(node7=librdf_node_decode(world, NULL, (unsigned char*)buffer, size))) {
@@ -1815,9 +1988,41 @@ main(int argc, char *argv[])
     return(1);
   }
     
+  big_literal_length=100000;
+  big_literal=(unsigned char *)LIBRDF_MALLOC(cstring, big_literal_length+1);
+  for(i=0; i<big_literal_length; i++)
+     big_literal[i]='X';
 
+  node8=librdf_new_node_from_typed_counted_literal(world, 
+                                                   big_literal, big_literal_length,
+                                                   NULL, 0, NULL);
+  if(!node8) {
+    fprintf(stderr, "%s: Failed to make big %d byte literal\n", program,
+            (int)big_literal_length);
+    return(1);
+  }
+  LIBRDF_FREE(cstring, big_literal);
+
+  size=librdf_node_encode(node8, NULL, 0);
+  fprintf(stdout, "%s: Encoding big literal node requires %d bytes\n", program, size);
+  buffer=(unsigned char*)LIBRDF_MALLOC(cstring, size);
+  fprintf(stdout, "%s: Encoding big literal node in buffer\n", program);
+  size2=librdf_node_encode(node8, (unsigned char*)buffer, size);
+  if(size2 != size) {
+    fprintf(stderr, "%s: Encoding big literal node used %d bytes, expected it to use %d\n", program, size2, size);
+    return(1);
+  }
+
+  /* Just check first 32 bytes */
+  if(0)
+    dump_node_as_C(stdout, "big_literal_N_encoded", buffer, 32);
+  if(check_node(program, big_literal_N_encoded, buffer, 32))
+    return(1);
+  LIBRDF_FREE(cstring, buffer);
+    
 
   fprintf(stdout, "%s: Freeing nodes\n", program);
+  librdf_free_node(node8);
   librdf_free_node(node7);
   librdf_free_node(node6);
   librdf_free_node(node5);
@@ -1826,12 +2031,7 @@ main(int argc, char *argv[])
   librdf_free_node(node2);
   librdf_free_node(node);
 
-  librdf_finish_node(world);
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-  librdf_finish_digest(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
 
   /* keep gcc -Wall happy */
   return(0);
