@@ -2,9 +2,9 @@
  *
  * rdf_parser.c - RDF Parser (syntax to RDF triples) interface
  *
- * $Id: rdf_parser.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_parser.c 12204 2007-05-05 06:31:29Z dajobe $
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -44,8 +44,35 @@
 
 #ifndef STANDALONE
 
-/* prototypes for helper functions */
-static void librdf_delete_parser_factories(librdf_world *world);
+/**
+ * librdf_init_parser:
+ * @world: redland world object
+ *
+ * INTERNAL - Initialise the parser module.
+ *
+ **/
+void
+librdf_init_parser(librdf_world *world) 
+{
+  librdf_parser_raptor_constructor(world);
+}
+
+
+/**
+ * librdf_finish_parser:
+ * @world: redland world object
+ *
+ * INTERNAL - Terminate the parser module.
+ *
+ **/
+void
+librdf_finish_parser(librdf_world *world) 
+{
+  raptor_free_sequence(world->parsers);
+  world->parsers=NULL;
+
+  librdf_parser_raptor_destructor();
+}
 
 
 /* helper functions */
@@ -53,32 +80,22 @@ static void
 librdf_free_parser_factory(librdf_parser_factory *factory) 
 {
   if(factory->name)
-    LIBRDF_FREE(librdf_parser_factory, factory->name);
+    LIBRDF_FREE(cstring, factory->name);
+  if(factory->label)
+    LIBRDF_FREE(cstring, factory->label);
   if(factory->mime_type)
-    LIBRDF_FREE(librdf_parser_factory, factory->mime_type);
+    LIBRDF_FREE(cstring, factory->mime_type);
   if(factory->type_uri)
     librdf_free_uri(factory->type_uri);
   LIBRDF_FREE(librdf_parser_factory, factory);
 }
 
 
-static void
-librdf_delete_parser_factories(librdf_world *world)
-{
-  librdf_parser_factory *factory, *next;
-  
-  for(factory=world->parsers; factory; factory=next) {
-    next=factory->next;
-    librdf_free_parser_factory(factory);
-  }
-  world->parsers=NULL;
-}
-
-
 /**
  * librdf_parser_register_factory:
  * @world: redland world object
  * @name: the name of the parser
+ * @label: the label of the parser (optional)
  * @mime_type: MIME type of the syntax (optional)
  * @uri_string: URI of the syntax (optional)
  * @factory: function to be called to register the factor parameters
@@ -88,40 +105,57 @@ librdf_delete_parser_factories(librdf_world *world)
  **/
 void
 librdf_parser_register_factory(librdf_world *world,
-                               const char *name, const char *mime_type,
+                               const char *name, const char *label, 
+                               const char *mime_type,
                                const unsigned char *uri_string,
 			       void (*factory) (librdf_parser_factory*))
 {
-  librdf_parser_factory *parser_factory;
+  librdf_parser_factory *parser;
   char *name_copy;
+  char *label_copy;
+
+  librdf_world_open(world);
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for parser %s\n", name);
 #endif
   
-  parser_factory=(librdf_parser_factory*)LIBRDF_CALLOC(librdf_parser_factory, 1,
+  if(!world->parsers)
+    world->parsers=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_parser_factory, NULL);
+
+  parser=(librdf_parser_factory*)LIBRDF_CALLOC(librdf_parser_factory, 1,
 					       sizeof(librdf_parser_factory));
-  if(!parser_factory)
+  if(!parser)
     LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
   
   name_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(name)+1);
   if(!name_copy) {
-    librdf_free_parser_factory(parser_factory);
+    librdf_free_parser_factory(parser);
     LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
   }
   strcpy(name_copy, name);
-  parser_factory->name=name_copy;
+  parser->name=name_copy;
 
+  if(label) {
+    label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
+    if(!label_copy) {
+      librdf_free_parser_factory(parser);
+      LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
+    }
+    strcpy(label_copy, label);
+    parser->label=label_copy;
+  }
+        
   /* register mime type if any */
   if(mime_type) {
     char *mime_type_copy;
     mime_type_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(mime_type)+1);
     if(!mime_type_copy) {
-      librdf_free_parser_factory(parser_factory);
+      librdf_free_parser_factory(parser);
       LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
     }
     strcpy(mime_type_copy, mime_type);
-    parser_factory->mime_type=mime_type_copy;
+    parser->mime_type=mime_type_copy;
   }
 
   /* register URI if any */
@@ -130,24 +164,22 @@ librdf_parser_register_factory(librdf_world *world,
 
     uri=librdf_new_uri(world, uri_string);
     if(!uri) {
-      librdf_free_parser_factory(parser_factory);
+      librdf_free_parser_factory(parser);
       LIBRDF_FATAL1(world, LIBRDF_FROM_PARSER, "Out of memory");
     }
-    parser_factory->type_uri=uri;
+    parser->type_uri=uri;
   }
   
         
   /* Call the parser registration function on the new object */
-  (*factory)(parser_factory);
+  (*factory)(parser);
 
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  LIBRDF_DEBUG3("%s has context size %d\n", name, parser_factory->context_length);
+  LIBRDF_DEBUG3("%s has context size %d\n", name, parser->context_length);
 #endif
   
-  parser_factory->next = world->parsers;
-  world->parsers = parser_factory;
-  
+  raptor_sequence_push(world->parsers, parser);
 }
 
 
@@ -172,6 +204,8 @@ librdf_get_parser_factory(librdf_world *world,
 {
   librdf_parser_factory *factory;
   
+  librdf_world_open(world);
+
   if(name && !*name)
     name=NULL;
   if(!mime_type || (mime_type && !*mime_type)) {
@@ -183,13 +217,17 @@ librdf_get_parser_factory(librdf_world *world,
 
   /* return 1st parser if no particular one wanted */
   if(!name && !mime_type && !type_uri) {
-    factory=world->parsers;
+    factory=(librdf_parser_factory*)raptor_sequence_get_at(world->parsers, 0);
     if(!factory) {
       LIBRDF_DEBUG1("No parsers available\n");
       return NULL;
     }
   } else {
-    for(factory=world->parsers; factory; factory=factory->next) {
+    int i;
+    
+    for(i=0;
+        (factory=(librdf_parser_factory*)raptor_sequence_get_at(world->parsers, i));
+        i++) {
       /* next if name does not match */
       if(name && strcmp(factory->name, name))
 	continue;
@@ -206,7 +244,7 @@ librdf_get_parser_factory(librdf_world *world,
       if(type_uri) {
         if(!factory->type_uri)
           continue;
-        if(librdf_uri_equals(factory->type_uri, type_uri))
+        if(!librdf_uri_equals(factory->type_uri, type_uri))
           continue;
       }
 
@@ -222,6 +260,39 @@ librdf_get_parser_factory(librdf_world *world,
 }
 
 
+/**
+ * librdf_parser_enumerate:
+ * @world: redland world object
+ * @counter: index into the list of parsers
+ * @name: pointer to store the name of the parser (or NULL)
+ * @label: pointer to store syntax readable label (or NULL)
+ *
+ * Get information on parsers.
+ * 
+ * Return value: non 0 on failure of if counter is out of range
+ **/
+int
+librdf_parser_enumerate(librdf_world* world,
+                        const unsigned int counter,
+                        const char **name, const char **label)
+{
+  librdf_parser_factory *factory;
+  
+  librdf_world_open(world);
+
+  factory=(librdf_parser_factory*)raptor_sequence_get_at(world->parsers,
+                                                         counter);
+  if(!factory)
+    return 1;
+  
+  if(name)
+    *name=factory->name;
+  if(label)
+    *label=factory->label;
+  return 0;
+}
+
+
 /**
  * librdf_new_parser:
  * @world: redland world object
@@ -243,6 +314,8 @@ librdf_new_parser(librdf_world *world,
 {
   librdf_parser_factory* factory;
 
+  librdf_world_open(world);
+
   factory=librdf_get_parser_factory(world, name, mime_type, type_uri);
   if(!factory)
     return NULL;
@@ -266,6 +339,8 @@ librdf_new_parser_from_factory(librdf_world *world,
 {
   librdf_parser* d;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(factory, librdf_parser_factory, NULL);
 
   d=(librdf_parser*)LIBRDF_CALLOC(librdf_parser, 1, sizeof(librdf_parser));
@@ -317,7 +392,7 @@ librdf_free_parser(librdf_parser *parser)
  * librdf_parser_parse_as_stream:
  * @parser: the parser
  * @uri: the URI to read
- * @base_uri: the base URI to use (or NULL if the same)
+ * @base_uri: the base URI to use or NULL
  *
  * Parse a URI to a librdf_stream of statements.
  * 
@@ -347,7 +422,7 @@ librdf_parser_parse_as_stream(librdf_parser* parser, librdf_uri* uri,
  * librdf_parser_parse_into_model:
  * @parser: the parser
  * @uri: the URI to read the content
- * @base_uri: the base URI to use (or NULL if the same as @uri)
+ * @base_uri: the base URI to use or NULL
  * @model: the model to use
  *
  * Parse a URI of content into an librdf_model.
@@ -379,7 +454,7 @@ librdf_parser_parse_into_model(librdf_parser* parser, librdf_uri* uri,
  * librdf_parser_parse_string_as_stream:
  * @parser: the parser
  * @string: the string to parse
- * @base_uri: the base URI to use
+ * @base_uri: the base URI to use or NULL
  *
  * Parse a string of content to a librdf_stream of statements.
  * 
@@ -392,7 +467,6 @@ librdf_parser_parse_string_as_stream(librdf_parser* parser,
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, NULL);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, NULL);
 
   if(parser->factory->parse_string_as_stream)
     return parser->factory->parse_string_as_stream(parser->context,
@@ -406,7 +480,7 @@ librdf_parser_parse_string_as_stream(librdf_parser* parser,
  * librdf_parser_parse_string_into_model:
  * @parser: the parser
  * @string: the content to parse
- * @base_uri: the base URI to use
+ * @base_uri: the base URI to use or NULL
  * @model: the model to use
  *
  * Parse a string of content into an librdf_model.
@@ -420,7 +494,6 @@ librdf_parser_parse_string_into_model(librdf_parser* parser,
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
 
   if(parser->factory->parse_string_into_model)
@@ -436,7 +509,7 @@ librdf_parser_parse_string_into_model(librdf_parser* parser,
  * @parser: the parser
  * @string: the string to parse
  * @length: length of the string content (must be >0)
- * @base_uri: the base URI to use
+ * @base_uri: the base URI to use or NULL
  *
  * Parse a counted string of content to a librdf_stream of statements.
  * 
@@ -450,7 +523,6 @@ librdf_parser_parse_counted_string_as_stream(librdf_parser* parser,
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, NULL);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, NULL);
   LIBRDF_ASSERT_RETURN(length < 1, "string length is not greater than zero", 
                        NULL);
 
@@ -468,7 +540,7 @@ librdf_parser_parse_counted_string_as_stream(librdf_parser* parser,
  * @parser: the parser
  * @string: the content to parse
  * @length: length of content (must be >0)
- * @base_uri: the base URI to use
+ * @base_uri: the base URI to use or NULL
  * @model: the model to use
  *
  * Parse a counted string of content into an librdf_model.
@@ -484,7 +556,6 @@ librdf_parser_parse_counted_string_into_model(librdf_parser* parser,
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(parser, librdf_parser, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, string, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
   LIBRDF_ASSERT_RETURN(length < 1, "string length is not greater than zero", 1);
 
@@ -497,39 +568,6 @@ librdf_parser_parse_counted_string_into_model(librdf_parser* parser,
 }
 
 
-/**
- * librdf_init_parser:
- * @world: redland world object
- *
- * INTERNAL - Initialise the librdf_parser class.
- *
- **/
-void
-librdf_init_parser(librdf_world *world) 
-{
-#ifdef HAVE_RAPTOR_RDF_PARSER
-  librdf_parser_raptor_constructor(world);
-#endif
-}
-
-
-/**
- * librdf_finish_parser:
- * @world: redland world object
- *
- * INTERNAL - Terminate the librdf_parser class.
- *
- **/
-void
-librdf_finish_parser(librdf_world *world) 
-{
-  librdf_delete_parser_factories(world);
-#ifdef HAVE_RAPTOR_RDF_PARSER
-  librdf_parser_raptor_destructor();
-#endif
-}
-
-
 /**
  * librdf_parser_set_error:
  * @parser: the parser
@@ -613,6 +651,99 @@ librdf_parser_set_feature(librdf_parser* parser, librdf_uri* feature,
   return (-1);
 }
 
+
+/**
+ * librdf_parser_get_accept_header:
+ * @parser: parser
+ * 
+ * Get an HTTP Accept value for the parser.
+ *
+ * The returned string must be freed by the caller such as with
+ * raptor_free_memory().
+ *
+ * Return value: a new Accept: header string or NULL on failure
+ **/
+char*
+librdf_parser_get_accept_header(librdf_parser* parser)
+{
+  if(parser->factory->get_accept_header)
+    return parser->factory->get_accept_header(parser->context);
+  return NULL;
+}
+
+
+/**
+ * librdf_parser_guess_name:
+ * @mime_type: MIME type of syntax or NULL
+ * @buffer: content buffer or NULL
+ * @identifier: content identifier or NULL
+ * 
+ * Get a parser name for content with type or identiifer
+ * 
+ * Return value: a parser name or NULL if nothing was guessable
+ **/
+const char*
+librdf_parser_guess_name(const char *mime_type,
+                         const unsigned char *buffer, 
+                         const unsigned char *identifier)
+{
+  return raptor_guess_parser_name(NULL, mime_type, buffer,
+                                  strlen((const char*)buffer),
+                                  identifier);
+}
+
+
+/**
+ * librdf_parser_get_namespaces_seen_prefix:
+ * @parser: #librdf_parser object
+ * @offset: index into list of namespaces
+ * 
+ * Get the prefix of namespaces seen during parsing
+ * 
+ * Return value: prefix or NULL if no such namespace prefix
+ **/
+const char*
+librdf_parser_get_namespaces_seen_prefix(librdf_parser* parser, int offset)
+{
+  if(parser->factory->get_namespaces_seen_prefix)
+    return parser->factory->get_namespaces_seen_prefix(parser->context, offset);
+  return NULL;
+}
+
+
+/**
+ * librdf_parser_get_namespaces_seen_uri:
+ * @parser: #librdf_parser object
+ * @offset: index into list of namespaces
+ * 
+ * Get the uri of namespaces seen during parsing
+ * 
+ * Return value: uri or NULL if no such namespace uri
+ **/
+librdf_uri*
+librdf_parser_get_namespaces_seen_uri(librdf_parser* parser, int offset)
+{
+  if(parser->factory->get_namespaces_seen_uri)
+    return parser->factory->get_namespaces_seen_uri(parser->context, offset);
+  return NULL;
+}
+
+/**
+ * librdf_parser_get_namespaces_seen_count:
+ * @parser: #librdf_parser object
+ * 
+ * Get the number of namespaces seen during parsing
+ * 
+ * Return value: uri or NULL if no such namespace uri
+ **/
+int
+librdf_parser_get_namespaces_seen_count(librdf_parser* parser)
+{
+  if(parser->factory->get_namespaces_seen_count)
+    return parser->factory->get_namespaces_seen_count(parser->context);
+  return 0;
+}
+
 #endif
 
 
@@ -657,16 +788,17 @@ int main(int argc, char *argv[]);
 int
 main(int argc, char *argv[]) 
 {
-  char *test_parser_types[]={"rdfxml", "ntriples", "turtle", NULL};
+  const char *test_parser_types[]={"rdfxml", "ntriples", "turtle", NULL};
   #define URI_STRING_COUNT 3
   const unsigned char *file_uri_strings[URI_STRING_COUNT]={(const unsigned char*)"http://example.org/test1.rdf", (const unsigned char*)"http://example.org/test2.nt", (const unsigned char*)"http://example.org/test3.ttl"};
   const unsigned char *file_content[URI_STRING_COUNT]={(const unsigned char*)RDFXML_CONTENT, (const unsigned char*)NTRIPLES_CONTENT, (const unsigned char*)TURTLE_CONTENT};
   librdf_uri* uris[URI_STRING_COUNT];
   int i;
-  char *type;
-  char *program=argv[0];
+  const char *type;
+  const char *program=librdf_basename((const char*)argv[0]);
   librdf_world *world;
-
+  int failures;
+  
   world=librdf_new_world();
   librdf_world_open(world);
 
@@ -674,14 +806,19 @@ main(int argc, char *argv[])
     uris[i]=librdf_new_uri(world, file_uri_strings[i]);
   }
 
+  failures=0;
   for(i=0; (type=test_parser_types[i]); i++) {
-    librdf_storage* storage;
-    librdf_model *model;
-    librdf_parser* parser;
+    librdf_storage* storage=NULL;
+    librdf_model *model=NULL;
+    librdf_parser* parser=NULL;
     librdf_stream *stream=NULL;
     size_t length=strlen((const char*)file_content[i]);
     int size;
+    char *accept_h;
 
+    if(i>0)
+      break;
+    
     fprintf(stderr, "%s: Testing parsing syntax '%s'\n", program, type);
 
     fprintf(stderr, "%s: Creating storage and model\n", program);
@@ -700,8 +837,17 @@ main(int argc, char *argv[])
     parser=librdf_new_parser(world, type, NULL, NULL);
     if(!parser) {
       fprintf(stderr, "%s: Failed to create new parser type %s\n", program, type);
-      continue;
+      failures++;
+      goto tidy_test;
     }
+
+
+    accept_h=librdf_parser_get_accept_header(parser);
+    if(accept_h) {
+      fprintf(stderr, "%s: Parser accept header: '%s'\n", program, accept_h);
+      free(accept_h);
+    } else
+      fprintf(stderr, "%s: Parser has no accept header\n", program);
     
 
     fprintf(stderr, "%s: Adding %s counted string content as stream\n", 
@@ -711,35 +857,42 @@ main(int argc, char *argv[])
                                                              length,
                                                              uris[i]))) {
       fprintf(stderr, "%s: Failed to parse RDF from counted string %d as stream\n", program, i);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
     librdf_model_add_statements(model, stream);
     librdf_free_stream(stream);
+    stream=NULL;
+
 
     size=librdf_model_size(model);
     fprintf(stderr, "%s: Model size is %d triples\n", program, size);
     if(size != EXPECTED_TRIPLES_COUNT) {
       fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
               program, size, EXPECTED_TRIPLES_COUNT);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
 
+
     fprintf(stderr, "%s: Adding %s string content as stream\n", program, type);
     if(!(stream=librdf_parser_parse_string_as_stream(parser, 
                                                      file_content[i],
                                                      uris[i]))) {
       fprintf(stderr, "%s: Failed to parse RDF from string %d as stream\n", program, i);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
     librdf_model_add_statements(model, stream);
     librdf_free_stream(stream);
+    stream=NULL;
     
     size=librdf_model_size(model);
     fprintf(stderr, "%s: Model size is %d triples\n", program, size);
     if(size != EXPECTED_TRIPLES_COUNT) {
       fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
               program, size, EXPECTED_TRIPLES_COUNT);
-      exit(1);
+      failures++;
     }
     
 
@@ -749,7 +902,8 @@ main(int argc, char *argv[])
                                                      length,
                                                      uris[i], model)) {
       fprintf(stderr, "%s: Failed to parse RDF from counted string %d into model\n", program, i);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
     
     size=librdf_model_size(model);
@@ -757,31 +911,52 @@ main(int argc, char *argv[])
     if(size != EXPECTED_TRIPLES_COUNT) {
       fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
               program, size, EXPECTED_TRIPLES_COUNT);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
 
-    
+
     fprintf(stderr, "%s: Adding %s string content\n", program, type);
     if(librdf_parser_parse_string_into_model(parser, 
                                              file_content[i],
                                              uris[i], model)) {
       fprintf(stderr, "%s: Failed to parse RDF from string %d into model\n", program, i);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
 
+
+    for(i=0; i < librdf_parser_get_namespaces_seen_count(parser); i++) {
+      const char* prefix=librdf_parser_get_namespaces_seen_prefix(parser, i);
+      librdf_uri* uri=librdf_parser_get_namespaces_seen_uri(parser, i);
+
+      fprintf(stderr, "%s: Saw namespace %d): prefix:%s URI:%s\n", program, i,
+              (!prefix ? "" : (const char*)prefix),
+              (!uri ? "(none)" : (const char*)librdf_uri_as_string(uri)));
+      
+    }
+    
+
     size=librdf_model_size(model);
     fprintf(stderr, "%s: Model size is %d triples\n", program, size);
     if(size != EXPECTED_TRIPLES_COUNT) {
       fprintf(stderr, "%s: Returned %d triples, not %d as expected\n",
               program, size, EXPECTED_TRIPLES_COUNT);
-      exit(1);
+      failures++;
+      goto tidy_test;
     }
 
     
     fprintf(stderr, "%s: Freeing parser, model and storage\n", program);
-    librdf_free_parser(parser);
-    librdf_free_model(model);
-    librdf_free_storage(storage);
+  tidy_test:
+    if(stream)
+      librdf_free_stream(stream);
+    if(parser)
+      librdf_free_parser(parser);
+    if(model)
+      librdf_free_model(model);
+    if(storage)
+      librdf_free_storage(storage);
   }
   
 
@@ -792,8 +967,7 @@ main(int argc, char *argv[])
   
   librdf_free_world(world);
   
-  /* keep gcc -Wall happy */
-  return(0);
+  return failures;
 }
 
 #endif
