@@ -2,9 +2,9 @@
  *
  * rdf_parser.c - RDF Parser using Raptor
  *
- * $Id: rdf_parser_raptor.c 10942 2006-05-04 05:01:09Z dajobe $
+ * $Id: rdf_parser_raptor.c 12220 2007-05-06 03:17:58Z dajobe $
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -55,9 +55,12 @@ static void librdf_parser_raptor_serialise_finished(void* context);
 typedef struct {
   librdf_parser *parser;        /* librdf parser object */
   librdf_hash *bnode_hash;      /* bnode id (raptor=>internal) map */
-  raptor_parser *rdf_parser;      /* source URI string (for raptor) */
-  char *parser_name;            /* raptor parser name to use */
+  raptor_parser *rdf_parser;    /* source URI string (for raptor) */
+  const char *parser_name;      /* raptor parser name to use */
 
+  raptor_sequence* nspace_prefixes;
+  raptor_sequence* nspace_uris;
+  
   int errors;
   int warnings;
 } librdf_parser_raptor_context;
@@ -86,11 +89,10 @@ typedef struct {
    * sequence is empty := current=NULL and librdf_list_size(statements)=0
    */
   librdf_statement* current; /* current statement */
-  librdf_list statements;   /* STATIC following statements after current */
+  librdf_list* statements;
 } librdf_parser_raptor_stream_context;
 
 
-
 /**
  * librdf_parser_raptor_init:
  * @parser: the parser
@@ -141,6 +143,11 @@ librdf_parser_raptor_terminate(void *context)
 
   if(pcontext->bnode_hash)
     librdf_free_hash(pcontext->bnode_hash);
+
+  if(pcontext->nspace_prefixes)
+    raptor_free_sequence(pcontext->nspace_prefixes);
+  if(pcontext->nspace_uris)
+    raptor_free_sequence(pcontext->nspace_uris);
 }
   
 
@@ -172,10 +179,7 @@ librdf_parser_raptor_new_statement_handler(void *context,
   if(rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
     node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->subject);
   } else if (rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
-    node=librdf_new_node_from_normalised_uri_string(world,
-                                                    librdf_uri_as_string((librdf_uri*)rstatement->subject),
-                                                    scontext->source_uri,
-                                                    scontext->base_uri);
+    node=librdf_new_node_from_uri(world, (librdf_uri*)rstatement->subject);
   } else {
     librdf_log(world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
@@ -198,10 +202,7 @@ librdf_parser_raptor_new_statement_handler(void *context,
     node=librdf_new_node_from_uri_string(world, (const unsigned char*)ordinal_buffer);
   } else if (rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE ||
              rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE) {
-    node=librdf_new_node_from_normalised_uri_string(world,
-                                                    librdf_uri_as_string((librdf_uri*)rstatement->predicate),
-                                                    scontext->source_uri,
-                                                    scontext->base_uri);
+    node=librdf_new_node_from_uri(world, (librdf_uri*)rstatement->predicate);
   } else {
     librdf_log(world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
@@ -235,10 +236,7 @@ librdf_parser_raptor_new_statement_handler(void *context,
     node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->object);
     librdf_statement_set_object(statement, node);
   } else if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
-    node=librdf_new_node_from_normalised_uri_string(world,
-                                                    librdf_uri_as_string((librdf_uri*)rstatement->object),
-                                                    scontext->source_uri,
-                                                    scontext->base_uri);
+    node=librdf_new_node_from_uri(world, (librdf_uri*)rstatement->object);
     librdf_statement_set_object(statement, node);
   } else {
     librdf_log(world,
@@ -261,7 +259,49 @@ librdf_parser_raptor_new_statement_handler(void *context,
     librdf_model_add_statement(scontext->model, statement);
     librdf_free_statement(statement);
   } else
-    librdf_list_add(&scontext->statements, statement);
+    librdf_list_add(scontext->statements, statement);
+}
+
+
+/*
+ * librdf_parser_raptor_namespace_handler - helper callback function for raptor RDF when a namespace is seen
+ * @context: context for callback
+ * @statement: raptor_statement
+ * 
+ * Adds the statement to the list of statements.
+ */
+static void
+librdf_parser_raptor_namespace_handler(void* user_data, 
+                                       raptor_namespace *nspace)
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)user_data;
+  const unsigned char* prefix;
+  unsigned char* nprefix;
+  size_t prefix_length;
+  librdf_uri* uri;
+  int i;
+  
+  uri=(librdf_uri*)raptor_namespace_get_uri(nspace);
+  if(!uri)
+    return;
+  
+  for(i=0; i < raptor_sequence_size(pcontext->nspace_uris); i++) {
+    librdf_uri* u=(librdf_uri*)raptor_sequence_get_at(pcontext->nspace_uris, i);
+    if(librdf_uri_equals(uri, u))
+      return;
+  }
+  
+  /* new namespace */
+  uri=librdf_new_uri_from_uri(uri);
+  raptor_sequence_push(pcontext->nspace_uris, uri);
+
+  prefix=raptor_namespace_get_counted_prefix(nspace, &prefix_length);
+  if(prefix) {
+    nprefix=(unsigned char*)LIBRDF_MALLOC(cstring, prefix_length+1);
+    strncpy((char*)nprefix, (const char*)prefix, prefix_length+1);
+  } else
+    nprefix=NULL;
+  raptor_sequence_push(pcontext->nspace_prefixes, nprefix);
 }
 
 
@@ -296,7 +336,7 @@ librdf_parser_raptor_warning_handler(void *data, raptor_locator *locator,
 
 
 /*
-v * librdf_parser_raptor_get_next_statement - helper function to get the next statement
+ * librdf_parser_raptor_get_next_statement - helper function to get the next statement
  * @context: serialisation context
  * 
  * Return value: >0 if a statement found, 0 at end of file, or <0 on error
@@ -320,8 +360,8 @@ librdf_parser_raptor_get_next_statement(librdf_parser_raptor_stream_context *con
     }
 
     /* parsing found at least 1 statement, return */
-    if(librdf_list_size(&context->statements)) {
-      context->current=(librdf_statement*)librdf_list_pop(&context->statements);
+    if(librdf_list_size(context->statements)) {
+      context->current=(librdf_statement*)librdf_list_pop(context->statements);
       status=1;
       break;
     }
@@ -379,6 +419,17 @@ librdf_parser_raptor_parse_file_handle_as_stream(librdf_world* world,
   librdf_stream *stream;
   int rc;
 
+  librdf_world_open(world);
+
+  if(raptor_get_need_base_uri(pcontext->rdf_parser) && !base_uri) {
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Missing base URI for %s parser", 
+               pcontext->parser_name);
+    return NULL;
+  }
+  
+
   pcontext->errors=0;
   pcontext->warnings=0;
   
@@ -386,8 +437,19 @@ librdf_parser_raptor_parse_file_handle_as_stream(librdf_world* world,
   if(!scontext)
     return NULL;
 
+  scontext->statements=librdf_new_list(world);
+
+  if(pcontext->nspace_prefixes)
+    raptor_free_sequence(pcontext->nspace_prefixes);
+  if(pcontext->nspace_uris)
+    raptor_free_sequence(pcontext->nspace_uris);
+  pcontext->nspace_prefixes=raptor_new_sequence(free, NULL);
+  pcontext->nspace_uris=raptor_new_sequence((raptor_sequence_free_handler*)librdf_free_uri, NULL);
+  
   raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
+  raptor_set_namespace_handler(pcontext->rdf_parser, pcontext, 
+                               librdf_parser_raptor_namespace_handler);
   
   raptor_set_error_handler(pcontext->rdf_parser, pcontext, 
                            librdf_parser_raptor_error_handler);
@@ -400,24 +462,28 @@ librdf_parser_raptor_parse_file_handle_as_stream(librdf_world* world,
   
   scontext->pcontext=pcontext;
   if(base_uri)
-    scontext->source_uri = librdf_new_uri_from_uri(base_uri);
-  if(base_uri)
-    scontext->base_uri = librdf_new_uri_from_uri(base_uri);
+    base_uri = librdf_new_uri_from_uri(base_uri);
+  if(scontext->base_uri)
+    librdf_free_uri(scontext->base_uri); 
+  scontext->base_uri = base_uri;
+
   scontext->fh=fh;
   scontext->close_fh=close_fh;
 
   /* Start the parse */
+  stream=NULL;
   rc=raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri);
-
-  /* start parsing; initialises scontext->statements, scontext->current */
-  librdf_parser_raptor_get_next_statement(scontext);
-
-  stream=librdf_new_stream(world,
-                           (void*)scontext,
-                           &librdf_parser_raptor_serialise_end_of_stream,
-                           &librdf_parser_raptor_serialise_next_statement,
-                           &librdf_parser_raptor_serialise_get_statement,
-                           &librdf_parser_raptor_serialise_finished);
+  if(!rc) {
+    /* start parsing; initialises scontext->statements, scontext->current */
+    librdf_parser_raptor_get_next_statement(scontext);
+    
+    stream=librdf_new_stream(world,
+                             (void*)scontext,
+                             &librdf_parser_raptor_serialise_end_of_stream,
+                             &librdf_parser_raptor_serialise_next_statement,
+                             &librdf_parser_raptor_serialise_get_statement,
+                             &librdf_parser_raptor_serialise_finished);
+  }
   if(!stream) {
     librdf_parser_raptor_serialise_finished((void*)scontext);
     return NULL;
@@ -445,12 +511,12 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
 /**
  * librdf_parser_raptor_parse_as_stream_common:
  * @context: parser context
- * @uri: #librdf_uri URI of ontent source
+ * @uri: #librdf_uri URI of RDF content source
  * @string: or content string
  * @length: length of the string or 0 if not yet counted
  * @base_uri: #librdf_uri URI of the content location or NULL if the same
  *
- * Retrieve the content at URI/string and parse it into a librdf_stream.
+ * Retrieve the content at URI/string and parse it into a #librdf_stream.
  *
  * Only one of uri or string may be given
  *
@@ -464,7 +530,20 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   librdf_parser_raptor_stream_context* scontext;
   librdf_stream *stream;
+
+  if(!base_uri && uri)
+    base_uri=uri;
   
+  if(raptor_get_need_base_uri(pcontext->rdf_parser) && !base_uri) {
+    librdf_log(pcontext->parser->world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Missing base URI for %s parser", 
+               pcontext->parser_name);
+    
+    return NULL;
+  }
+  
+
   pcontext->errors=0;
   pcontext->warnings=0;
   
@@ -477,7 +556,9 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
     
     fh=fopen(filename, "r");
     if(!fh) {
-      LIBRDF_DEBUG3("Failed to open file '%s' - %s\n", filename, strerror(errno));
+      librdf_log(pcontext->parser->world, 0, LIBRDF_LOG_ERROR,
+                 LIBRDF_FROM_PARSER, NULL, "failed to open file '%s' - %s",
+                 filename, strerror(errno));
       SYSTEM_FREE(filename);
       return NULL;
     }
@@ -496,8 +577,19 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   if(!scontext)
     return NULL;
 
+  scontext->statements=librdf_new_list(pcontext->parser->world);
+
+  if(pcontext->nspace_prefixes)
+    raptor_free_sequence(pcontext->nspace_prefixes);
+  if(pcontext->nspace_uris)
+    raptor_free_sequence(pcontext->nspace_uris);
+  pcontext->nspace_prefixes=raptor_new_sequence(free, NULL);
+  pcontext->nspace_uris=raptor_new_sequence((raptor_sequence_free_handler*)librdf_free_uri, NULL);
+  
   raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
+  raptor_set_namespace_handler(pcontext->rdf_parser, pcontext, 
+                               librdf_parser_raptor_namespace_handler);
   
   raptor_set_error_handler(pcontext->rdf_parser, pcontext, 
                            librdf_parser_raptor_error_handler);
@@ -509,26 +601,35 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   
   
   scontext->pcontext=pcontext;
-  if(!base_uri)
-    base_uri=uri;
-
-  /* No base URI given, cannot proceed */
-  if(!base_uri)
-    return NULL;
-
-  if(uri)
-    scontext->source_uri = librdf_new_uri_from_uri(uri);
-  else
-    scontext->source_uri = librdf_new_uri_from_uri(base_uri);
-  scontext->base_uri = librdf_new_uri_from_uri(base_uri);
 
   if(uri) {
-    raptor_www *www=raptor_www_new();
+    if(scontext->source_uri)
+      librdf_free_uri(scontext->source_uri); 
+    scontext->source_uri = librdf_new_uri_from_uri(uri);
+  }
+
+  if(base_uri)
+    base_uri= librdf_new_uri_from_uri(base_uri);
+  if(scontext->base_uri)
+    librdf_free_uri(scontext->base_uri); 
+  scontext->base_uri = base_uri;
+
+  if(uri) {
+    raptor_www *www;
+    const char *accept_h;
+
+    www=raptor_www_new();
     if(!www) {
       LIBRDF_FREE(librdf_parser_raptor_stream_context, scontext);
       return NULL;
     }
     
+    accept_h=raptor_parser_get_accept_header(pcontext->rdf_parser);
+    if(accept_h) {
+      raptor_www_set_http_accept(www, accept_h);
+      raptor_free_memory((char*)accept_h);
+    }
+
     raptor_www_set_write_bytes_handler(www, 
                                        librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler,
                                        scontext);
@@ -553,9 +654,9 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   
 
   /* get first statement, else is empty */
-  scontext->current=(librdf_statement*)librdf_list_pop(&scontext->statements);
+  scontext->current=(librdf_statement*)librdf_list_pop(scontext->statements);
 
-  stream=librdf_new_stream(base_uri->world,
+  stream=librdf_new_stream(pcontext->parser->world,
                            (void*)scontext,
                            &librdf_parser_raptor_serialise_end_of_stream,
                            &librdf_parser_raptor_serialise_next_statement,
@@ -633,19 +734,18 @@ librdf_parser_raptor_parse_counted_string_as_stream(void *context,
                                                      base_uri);
 }
 
-/**
- * librdf_parser_raptor_parse_uri_into_model:
+/*
+ * librdf_parser_raptor_parse_uri_into_model_common:
  * @context: parser context
- * @uri: #librdf_uri URI of RDF/XML content source
- * @string: string content to parser
+ * @uri: #librdf_uri URI of RDF content source or NULL if from a string
+ * @string: string content to parser or NULL if from a URI
  * @length: length of the string or 0 if not yet counted
- * @base_uri: #librdf_uri URI of the content location
+ * @base_uri: #librdf_uri URI of the content location or NULL
  * @model: #librdf_model of model
  *
- * Retrieve the RDF/XML content at URI and store it into a librdf_model.
+ * Retrieve the RDF content at URI and store it into a librdf_model.
  *
  * Parses the content at @uri or @string and store it in the given model.
- * A base URI must be given if @uri is NULL, when @string is used.
  *
  * Return value: non 0 on failure
  **/
@@ -661,6 +761,18 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   librdf_parser_raptor_stream_context* scontext;
   
+  if(!base_uri)
+    base_uri=uri;
+  
+  if(raptor_get_need_base_uri(pcontext->rdf_parser) && !base_uri) {
+    librdf_log(pcontext->parser->world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Missing base URI for %s parser", 
+               pcontext->parser_name);
+
+    return 1;
+  }
+
   pcontext->errors=0;
   pcontext->warnings=0;
   
@@ -668,8 +780,17 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   if(!scontext)
     return 1;
 
+  if(pcontext->nspace_prefixes)
+    raptor_free_sequence(pcontext->nspace_prefixes);
+  if(pcontext->nspace_uris)
+    raptor_free_sequence(pcontext->nspace_uris);
+  pcontext->nspace_prefixes=raptor_new_sequence(free, NULL);
+  pcontext->nspace_uris=raptor_new_sequence((raptor_sequence_free_handler*)librdf_free_uri, NULL);
+  
   raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
+  raptor_set_namespace_handler(pcontext->rdf_parser, pcontext, 
+                               librdf_parser_raptor_namespace_handler);
   
   raptor_set_error_handler(pcontext->rdf_parser, pcontext, 
                            librdf_parser_raptor_error_handler);
@@ -682,25 +803,24 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   
   scontext->pcontext=pcontext;
 
-  if(!base_uri)
-    base_uri=uri;
-
-  /* No base URI given, cannot proceed */
-  if(!base_uri)
-    return 1;
-    
-  if(uri)
+  if(uri) {
+    if(scontext->source_uri)
+      librdf_free_uri(scontext->source_uri); 
     scontext->source_uri = librdf_new_uri_from_uri(uri);
-  else
-    scontext->source_uri = librdf_new_uri_from_uri(base_uri);
+  }
 
-  scontext->base_uri = librdf_new_uri_from_uri(base_uri);
+  if(base_uri)
+    base_uri = librdf_new_uri_from_uri(base_uri);
+  if(scontext->base_uri)
+    librdf_free_uri(scontext->base_uri); 
+  scontext->base_uri = base_uri;
 
   /* direct into model */
   scontext->model = model;
 
   if(uri) {
-    status=raptor_parse_uri(pcontext->rdf_parser, (raptor_uri*)uri, (raptor_uri*)base_uri);
+    status=raptor_parse_uri(pcontext->rdf_parser, (raptor_uri*)uri, 
+                            (raptor_uri*)base_uri);
   } else {
     status=raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri);
     if(!status) {
@@ -721,11 +841,11 @@ librdf_parser_raptor_parse_into_model_common(void *context,
 /**
  * librdf_parser_raptor_parse_uri_into_model:
  * @context: parser context
- * @uri: #librdf_uri URI of RDF/XML content source
+ * @uri: #librdf_uri URI of RDF content source
  * @base_uri: #librdf_uri URI of the content location (or NULL if the same as @uri)
  * @model: #librdf_model of model
  *
- * Retrieve the RDF/XML content at URI and store it into a librdf_model.
+ * Retrieve the RDF content at URI and store it into a librdf_model.
  *
  * Retrieves all statements and stores them in the given model.
  *
@@ -748,7 +868,7 @@ librdf_parser_raptor_parse_uri_into_model(void *context, librdf_uri *uri,
  * @base_uri: #librdf_uri URI of the content location
  * @model: #librdf_model of model
  *
- * Parse the RDF/XML content in a string and store it into a librdf_model.
+ * Parse the RDF content in a string and store it into a librdf_model.
  *
  * Stores the statements found parsing string and stores in the given model.
  *
@@ -804,7 +924,7 @@ librdf_parser_raptor_serialise_end_of_stream(void* context)
 {
   librdf_parser_raptor_stream_context* scontext=(librdf_parser_raptor_stream_context*)context;
 
-  return (!scontext->current && !librdf_list_size(&scontext->statements));
+  return (!scontext->current && !librdf_list_size(scontext->statements));
 }
 
 
@@ -826,7 +946,7 @@ librdf_parser_raptor_serialise_next_statement(void* context)
 
   /* get another statement if there is one */
   while(!scontext->current) {
-    scontext->current=(librdf_statement*)librdf_list_pop(&scontext->statements);
+    scontext->current=(librdf_statement*)librdf_list_pop(scontext->statements);
     if(scontext->current)
       break;
   
@@ -896,13 +1016,15 @@ librdf_parser_raptor_serialise_finished(void* context)
     if(scontext->base_uri)
       librdf_free_uri(scontext->base_uri); 
  
-    while((statement=(librdf_statement*)librdf_list_pop(&scontext->statements)))
-      librdf_free_statement(statement);
-    librdf_list_clear(&scontext->statements);
+    if(scontext->statements) {
+      while((statement=(librdf_statement*)librdf_list_pop(scontext->statements)))
+        librdf_free_statement(statement);
+      librdf_free_list(scontext->statements);
+    }
 
     if(scontext->fh && scontext->close_fh)
       fclose(scontext->fh); 
- 
+
     LIBRDF_FREE(librdf_parser_raptor_context, scontext);
   }
 }
@@ -988,7 +1110,7 @@ librdf_raptor_new_uri_from_uri_local_name(void *context,
 static raptor_uri*
 librdf_raptor_new_uri_relative_to_base(void *context,
                                        raptor_uri *base_uri,
-                                       const unsigned char *uri_string) 
+                                       const unsigned char *uri_string)
 {
   return (raptor_uri*)librdf_new_uri_relative_to_base((librdf_uri*)base_uri, uri_string);
 }
@@ -1037,6 +1159,67 @@ librdf_raptor_uri_as_counted_string(void *context, raptor_uri *uri, size_t *len_
 }
 
 
+static char*
+librdf_parser_raptor_get_accept_header(void* context) 
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+  const char* accept;
+  char* r_accept;
+  size_t length;
+  
+  accept=raptor_parser_get_accept_header(pcontext->rdf_parser);
+  if(!accept)
+    return NULL;
+  length=strlen(accept);
+  r_accept=(char*)LIBRDF_MALLOC(cstring, length+1);
+  strncpy(r_accept, accept, length+1);
+  raptor_free_memory((void*)accept);
+  
+  return r_accept;
+}
+
+
+static const char*
+librdf_parser_raptor_get_namespaces_seen_prefix(void* context, int offset)
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+
+  if(!pcontext->nspace_prefixes)
+    return NULL;
+
+  if(offset < 0 || offset > raptor_sequence_size(pcontext->nspace_prefixes))
+    return NULL;
+
+  return (const char*)raptor_sequence_get_at(pcontext->nspace_prefixes, offset);
+}
+
+
+static librdf_uri*
+librdf_parser_raptor_get_namespaces_seen_uri(void* context, int offset)
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+
+  if(!pcontext->nspace_uris)
+    return NULL;
+
+  if(offset < 0 || offset > raptor_sequence_size(pcontext->nspace_uris))
+    return NULL;
+
+  return (librdf_uri*)raptor_sequence_get_at(pcontext->nspace_uris, offset);
+}
+
+
+static int
+librdf_parser_raptor_get_namespaces_seen_count(void* context)
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+
+  if(!pcontext->nspace_uris)
+    return 0;
+
+  return raptor_sequence_size(pcontext->nspace_uris);
+}
+
 
 static raptor_uri_handler librdf_raptor_uri_handler = {
   librdf_raptor_new_uri,
@@ -1074,6 +1257,10 @@ librdf_parser_raptor_register_factory(librdf_parser_factory *factory)
   factory->parse_counted_string_into_model = librdf_parser_raptor_parse_counted_string_into_model;
   factory->get_feature = librdf_parser_raptor_get_feature;
   factory->set_feature = librdf_parser_raptor_set_feature;
+  factory->get_accept_header = librdf_parser_raptor_get_accept_header;
+  factory->get_namespaces_seen_prefix = librdf_parser_raptor_get_namespaces_seen_prefix;
+  factory->get_namespaces_seen_uri = librdf_parser_raptor_get_namespaces_seen_uri;
+  factory->get_namespaces_seen_count = librdf_parser_raptor_get_namespaces_seen_count;
 }
 
 
@@ -1095,24 +1282,27 @@ librdf_parser_raptor_constructor(librdf_world *world)
   /* enumerate from parser 1, so the default parser 0 is done last */
   for(i=1; 1; i++) {
     const char *syntax_name=NULL;
+    const char *syntax_label=NULL;
     const char *mime_type=NULL;
     const unsigned char *uri_string=NULL;
 
-    if(raptor_syntaxes_enumerate(i, &syntax_name, NULL, 
+    if(raptor_syntaxes_enumerate(i, &syntax_name, &syntax_label, 
                                  &mime_type, &uri_string)) {
       /* reached the end of the parsers, now register the default one */
       i=0;
-      raptor_syntaxes_enumerate(i, &syntax_name, NULL,
+      raptor_syntaxes_enumerate(i, &syntax_name, &syntax_label,
                                 &mime_type, &uri_string);
     }
 
     if(!strcmp(syntax_name, "rdfxml")) {
       /* legacy name - see librdf_parser_raptor_init */
-      librdf_parser_register_factory(world, "raptor", mime_type, uri_string,
+      librdf_parser_register_factory(world, "raptor", NULL,
+                                     mime_type, uri_string,
                                      &librdf_parser_raptor_register_factory);
     }
 
-    librdf_parser_register_factory(world, syntax_name, mime_type, uri_string,
+    librdf_parser_register_factory(world, syntax_name, syntax_label,
+                                   mime_type, uri_string,
                                    &librdf_parser_raptor_register_factory);
 
     if(!i) /* registered default parser, end */
