@@ -2,7 +2,7 @@
  *
  * rdf_serializer.c - RDF Serializer (RDF triples to syntax) interface
  *
- * $Id: rdf_serializer.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_serializer.c 12151 2007-04-23 15:19:14Z dajobe $
  *
  * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2002-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -38,14 +38,46 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
 
 #include <redland.h>
 
 
 #ifndef STANDALONE
 
-/* prototypes for helper functions */
-static void librdf_delete_serializer_factories(librdf_world *world);
+/**
+ * librdf_init_serializer:
+ * @world: redland world object
+ *
+ * INTERNAL - Initialise the serializer module.
+ *
+ **/
+void
+librdf_init_serializer(librdf_world *world) 
+{
+  librdf_serializer_raptor_constructor(world);
+}
+
+
+/**
+ * librdf_finish_serializer:
+ * @world: redland world object
+ *
+ * INTERNAL - Terminate the serializer module.
+ *
+ **/
+void
+librdf_finish_serializer(librdf_world *world) 
+{
+  raptor_free_sequence(world->serializers);
+  world->serializers=NULL;
+
+#ifdef HAVE_RAPTOR_RDF_SERIALIZER
+  librdf_serializer_raptor_destructor();
+#endif
+}
 
 
 /* helper functions */
@@ -53,32 +85,22 @@ static void
 librdf_free_serializer_factory(librdf_serializer_factory *factory) 
 {
   if(factory->name)
-    LIBRDF_FREE(librdf_serializer_factory, factory->name);
+    LIBRDF_FREE(cstring, factory->name);
+  if(factory->label)
+    LIBRDF_FREE(cstring, factory->label);
   if(factory->mime_type)
-    LIBRDF_FREE(librdf_serializer_factory, factory->mime_type);
+    LIBRDF_FREE(cstring, factory->mime_type);
   if(factory->type_uri)
     librdf_free_uri(factory->type_uri);
   LIBRDF_FREE(librdf_serializer_factory, factory);
 }
 
 
-static void
-librdf_delete_serializer_factories(librdf_world *world)
-{
-  librdf_serializer_factory *factory, *next;
-  
-  for(factory=world->serializers; factory; factory=next) {
-    next=factory->next;
-    librdf_free_serializer_factory(factory);
-  }
-  world->serializers=NULL;
-}
-
-
 /**
  * librdf_serializer_register_factory:
  * @world: redland world object
  * @name: the name of the serializer
+ * @label: the label of the serializer (optional)
  * @mime_type: MIME type of the syntax (optional)
  * @uri_string: URI of the syntax (optional)
  * @factory: function to be called to register the factor parameters
@@ -88,40 +110,57 @@ librdf_delete_serializer_factories(librdf_world *world)
  **/
 void
 librdf_serializer_register_factory(librdf_world *world,
-                                   const char *name, const char *mime_type,
+                                   const char *name, const char *label,
+                                   const char *mime_type,
                                    const unsigned char *uri_string,
                                    void (*factory) (librdf_serializer_factory*))
 {
-  librdf_serializer_factory *serializer_factory;
+  librdf_serializer_factory *serializer;
   char *name_copy;
+  char *label_copy;
+
+  librdf_world_open(world);
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for serializer %s\n", name);
 #endif
   
-  serializer_factory=(librdf_serializer_factory*)LIBRDF_CALLOC(librdf_serializer_factory, 1,
-					       sizeof(librdf_serializer_factory));
-  if(!serializer_factory)
+  if(!world->serializers)
+    world->serializers=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_serializer_factory, NULL);
+  
+  serializer=(librdf_serializer_factory*)LIBRDF_CALLOC(librdf_serializer_factory, 1,
+                                                       sizeof(librdf_serializer_factory));
+  if(!serializer)
     LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
   
   name_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(name)+1);
   if(!name_copy) {
-    librdf_free_serializer_factory(serializer_factory);
+    librdf_free_serializer_factory(serializer);
     LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
   }
   strcpy(name_copy, name);
-  serializer_factory->name=name_copy;
+  serializer->name=name_copy;
 
+  if(label) {
+    label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
+    if(!label_copy) {
+      librdf_free_serializer_factory(serializer);
+      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
+    }
+    strcpy(label_copy, label);
+    serializer->label=label_copy;
+  }
+        
   /* register mime type if any */
   if(mime_type) {
     char *mime_type_copy;
     mime_type_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(mime_type)+1);
     if(!mime_type_copy) {
-      librdf_free_serializer_factory(serializer_factory);
+      librdf_free_serializer_factory(serializer);
       LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
     }
     strcpy(mime_type_copy, mime_type);
-    serializer_factory->mime_type=mime_type_copy;
+    serializer->mime_type=mime_type_copy;
   }
 
   /* register URI if any */
@@ -130,24 +169,22 @@ librdf_serializer_register_factory(librdf_world *world,
 
     uri=librdf_new_uri(world, uri_string);
     if(!uri) {
-      librdf_free_serializer_factory(serializer_factory);
+      librdf_free_serializer_factory(serializer);
       LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
     }
-    serializer_factory->type_uri=uri;
+    serializer->type_uri=uri;
   }
   
         
   /* Call the serializer registration function on the new object */
-  (*factory)(serializer_factory);
+  (*factory)(serializer);
 
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  LIBRDF_DEBUG3("%s has context size %d\n", name, serializer_factory->context_length);
+  LIBRDF_DEBUG3("%s has context size %d\n", name, serializer->context_length);
 #endif
   
-  serializer_factory->next = world->serializers;
-  world->serializers = serializer_factory;
-  
+  raptor_sequence_push(world->serializers, serializer);
 }
 
 
@@ -172,24 +209,30 @@ librdf_get_serializer_factory(librdf_world *world,
 {
   librdf_serializer_factory *factory;
   
+  librdf_world_open(world);
+
   if(name && !*name)
     name=NULL;
   if(!mime_type || (mime_type && !*mime_type)) {
     if(!name && !type_uri)
-      mime_type="application/rdf+xml";
+      name="rdfxml";
     else
       mime_type=NULL;
   }
 
   /* return 1st serializer if no particular one wanted */
   if(!name && !mime_type && !type_uri) {
-    factory=world->serializers;
+    factory=(librdf_serializer_factory*)raptor_sequence_get_at(world->serializers, 0);
     if(!factory) {
       LIBRDF_DEBUG1("No serializers available\n");
       return NULL;
     }
   } else {
-    for(factory=world->serializers; factory; factory=factory->next) {
+    int i;
+    
+    for(i=0;
+        (factory=(librdf_serializer_factory*)raptor_sequence_get_at(world->serializers, i));
+        i++) {
       /* next if name does not match */
       if(name && strcmp(factory->name, name))
 	continue;
@@ -207,7 +250,7 @@ librdf_get_serializer_factory(librdf_world *world,
         if(!factory->type_uri)
           continue;
         
-        if(librdf_uri_equals(factory->type_uri, type_uri))
+        if(!librdf_uri_equals(factory->type_uri, type_uri))
           continue;
       }
 
@@ -223,6 +266,39 @@ librdf_get_serializer_factory(librdf_world *world,
 }
 
 
+/**
+ * librdf_serializer_enumerate:
+ * @world: redland world object
+ * @counter: index into the list of serializers
+ * @name: pointer to store the name of the serializer (or NULL)
+ * @label: pointer to store syntax readable label (or NULL)
+ *
+ * Get information on serializers.
+ * 
+ * Return value: non 0 on failure of if counter is out of range
+ **/
+int
+librdf_serializer_enumerate(librdf_world* world,
+                        const unsigned int counter,
+                        const char **name, const char **label)
+{
+  librdf_serializer_factory *factory;
+  
+  librdf_world_open(world);
+
+  factory=(librdf_serializer_factory*)raptor_sequence_get_at(world->serializers,
+                                                         counter);
+  if(!factory)
+    return 1;
+  
+  if(name)
+    *name=factory->name;
+  if(label)
+    *label=factory->label;
+  return 0;
+}
+
+
 /**
  * librdf_new_serializer:
  * @world: redland world object
@@ -241,6 +317,8 @@ librdf_new_serializer(librdf_world *world,
 {
   librdf_serializer_factory* factory;
 
+  librdf_world_open(world);
+
   factory=librdf_get_serializer_factory(world, name, mime_type, type_uri);
   if(!factory)
     return NULL;
@@ -264,6 +342,8 @@ librdf_new_serializer_from_factory(librdf_world *world,
 {
   librdf_serializer* d;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(factory, librdf_serializer_factory, NULL);
 
   d=(librdf_serializer*)LIBRDF_CALLOC(librdf_serializer, 1, sizeof(librdf_serializer));
@@ -335,6 +415,32 @@ librdf_serializer_serialize_model(librdf_serializer* serializer,
 }
 
 
+/**
+ * librdf_serializer_serialize_stream_to_file_handle:
+ * @serializer: the serializer
+ * @handle: file handle to serialize to
+ * @base_uri: the base URI to use (or NULL)
+ * @stream: the #librdf_stream model to use
+ *
+ * Write a #librdf_stream to a FILE*.
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+librdf_serializer_serialize_stream_to_file_handle(librdf_serializer* serializer,
+                                                  FILE *handle, 
+                                                  librdf_uri* base_uri,
+                                                  librdf_stream* stream) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(handle, FILE*, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, 1);
+
+  return serializer->factory->serialize_stream_to_file_handle(serializer->context,
+                                                              handle, base_uri, stream);
+}
+
+
 /**
  * librdf_serializer_serialize_model_to_file_handle:
  * @serializer: the serializer
@@ -361,6 +467,45 @@ librdf_serializer_serialize_model_to_file_handle(librdf_serializer* serializer,
 }
 
 
+/**
+ * librdf_serializer_serialize_stream_to_file:
+ * @serializer: the serializer
+ * @name: filename to serialize to
+ * @base_uri: the base URI to use (or NULL)
+ * @stream: the #librdf_stream stream to use
+ *
+ * Write a #librdf_stream to a file.
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+librdf_serializer_serialize_stream_to_file(librdf_serializer* serializer,
+                                           const char *name, 
+                                           librdf_uri* base_uri,
+                                           librdf_stream* stream) 
+{
+  FILE* fh;
+  int status;
+  
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, string, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, 1);
+
+  fh=fopen(name, "w+");
+  if(!fh) {
+    librdf_log(serializer->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER,
+               NULL, "failed to open file '%s' for writing - %s",
+               name, strerror(errno));
+    return 1;
+  }
+  
+  status=librdf_serializer_serialize_stream_to_file_handle(serializer, fh, 
+                                                           base_uri, stream);
+  fclose(fh);
+  return status;
+}
+
+
 /**
  * librdf_serializer_serialize_model_to_file:
  * @serializer: the serializer
@@ -386,8 +531,13 @@ librdf_serializer_serialize_model_to_file(librdf_serializer* serializer,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
 
   fh=fopen(name, "w+");
-  if(!fh)
+  if(!fh) {
+    librdf_log(serializer->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER,
+               NULL, "failed to open file '%s' for writing - %s",
+               name, strerror(errno));
     return 1;
+  }
+  
   status=librdf_serializer_serialize_model_to_file_handle(serializer, fh, 
                                                           base_uri, model);
   fclose(fh);
@@ -395,6 +545,36 @@ librdf_serializer_serialize_model_to_file(librdf_serializer* serializer,
 }
 
 
+/**
+ * librdf_serializer_serialize_stream_to_counted_string:
+ * @serializer: the serializer
+ * @base_uri: the base URI to use (or NULL)
+ * @stream: the #librdf_stream stream to use
+ * @length_p: pointer to store length or NULL
+ *
+ * Write a #librdf_stream to a counted string.
+ * 
+ * Return value: non 0 on failure
+ **/
+unsigned char*
+librdf_serializer_serialize_stream_to_counted_string(librdf_serializer* serializer,
+                                                     librdf_uri* base_uri,
+                                                     librdf_stream* stream,
+                                                     size_t* length_p) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, NULL);
+
+  if(length_p)
+    *length_p=0;
+  
+  return serializer->factory->serialize_stream_to_counted_string(serializer->context,
+                                                                 base_uri,
+                                                                 stream,
+                                                                 length_p);
+}
+
+
 /**
  * librdf_serializer_serialize_model_to_counted_string:
  * @serializer: the serializer
@@ -424,6 +604,31 @@ librdf_serializer_serialize_model_to_counted_string(librdf_serializer* serialize
 }
 
 
+/**
+ * librdf_serializer_serialize_stream_to_string:
+ * @serializer: the serializer
+ * @base_uri: the base URI to use (or NULL)
+ * @stream: the #librdf_stream stream to use
+ *
+ * Write a #librdf_stream to a string.
+ * 
+ * Return value: NULL on failure
+ **/
+unsigned char*
+librdf_serializer_serialize_stream_to_string(librdf_serializer* serializer,
+                                             librdf_uri* base_uri,
+                                             librdf_stream* stream) 
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, NULL);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, NULL);
+
+  return serializer->factory->serialize_stream_to_counted_string(serializer->context,
+                                                                 base_uri,
+                                                                 stream,
+                                                                 NULL);
+}
+
+
 /**
  * librdf_serializer_serialize_model_to_string:
  * @serializer: the serializer
@@ -448,6 +653,33 @@ librdf_serializer_serialize_model_to_string(librdf_serializer* serializer,
 }
 
 
+/**
+ * librdf_serializer_serialize_stream_to_iostream:
+ * @serializer: the serializer
+ * @base_uri: the base URI to use (or NULL)
+ * @stream: the #librdf_stream stream to use
+ * @iostr: the #raptor_iostream to write to
+ *
+ * Write a #librdf_stream to a #raptor_iostream.
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+librdf_serializer_serialize_stream_to_iostream(librdf_serializer* serializer,
+                                              librdf_uri* base_uri,
+                                              librdf_stream *stream,
+                                              raptor_iostream* iostr)
+{
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(stream, librdf_stream, 1);
+
+  return serializer->factory->serialize_stream_to_iostream(serializer->context,
+                                                           base_uri, stream,
+                                                           iostr);
+}
+
+
+
 /**
  * librdf_serializer_serialize_model_to_iostream:
  * @serializer: the serializer
@@ -475,34 +707,6 @@ librdf_serializer_serialize_model_to_iostream(librdf_serializer* serializer,
 
 
 
-/**
- * librdf_init_serializer:
- * @world: redland world object
- *
- * INTERNAL - Initialise the librdf_serializer class.
- *
- **/
-void
-librdf_init_serializer(librdf_world *world) 
-{
-  librdf_serializer_raptor_constructor(world);
-}
-
-
-/**
- * librdf_finish_serializer:
- * @world: redland world object
- *
- * INTERNAL - Terminate the librdf_serializer class.
- *
- **/
-void
-librdf_finish_serializer(librdf_world *world) 
-{
-  librdf_delete_serializer_factories(world);
-}
-
-
 /**
  * librdf_serializer_set_error:
  * @serializer: the serializer
@@ -620,6 +824,14 @@ librdf_serializer_set_namespace(librdf_serializer* serializer,
 
 #ifdef STANDALONE
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 /* one more prototype */
 int main(int argc, char *argv[]);
 
@@ -658,14 +870,20 @@ log_handler(void *user_data, librdf_log_message *message)
 #define EXPECTED_ERRORS 3
 #define EXPECTED_WARNINGS 0
 
+#define SYNTAX_TYPE "ntriples"
+#define SYNTAX_CONTENT \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/creator> \"Dave Beckett\" .\n" \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/description> \"The generic home page of Dave Beckett.\" .\n" \
+"<http://purl.org/net/dajobe/> <http://purl.org/dc/elements/1.1/title> \"Dave Beckett's Home Page\" . \n"
+
 
 int
 main(int argc, char *argv[]) 
 {
-  char *program=argv[0];
-  char *test_serializer_types[]={"rdfxml", "ntriples", NULL};
+  const char *program=librdf_basename((const char*)argv[0]);
+  const char *test_serializer_types[]={"rdfxml", "ntriples", NULL};
   int i;
-  char *type;
+  const char *type;
   unsigned char *string;
   size_t string_length;
   librdf_world *world;
@@ -674,6 +892,10 @@ main(int argc, char *argv[])
   librdf_uri* base_uri;
   librdf_statement* statement;
   librdf_serializer* serializer;
+  librdf_parser* parser;
+  librdf_stream* stream;
+  FILE *fh;
+  struct stat st_buf;
 
   world=librdf_new_world();
   librdf_world_open(world);
@@ -729,9 +951,9 @@ main(int argc, char *argv[])
   string=librdf_serializer_serialize_model_to_counted_string(serializer,
                                                              base_uri, model,
                                                              &string_length);
-#define EXPECTED_BAD_STRING_LENGTH 346
+#define EXPECTED_BAD_STRING_LENGTH 382
   if(string_length != EXPECTED_BAD_STRING_LENGTH) {
-    fprintf(stderr, "%s: Serialising to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
+    fprintf(stderr, "%s: Serialising model to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
             (int)string_length, EXPECTED_BAD_STRING_LENGTH);
     return 1;
   }
@@ -739,12 +961,10 @@ main(int argc, char *argv[])
   if(string)
     free(string);
 
-  librdf_free_serializer(serializer);
-
-  librdf_free_uri(base_uri);
+  librdf_free_uri(base_uri); base_uri=NULL;
+  librdf_free_model(model); model=NULL;
+  librdf_free_storage(storage); storage=NULL;
   
-  librdf_free_model(model);
-  librdf_free_storage(storage);
 
   if(LogData.errors != EXPECTED_ERRORS) {
     fprintf(stderr, "%s: Serialising to RDF/XML returned %d errors, expected %d\n", program,
@@ -758,6 +978,80 @@ main(int argc, char *argv[])
     return 1;
   }
   
+
+  /* Good model to serialize */
+  storage=librdf_new_storage(world, NULL, NULL, NULL);
+  model=librdf_new_model(world, storage, NULL);
+
+  parser=librdf_new_parser(world, SYNTAX_TYPE, NULL, NULL);
+  if(!parser) {
+    fprintf(stderr, "%s: Failed to create new parser type %s\n", program, 
+            SYNTAX_TYPE);
+    return 1;
+  }
+
+  fprintf(stderr, "%s: Adding %s string content\n", program, SYNTAX_TYPE);
+  if(librdf_parser_parse_string_into_model(parser, 
+                                           (const unsigned char*)SYNTAX_CONTENT,
+                                           NULL /* no base URI*/, 
+                                           model)) {
+    fprintf(stderr, "%s: Failed to parse RDF from %s string into model\n", 
+            SYNTAX_TYPE, program);
+    return 1;
+  }
+  librdf_free_parser(parser);
+  
+
+  fprintf(stderr, "%s: Serializing stream to a string\n", program);
+
+  stream=librdf_model_as_stream(model);
+  string_length=0;
+  string=librdf_serializer_serialize_stream_to_counted_string(serializer,
+                                                              NULL, stream,
+                                                              &string_length);
+#define EXPECTED_GOOD_STRING_LENGTH 668
+  if(string_length != EXPECTED_GOOD_STRING_LENGTH) {
+    fprintf(stderr, "%s: Serialising stream to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
+            (int)string_length, EXPECTED_GOOD_STRING_LENGTH);
+    return 1;
+  }
+  librdf_free_stream(stream);
+
+  if(string)
+    free(string);
+
+
+  fprintf(stderr, "%s: Serializing stream to a file handle\n", program);
+
+  stream=librdf_model_as_stream(model);
+
+#define FILENAME "test.rdf"
+  fh=fopen(FILENAME, "w");
+  if(!fh) {
+    fprintf(stderr, "%s: Failed to fopen for writing '%s' - %s\n",
+            program, FILENAME, strerror(errno));
+    return 1;
+  }
+  librdf_serializer_serialize_stream_to_file_handle(serializer, fh, NULL, 
+                                                    stream);
+  fclose(fh);
+  stat(FILENAME, &st_buf);
+  
+  if((int)st_buf.st_size != EXPECTED_GOOD_STRING_LENGTH) {
+    fprintf(stderr, "%s: Serialising stream to file handle returned file '%s' of size %d bytes, expected %d\n", program, FILENAME, (int)st_buf.st_size, 
+            EXPECTED_GOOD_STRING_LENGTH);
+    return 1;
+  }
+  unlink(FILENAME);
+  
+  librdf_free_stream(stream);
+
+
+  librdf_free_serializer(serializer); serializer=NULL;
+  librdf_free_model(model); model=NULL;
+  librdf_free_storage(storage); storage=NULL;
+
+
   librdf_free_world(world);
   
   /* keep gcc -Wall happy */
