@@ -2,7 +2,7 @@
  *
  * rdf_serializer_raptor.c - RDF Serializers via Raptor
  *
- * $Id: rdf_serializer_raptor.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_serializer_raptor.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -176,7 +176,6 @@ librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
   } else if(librdf_node_is_resource(subject)) {
     rstatement.subject=(raptor_uri*)librdf_node_get_uri(subject);
     rstatement.subject_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
-    /* or  RAPTOR_IDENTIFIER_TYPE_ORDINAL ? */
   } else {
     librdf_log(statement->world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
@@ -203,7 +202,6 @@ librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
       rstatement.object=librdf_node_get_literal_value(object);
       rstatement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
       
-      /* or RAPTOR_IDENTIFIER_TYPE_XML_LITERAL ? */
       rstatement.object_literal_language=(const unsigned char*)librdf_node_get_literal_value_language(object);
       rstatement.object_literal_datatype=(raptor_uri*)librdf_node_get_literal_value_datatype_uri(object);
       break;
@@ -214,7 +212,6 @@ librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
       break;
 
     case LIBRDF_NODE_TYPE_RESOURCE:
-      /* or  RAPTOR_IDENTIFIER_TYPE_ORDINAL ? */
       rstatement.object=librdf_node_get_uri(object);
       rstatement.object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
       break;
@@ -255,16 +252,14 @@ librdf_serializer_raptor_warning_handler(void *data, raptor_locator *locator,
 
 
 static int
-librdf_serializer_raptor_serialize_model_to_file_handle(void *context,
-                                                        FILE *handle, 
-                                                        librdf_uri* base_uri,
-                                                        librdf_model *model) 
+librdf_serializer_raptor_serialize_stream_to_file_handle(void *context,
+                                                         FILE *handle, 
+                                                         librdf_uri* base_uri,
+                                                         librdf_stream *stream) 
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
   int rc=0;
-  librdf_stream *stream;
 
-  stream=librdf_model_as_stream(model);
   if(!stream)
     return 1;
 
@@ -287,7 +282,6 @@ librdf_serializer_raptor_serialize_model_to_file_handle(void *context,
                                                  statement);
     librdf_stream_next(stream);
   }
-  librdf_free_stream(stream);
   raptor_serialize_end(scontext->rdf_serializer);
 
   return 0;
@@ -295,10 +289,31 @@ librdf_serializer_raptor_serialize_model_to_file_handle(void *context,
 
 
 
+static int
+librdf_serializer_raptor_serialize_model_to_file_handle(void *context,
+                                                        FILE *handle, 
+                                                        librdf_uri* base_uri,
+                                                        librdf_model *model) 
+{
+  int rc;
+  librdf_stream *stream;
+
+  stream=librdf_model_as_stream(model);
+  if(!stream)
+    return 1;
+  rc=librdf_serializer_raptor_serialize_stream_to_file_handle(context, handle,
+                                                              base_uri, stream);
+  librdf_free_stream(stream);
+
+  return rc;
+}
+
+
+
 static unsigned char*
-librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
+librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
                                                            librdf_uri* base_uri,
-                                                           librdf_model *model,
+                                                           librdf_stream *stream,
                                                            size_t* length_p)
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
@@ -306,9 +321,7 @@ librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
   unsigned char *string=NULL;
   size_t string_length=0;
   int rc=0;
-  librdf_stream *stream;
 
-  stream=librdf_model_as_stream(model);
   if(!stream)
     return NULL;
 
@@ -324,7 +337,6 @@ librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
                             (raptor_uri*)base_uri, iostr);
 
   if(rc) {
-    librdf_free_stream(stream);    
     raptor_free_iostream(iostr);
     return NULL;
   }
@@ -342,7 +354,6 @@ librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
                                                  statement);
     librdf_stream_next(stream);
   }
-  librdf_free_stream(stream);
   raptor_serialize_end(scontext->rdf_serializer);
 
   if(length_p)
@@ -352,20 +363,41 @@ librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
 }
 
 
+static unsigned char*
+librdf_serializer_raptor_serialize_model_to_counted_string(void *context,
+                                                           librdf_uri* base_uri,
+                                                           librdf_model *model,
+                                                           size_t* length_p)
+{
+  unsigned char *string=NULL;
+  librdf_stream *stream;
+
+  stream=librdf_model_as_stream(model);
+  if(!stream)
+    return NULL;
+
+  string=librdf_serializer_raptor_serialize_stream_to_counted_string(context,
+                                                                     base_uri,
+                                                                     stream,
+                                                                     length_p);
+  librdf_free_stream(stream);
+  
+  return string;
+}
+
+
 static int
-librdf_serializer_raptor_serialize_model_to_iostream(void *context,
+librdf_serializer_raptor_serialize_stream_to_iostream(void *context,
                                                      librdf_uri* base_uri,
-                                                     librdf_model *model,
+                                                     librdf_stream *stream,
                                                      raptor_iostream* iostr)
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
   int rc=0;
-  librdf_stream *stream;
   
   if(!iostr)
     return 1;
   
-  stream=librdf_model_as_stream(model);
   if(!stream)
     return 1;
 
@@ -374,7 +406,6 @@ librdf_serializer_raptor_serialize_model_to_iostream(void *context,
                             (raptor_uri*)base_uri, iostr);
 
   if(rc) {
-    librdf_free_stream(stream);    
     raptor_free_iostream(iostr);
     return 1;
   }
@@ -392,13 +423,36 @@ librdf_serializer_raptor_serialize_model_to_iostream(void *context,
                                                  statement);
     librdf_stream_next(stream);
   }
-  librdf_free_stream(stream);
   raptor_serialize_end(scontext->rdf_serializer);
 
   return 0;
 }
 
 
+static int
+librdf_serializer_raptor_serialize_model_to_iostream(void *context,
+                                                     librdf_uri* base_uri,
+                                                     librdf_model *model,
+                                                     raptor_iostream* iostr)
+{
+  int rc=0;
+  librdf_stream *stream;
+  
+  if(!iostr)
+    return 1;
+  
+  stream=librdf_model_as_stream(model);
+  if(!stream)
+    return 1;
+  rc=librdf_serializer_raptor_serialize_stream_to_iostream(context,
+                                                           base_uri,
+                                                           stream, iostr);
+  librdf_free_stream(stream);
+
+  return rc;
+}
+
+
 /**
  * librdf_serializer_raptor_register_factory:
  * @factory: factory
@@ -418,8 +472,11 @@ librdf_serializer_raptor_register_factory(librdf_serializer_factory *factory)
   factory->set_feature = librdf_serializer_raptor_set_feature;
   factory->set_namespace = librdf_serializer_raptor_set_namespace;
 
+  factory->serialize_stream_to_file_handle= librdf_serializer_raptor_serialize_stream_to_file_handle;
   factory->serialize_model_to_file_handle = librdf_serializer_raptor_serialize_model_to_file_handle;
+  factory->serialize_stream_to_counted_string = librdf_serializer_raptor_serialize_stream_to_counted_string;
   factory->serialize_model_to_counted_string = librdf_serializer_raptor_serialize_model_to_counted_string;
+  factory->serialize_stream_to_iostream = librdf_serializer_raptor_serialize_stream_to_iostream;
   factory->serialize_model_to_iostream = librdf_serializer_raptor_serialize_model_to_iostream;
 }
 
@@ -428,7 +485,7 @@ librdf_serializer_raptor_register_factory(librdf_serializer_factory *factory)
  * librdf_serializer_raptor_constructor:
  * @world: redland world object
  *
- * Initialise the raptor RDF serializer module.
+ * INTERNAL - Initialise the serializer_raptor module.
  *
  **/
 void
@@ -439,19 +496,20 @@ librdf_serializer_raptor_constructor(librdf_world *world)
   /* enumerate from serializer 1, so the default serializer 0 is done last */
   for(i=1; 1; i++) {
     const char *syntax_name=NULL;
+    const char *syntax_label=NULL;
     const char *mime_type=NULL;
     const unsigned char *uri_string=NULL;
 
-    if(raptor_serializers_enumerate(i, &syntax_name, NULL, 
+    if(raptor_serializers_enumerate(i, &syntax_name, &syntax_label, 
                                     &mime_type, &uri_string)) {
       /* reached the end of the serializers, now register the default one */
       i=0;
-      raptor_serializers_enumerate(i, &syntax_name, NULL,
+      raptor_serializers_enumerate(i, &syntax_name, &syntax_label,
                                    &mime_type, &uri_string);
     }
     
-    librdf_serializer_register_factory(world, syntax_name, mime_type, 
-                                       uri_string,
+    librdf_serializer_register_factory(world, syntax_name, syntax_label,
+                                       mime_type, uri_string,
                                        &librdf_serializer_raptor_register_factory);
 
     if(!i) /* registered default serializer, end */
