@@ -2,7 +2,7 @@
  *
  * rdf_statement.c - RDF Triple (Statement) interface
  *
- * $Id: rdf_statement.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_statement.c 12180 2007-05-01 03:42:32Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -50,7 +50,7 @@
  * librdf_init_statement:
  * @world: redland world object
  *
- * INTERNAL - Initialise the librdf_statement module.
+ * INTERNAL - Initialise the statement module.
  *
  **/
 void
@@ -63,7 +63,7 @@ librdf_init_statement(librdf_world *world)
  * librdf_finish_statement:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_statement module.
+ * INTERNAL - Terminate the statement module.
  *
  **/
 void
@@ -83,8 +83,12 @@ librdf_finish_statement(librdf_world *world)
 librdf_statement*
 librdf_new_statement(librdf_world *world) 
 {
-  librdf_statement* new_statement=(librdf_statement*)LIBRDF_CALLOC(librdf_statement, 1, 
-                                                                   sizeof(librdf_statement));
+  librdf_statement* new_statement;
+
+  librdf_world_open(world);
+
+  new_statement=(librdf_statement*)LIBRDF_CALLOC(librdf_statement, 1, 
+                                                 sizeof(librdf_statement));
   if(!new_statement)
     return NULL;
 
@@ -163,6 +167,8 @@ librdf_new_statement_from_nodes(librdf_world *world,
 {
   librdf_statement* new_statement;
 
+  librdf_world_open(world);
+
   new_statement = librdf_new_statement(world);
   if(!new_statement) {
     if(subject)
@@ -197,6 +203,8 @@ librdf_new_statement_from_nodes(librdf_world *world,
 void
 librdf_statement_init(librdf_world *world, librdf_statement *statement)
 {
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
   statement->world=world;
@@ -388,16 +396,30 @@ librdf_statement_set_object(librdf_statement *statement, librdf_node *node)
  * librdf_statement_is_complete:
  * @statement: #librdf_statement object
  *
- * Check if statement has all subject, predicate, object fields present.
+ * Check if statement is a complete and legal RDF triple.
+ *
+ * Checks that all subject, predicate, object fields are present
+ * and they have the allowed node types.
  * 
- * Return value: non 0 if the statement has subject, predicate and object.
+ * Return value: non 0 if the statement is complete and legal
  **/
 int
 librdf_statement_is_complete(librdf_statement *statement)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 0);
 
-  return (statement->subject && statement->predicate && statement->object);
+  if(!statement->subject ||
+     (!librdf_node_is_resource(statement->subject) && 
+      !librdf_node_is_blank(statement->subject)))
+    return 0;
+
+  if(!statement->predicate || !librdf_node_is_resource(statement->predicate))
+     return 0;
+
+  if(!statement->object)
+    return 0;
+
+  return 1;
 }
 
 
@@ -886,19 +908,12 @@ main(int argc, char *argv[])
 {
   librdf_statement *statement, *statement2;
   int size, size2;
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   char *s, *buffer;
   librdf_world *world;
   
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
-
-  /* initialise dependent modules */
-  librdf_init_digest(world);
-  librdf_init_hash(world);
-  librdf_init_uri(world);
-  librdf_init_node(world);
-  librdf_init_statement(world);
+  librdf_world_open(world);
 
   fprintf(stderr, "%s: Creating statement\n", program);
   statement=librdf_new_statement(world);
@@ -947,13 +962,7 @@ main(int argc, char *argv[])
   librdf_free_statement(statement);
 
 
-  librdf_finish_statement(world);
-  librdf_finish_node(world);
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-  librdf_finish_digest(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
