@@ -2,7 +2,7 @@
  *
  * rdf_storage.c - RDF Storage (Triple store) interface
  *
- * $Id: rdf_storage.c 8543 2006-01-26 06:34:54Z cmdjb $
+ * $Id: rdf_storage.c 12180 2007-05-01 03:42:32Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -47,10 +47,6 @@
 
 #ifndef STANDALONE
 
-/* prototypes for helper functions */
-static void librdf_delete_storage_factories(void);
-
-
 /* prototypes for functions implementing get_sources, arcs, targets
  * librdf_iterator via conversion from a librdf_stream of librdf_statement
  */
@@ -67,7 +63,7 @@ static librdf_iterator* librdf_storage_node_stream_to_node_create(librdf_storage
  * librdf_init_storage:
  * @world: redland world object
  *
- * INTERNAL - Initialise the librdf_storage module.
+ * INTERNAL - Initialise the storage module.
  * 
  * Initialises and registers all
  * compiled storage modules.  Must be called before using any of the storage
@@ -76,6 +72,15 @@ static librdf_iterator* librdf_storage_node_stream_to_node_create(librdf_storage
 void
 librdf_init_storage(librdf_world *world)
 {
+  /* Always have storage memory - must always be the default storage */
+  librdf_init_storage_list(world);
+
+  /* Always have storage list, hashes, file implementations available */
+  librdf_init_storage_hashes(world);
+#ifdef STORAGE_FILE
+  librdf_init_storage_file(world);
+#endif
+
 #ifdef STORAGE_MYSQL
   librdf_init_storage_mysql(world);
 #endif
@@ -88,13 +93,6 @@ librdf_init_storage(librdf_world *world)
 #ifdef STORAGE_SQLITE
   librdf_init_storage_sqlite(world);
 #endif
-#ifdef STORAGE_FILE
-  librdf_init_storage_file(world);
-#endif
-  /* Always have storage list, hashes, file implementations available */
-  librdf_init_storage_hashes(world);
-  /* Always have storage memory must always be available */
-  librdf_init_storage_list(world);
 }
 
 
@@ -102,43 +100,31 @@ librdf_init_storage(librdf_world *world)
  * librdf_finish_storage:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_storage module.
+ * INTERNAL - Terminate the storage module.
  *
  **/
 void
 librdf_finish_storage(librdf_world *world) 
 {
-  librdf_delete_storage_factories();
+  raptor_free_sequence(world->storages);
+  world->storages=NULL;
 }
 
 
-/* statics */
-
-/* list of storage factories */
-static librdf_storage_factory* storages;
-
 
 /* helper functions */
 
 
-/*
- * librdf_delete_storage_factories - helper function to delete all the registered storage factories
- */
 static void
-librdf_delete_storage_factories(void)
+librdf_free_storage_factory(librdf_storage_factory* factory)
 {
-  librdf_storage_factory *factory, *next;
-  
-  for(factory=storages; factory; factory=next) {
-    next=factory->next;
-    LIBRDF_FREE(librdf_storage_factory, factory->name);
-    LIBRDF_FREE(librdf_storage_factory, factory->label);
-    LIBRDF_FREE(librdf_storage_factory, factory);
-  }
-  storages=NULL;
+  LIBRDF_FREE(librdf_storage_factory, factory->name);
+  LIBRDF_FREE(librdf_storage_factory, factory->label);
+  LIBRDF_FREE(librdf_storage_factory, factory);
 }
 
 
+
 /* class methods */
 
 /**
@@ -158,11 +144,28 @@ librdf_storage_register_factory(librdf_world* world,
   librdf_storage_factory *storage, *h;
   char *name_copy;
   char *label_copy;
-  
+  int i;
+
+  librdf_world_open(world);
+
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for storage %s\n", name);
 #endif
   
+  if(!world->storages)
+    world->storages=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_storage_factory, NULL);
+  
+  for(i=0;
+      (h=(librdf_storage_factory*)raptor_sequence_get_at(world->storages, i));
+      i++) {
+    if(!strcmp(h->name, name)) {
+      librdf_log(world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "storage %s already registered", h->name);
+      return;
+    }
+  }
+  
   storage=(librdf_storage_factory*)LIBRDF_CALLOC(librdf_storage_factory, 1,
                                                  sizeof(librdf_storage_factory));
   if(!storage)
@@ -175,18 +178,7 @@ librdf_storage_register_factory(librdf_world* world,
   }
   strcpy(name_copy, name);
   storage->name=name_copy;
-        
-  for(h = storages; h; h = h->next ) {
-    if(!strcmp(h->name, name_copy)) {
-      LIBRDF_FREE(cstring, name_copy); 
-      LIBRDF_FREE(librdf_storage, storage);
-      librdf_log(world,
-                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
-                 "storage %s already registered", h->name);
-      return;
-    }
-  }
-  
+
   label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
   if(!label_copy) {
     LIBRDF_FREE(librdf_storage, storage);
@@ -201,14 +193,14 @@ librdf_storage_register_factory(librdf_world* world,
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG3("%s has context size %d\n", name, storage->context_length);
 #endif
-  
-  storage->next = storages;
-  storages = storage;
+
+  raptor_sequence_push(world->storages, storage);
 }
 
 
 /**
  * librdf_get_storage_factory:
+ * @world: redland world object
  * @name: the factory name or NULL for the default factory
  *
  * Get a storage factory by name.
@@ -216,22 +208,27 @@ librdf_storage_register_factory(librdf_world* world,
  * Return value: the factory object or NULL if there is no such factory
  **/
 librdf_storage_factory*
-librdf_get_storage_factory (const char *name) 
+librdf_get_storage_factory(librdf_world* world, const char *name) 
 {
   librdf_storage_factory *factory;
 
+  librdf_world_open(world);
+
   /* return 1st storage if no particular one wanted - why? */
   if(!name) {
-    factory=storages;
+    factory=(librdf_storage_factory *)raptor_sequence_get_at(world->storages, 0);
     if(!factory) {
       LIBRDF_DEBUG1("No (default) storages registered\n");
       return NULL;
     }
   } else {
-    for(factory=storages; factory; factory=factory->next) {
-      if(!strcmp(factory->name, name)) {
+    int i;
+    
+    for(i=0;
+        (factory=(librdf_storage_factory*)raptor_sequence_get_at(world->storages, i));
+        i++) {
+      if(!strcmp(factory->name, name))
         break;
-      }
     }
     /* else FACTORY name not found */
     if(!factory) {
@@ -246,6 +243,7 @@ librdf_get_storage_factory (const char *name)
 
 /**
  * librdf_storage_enumerate:
+ * @world: redland world object
  * @counter: index into the list of storages
  * @name: pointer to store the name of the storage (or NULL)
  * @label: pointer to store syntax readable label (or NULL)
@@ -255,26 +253,24 @@ librdf_get_storage_factory (const char *name)
  * Return value: non 0 on failure of if counter is out of range
  **/
 int
-librdf_storage_enumerate(const unsigned int counter,
+librdf_storage_enumerate(librdf_world* world,
+                         const unsigned int counter,
                          const char **name, const char **label)
 {
-  unsigned int i;
-  librdf_storage_factory *factory=storages;
+  librdf_storage_factory *factory;
+  
+  librdf_world_open(world);
 
-  if(!factory || counter < 0)
+  factory=(librdf_storage_factory*)raptor_sequence_get_at(world->storages,
+                                                          counter);
+  if(!factory)
     return 1;
-
-  for(i=0; factory && i<=counter ; i++, factory=factory->next) {
-    if(i == counter) {
-      if(name)
-        *name=factory->name;
-      if(label)
-        *label=factory->label;
-      return 0;
-    }
-  }
-        
-  return 1;
+  
+  if(name)
+    *name=factory->name;
+  if(label)
+    *label=factory->label;
+  return 0;
 }
 
 
@@ -294,13 +290,16 @@ librdf_storage_enumerate(const unsigned int counter,
  *
  */
 librdf_storage*
-librdf_new_storage (librdf_world *world, 
-                    char *storage_name, char *name, 
-                    char *options_string) {
+librdf_new_storage(librdf_world *world, 
+                   const char *storage_name, const char *name, 
+                   const char *options_string)
+{
   librdf_storage_factory* factory;
   librdf_hash* options_hash;
   
-  factory=librdf_get_storage_factory(storage_name);
+  librdf_world_open(world);
+
+  factory=librdf_get_storage_factory(world, storage_name);
   if(!factory)
     return NULL;
 
@@ -337,13 +336,16 @@ librdf_new_storage (librdf_world *world,
  *
  */
 librdf_storage*
-librdf_new_storage_with_options (librdf_world *world, 
-                                 char *storage_name, char *name, 
-                                 librdf_hash *options) {
+librdf_new_storage_with_options(librdf_world *world, 
+                                const char *storage_name, const char *name, 
+                                librdf_hash *options)
+{
   librdf_storage_factory* factory;
   librdf_hash* options_hash;
   
-  factory=librdf_get_storage_factory(storage_name);
+  librdf_world_open(world);
+
+  factory=librdf_get_storage_factory(world, storage_name);
   if(!factory)
     return NULL;
 
@@ -439,12 +441,15 @@ librdf_new_storage_from_storage(librdf_storage* old_storage)
  *
  */
 librdf_storage*
-librdf_new_storage_from_factory (librdf_world *world,
-                                 librdf_storage_factory* factory,
-                                 char *name,
-                                 librdf_hash* options) {
+librdf_new_storage_from_factory(librdf_world *world,
+                                librdf_storage_factory* factory,
+                                const char *name,
+                                librdf_hash* options)
+{
   librdf_storage* storage;
 
+  librdf_world_open(world);
+
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(factory, librdf_storage_factory, NULL);
 
   if(!factory) {
@@ -515,7 +520,7 @@ librdf_storage_add_reference(librdf_storage *storage)
 void
 librdf_storage_remove_reference(librdf_storage *storage)
 {
-  storage->usage--;
+  librdf_free_storage(storage);
 }
 
 
@@ -588,7 +593,10 @@ librdf_storage_size(librdf_storage* storage)
  * If the statement already exists in the store, it is not added
  * unless Redland contexts are being used.
  *
- * Return value: non 0 on failure
+ * Enforces that the statement is legal for RDF - URI or blank subject,
+ * URI predicate and URI or blank or literal object (i.e. anything).
+ *
+ * Return value: non 0 on failure, <0 on error, >0 if statement was illegal
  **/
 int
 librdf_storage_add_statement(librdf_storage* storage,
@@ -597,9 +605,21 @@ librdf_storage_add_statement(librdf_storage* storage,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
 
+  /* subject can be a URI or blank node */
+  if(!librdf_node_is_resource(statement->subject) &&
+     !librdf_node_is_blank(statement->subject))
+    return 1;
+  
+  /* subject can only be a URI */
+  if(!librdf_node_is_resource(statement->predicate))
+     return 1;
+
+  /* object can be any node - no check needed */
+
   if(storage->factory->add_statement)
     return storage->factory->add_statement(storage, statement);
-  return 1;
+
+  return -1;
 }
 
 
@@ -630,8 +650,12 @@ librdf_storage_add_statements(librdf_storage* storage,
   while(!librdf_stream_end(statement_stream)) {
     librdf_statement* statement=librdf_stream_get_object(statement_stream);
 
-    if(statement)
+    if(statement) {
       status=librdf_storage_add_statement(storage, statement);
+      if(status > 0)
+        /* just skip illegal statements */
+        status=0;
+    }
     else
       status=1;
 
@@ -674,7 +698,7 @@ librdf_storage_remove_statement(librdf_storage* storage,
  *
  * Test if a given statement is present in the storage.
  *
- * Return value: non 0 if the storage contains the statement
+ * Return value: non 0 if the storage contains the statement (>0 if illegal statement)
  **/
 int
 librdf_storage_contains_statement(librdf_storage* storage,
@@ -683,6 +707,19 @@ librdf_storage_contains_statement(librdf_storage* storage,
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
 
+  /* subject can be a URI or blank node */
+  if(!statement->subject ||
+     (!librdf_node_is_resource(statement->subject) &&
+      !librdf_node_is_blank(statement->subject)))
+    return 1;
+  
+  /* subject can only be a URI */
+  if(!statement->predicate || !librdf_node_is_resource(statement->predicate))
+     return 1;
+
+  if(!statement->object)
+    return 1;
+
   return storage->factory->contains_statement(storage, statement);
 }
 
@@ -770,6 +807,8 @@ typedef struct {
   librdf_stream *stream;
   librdf_statement *partial_statement;
   librdf_statement_part want;
+  librdf_node *object_node;
+  librdf_node *context_node;
 } librdf_storage_stream_to_node_iterator_context;
 
 
@@ -787,6 +826,15 @@ librdf_storage_stream_to_node_iterator_next_method(void* iterator)
 {
   librdf_storage_stream_to_node_iterator_context* context=(librdf_storage_stream_to_node_iterator_context*)iterator;
 
+  if(context->object_node) {
+    librdf_free_node(context->object_node);
+    context->object_node=NULL;
+  }
+  if(context->context_node) {
+    librdf_free_node(context->context_node);
+    context->context_node=NULL;
+  }
+
   return librdf_stream_next(context->stream);
 }
 
@@ -804,30 +852,38 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
   switch(flags) {
     case LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT:
 
-      switch(context->want) {
-        case LIBRDF_STATEMENT_SUBJECT: /* SOURCES (subjects) */
-          node=librdf_statement_get_subject(statement);
-          break;
+      if(!context->object_node) {
+        switch(context->want) {
+          case LIBRDF_STATEMENT_SUBJECT: /* SOURCES (subjects) */
+            node=librdf_statement_get_subject(statement);
+            break;
+            
+          case LIBRDF_STATEMENT_PREDICATE: /* ARCS (predicates) */
+            node=librdf_statement_get_predicate(statement);
+            break;
+            
+          case LIBRDF_STATEMENT_OBJECT: /* TARGETS (objects) */
+            node=librdf_statement_get_object(statement);
+            break;
 
-        case LIBRDF_STATEMENT_PREDICATE: /* ARCS (predicates) */
-          node=librdf_statement_get_predicate(statement);
-          break;
-
-        case LIBRDF_STATEMENT_OBJECT: /* TARGETS (objects) */
-          node=librdf_statement_get_object(statement);
-          break;
-
-        case LIBRDF_STATEMENT_ALL:
-        default: /* error */
-          librdf_log(statement->world,
-                     0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
-                     "Unknown statement part %d", context->want);
-          node=NULL;
+          case LIBRDF_STATEMENT_ALL:
+            default: /* error */
+              librdf_log(statement->world,
+                         0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                         "Unknown statement part %d", context->want);
+              node=NULL;
+        }
+        context->object_node=librdf_new_node_from_node(node);
       }
+      node=context->object_node;
       break;
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
-      node=(librdf_node*)librdf_stream_get_context(context->stream);
+      if(!context->context_node) {
+        node=(librdf_node*)librdf_stream_get_context(context->stream);
+        context->context_node=node ? librdf_new_node_from_node(node) : NULL;
+      }
+      node=context->context_node;
       break;
       
     default:
@@ -847,20 +903,20 @@ librdf_storage_stream_to_node_iterator_finished(void* iterator)
   librdf_storage_stream_to_node_iterator_context* context=(librdf_storage_stream_to_node_iterator_context*)iterator;
   librdf_statement *partial_statement=context->partial_statement;
 
-  /* make sure librdf_free_statement() doesn't free anything here */
-  if(partial_statement) {
-    librdf_statement_set_subject(partial_statement, NULL);
-    librdf_statement_set_predicate(partial_statement, NULL);
-    librdf_statement_set_object(partial_statement, NULL);
-
+  if(partial_statement)
     librdf_free_statement(partial_statement);
-  }
 
   if(context->stream)
     librdf_free_stream(context->stream);
 
   if(context->storage)
     librdf_storage_remove_reference(context->storage);
+
+  if(context->object_node)
+    librdf_free_node(context->object_node);
+
+  if(context->context_node)
+    librdf_free_node(context->context_node);
   
   LIBRDF_FREE(librdf_storage_stream_to_node_iterator_context, context);
 }
@@ -901,6 +957,12 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
     return NULL;
   }
 
+
+  if(node1)
+    node1=librdf_new_node_from_node(node1);
+  if(node2)
+    node2=librdf_new_node_from_node(node2);
+  
   switch(want) {
     case LIBRDF_STATEMENT_SUBJECT:
       librdf_statement_set_predicate(partial_statement, node1);
@@ -917,6 +979,8 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
 
     case LIBRDF_STATEMENT_ALL:
     default:
+      librdf_free_node(node1);
+      librdf_free_node(node2);
       librdf_free_statement(partial_statement);
       librdf_log(storage->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
@@ -1265,8 +1329,6 @@ librdf_storage_context_remove_statement(librdf_storage* storage,
  *
  * Remove statements from a storage with the given context.
  * 
- * If @context is NULL, this is equivalent to librdf_storage_remove_statements
- *
  * Return value: Non 0 on failure
  **/
 int
@@ -1546,6 +1608,98 @@ librdf_storage_find_statements_with_options(librdf_storage* storage,
 }
 
 
+
+/**
+ * librdf_storage_transaction_start:
+ * @storage: the storage object
+ * 
+ * Start a transaction
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+librdf_storage_transaction_start(librdf_storage* storage) 
+{
+  if(storage->factory->transaction_start)
+    return storage->factory->transaction_start(storage);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_storage_transaction_start_with_handle:
+ * @storage: the storage object
+ * @handle: the transaction object
+ * 
+ * Start a transaction using an existing external transaction object.
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+librdf_storage_transaction_start_with_handle(librdf_storage* storage, void* handle)
+{
+  if(storage->factory->transaction_start_with_handle)
+    return storage->factory->transaction_start_with_handle(storage, handle);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_storage_transaction_commit:
+ * @storage: the storage object
+ * 
+ * Commit a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+int
+librdf_storage_transaction_commit(librdf_storage* storage) 
+{
+  if(storage->factory->transaction_commit)
+    return storage->factory->transaction_commit(storage);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_storage_transaction_rollback:
+ * @storage: the storage object
+ * 
+ * Rollback a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+int
+librdf_storage_transaction_rollback(librdf_storage* storage) 
+{
+  if(storage->factory->transaction_rollback)
+    return storage->factory->transaction_rollback(storage);
+  else
+    return 1;
+}
+
+
+/**
+ * librdf_storage_transaction_get_handle:
+ * @storage: the storage object
+ * 
+ * Get the current transaction handle.
+ * 
+ * Return value: non-0 on failure 
+ **/
+void*
+librdf_storage_transaction_get_handle(librdf_storage* storage) 
+{
+  if(storage->factory->transaction_get_handle)
+    return storage->factory->transaction_get_handle(storage);
+  else
+    return NULL;
+}
+
+
 #endif
 
 
@@ -1562,15 +1716,11 @@ int
 main(int argc, char *argv[]) 
 {
   librdf_storage* storage;
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   librdf_world *world;
   
   world=librdf_new_world();
-  
-  /* initialise hash, model and storage modules */
-  librdf_init_hash(world);
-  librdf_init_storage(world);
-  librdf_init_model(world);
+  librdf_world_open(world);
   
   fprintf(stdout, "%s: Creating storage\n", program);
   storage=librdf_new_storage(world, NULL, "test", NULL);
@@ -1596,12 +1746,7 @@ main(int argc, char *argv[])
   librdf_free_storage(storage);
   
 
-  /* finish model and storage modules */
-  librdf_finish_model(world);
-  librdf_finish_storage(world);
-  librdf_finish_hash(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
