@@ -2,7 +2,7 @@
  *
  * rdf_storage.h - RDF Storage Factory and Storage interfaces and definitions
  *
- * $Id: rdf_storage.h 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage.h 11706 2006-12-05 04:20:01Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -41,18 +41,18 @@ REDLAND_API
 void librdf_storage_register_factory(librdf_world *world, const char *name, const char *label, void (*factory) (librdf_storage_factory*));
 
 REDLAND_API
-int librdf_storage_enumerate(const unsigned int counter, const char **name, const char **label);
+int librdf_storage_enumerate(librdf_world* world, const unsigned int counter, const char **name, const char **label);
 
 
 /* constructor */
 REDLAND_API
-librdf_storage* librdf_new_storage(librdf_world *world, char *storage_name, char *name, char *options_string);
+librdf_storage* librdf_new_storage(librdf_world *world, const char *storage_name, const char *name, const char *options_string);
 REDLAND_API
-librdf_storage* librdf_new_storage_with_options(librdf_world *world, char *storage_name, char *name, librdf_hash *options);
+librdf_storage* librdf_new_storage_with_options(librdf_world *world, const char *storage_name, const char *name, librdf_hash *options);
 REDLAND_API
 librdf_storage* librdf_new_storage_from_storage(librdf_storage* old_storage);
 REDLAND_API
-librdf_storage* librdf_new_storage_from_factory(librdf_world *world, librdf_storage_factory* factory, char *name, librdf_hash* options);
+librdf_storage* librdf_new_storage_from_factory(librdf_world *world, librdf_storage_factory* factory, const char *name, librdf_hash* options);
 
 /* destructor */
 REDLAND_API
@@ -140,6 +140,17 @@ librdf_node* librdf_storage_get_feature(librdf_storage* storage, librdf_uri* fea
 REDLAND_API
 int librdf_storage_set_feature(librdf_storage* storage, librdf_uri* feature, librdf_node* value);
 
+REDLAND_API
+int librdf_storage_transaction_start(librdf_storage* storage);
+REDLAND_API
+int librdf_storage_transaction_start_with_handle(librdf_storage* storage, void* handle);
+REDLAND_API
+int librdf_storage_transaction_commit(librdf_storage* storage);
+REDLAND_API
+int librdf_storage_transaction_rollback(librdf_storage* storage);
+REDLAND_API
+void* librdf_storage_transaction_get_handle(librdf_storage* storage);
+
 #ifdef __cplusplus
 }
 #endif
