@@ -2,7 +2,7 @@
  *
  * rdf_storage_file.c - RDF Storage in a file, using an in-memory store
  *
- * $Id: rdf_storage_file.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage_file.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -64,7 +64,7 @@ typedef struct
 
 
 /* prototypes for local functions */
-static int librdf_storage_file_init(librdf_storage* storage, char *name, librdf_hash* options);
+static int librdf_storage_file_init(librdf_storage* storage, const char *name, librdf_hash* options);
 static int librdf_storage_file_open(librdf_storage* storage, librdf_model* model);
 static int librdf_storage_file_close(librdf_storage* storage);
 static int librdf_storage_file_size(librdf_storage* storage);
@@ -83,7 +83,7 @@ static void librdf_storage_file_register_factory(librdf_storage_factory *factory
 
 /* functions implementing storage api */
 static int
-librdf_storage_file_init(librdf_storage* storage, char *name,
+librdf_storage_file_init(librdf_storage* storage, const char *name,
                          librdf_hash* options)
 {
   librdf_storage_file_context *context=(librdf_storage_file_context*)storage->context;
@@ -127,6 +127,10 @@ librdf_storage_file_init(librdf_storage* storage, char *name,
     librdf_free_parser(parser);
   }
 
+  /* no more options, might as well free them now */
+  if(options)
+    librdf_free_hash(options);
+
   context->changed=0;
   
   return 0;
@@ -353,6 +357,12 @@ librdf_storage_file_register_factory(librdf_storage_factory *factory)
 }
 
 
+/**
+ * librdf_init_storage_file:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_file module.
+ **/
 void
 librdf_init_storage_file(librdf_world *world)
 {
