@@ -2,7 +2,7 @@
  *
  * rdf_storage_hashes.c - RDF Storage as Hashes Implementation
  *
- * $Id: rdf_storage_hashes.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage_hashes.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -48,7 +48,7 @@
 
 typedef struct 
 {
-  char *name;
+  const char *name;
   int key_fields; /* OR of LIBRDF_STATEMENT_* fields defined in rdf_statement.h */
   int value_fields; /* ditto */
 } librdf_hash_descriptor;
@@ -133,12 +133,12 @@ typedef struct
 
 
 /* helper function for implementing init and clone methods */
-static int librdf_storage_hashes_register(librdf_storage *storage, char *name, const librdf_hash_descriptor *source_desc);
-static int librdf_storage_hashes_init_common(librdf_storage* storage, char *name, char *hash_type, char *db_dir, char *indexes, int mode, int is_writable, int is_new, librdf_hash* options);
+static int librdf_storage_hashes_register(librdf_storage *storage, const char *name, const librdf_hash_descriptor *source_desc);
+static int librdf_storage_hashes_init_common(librdf_storage* storage, const char *name, char *hash_type, char *db_dir, char *indexes, int mode, int is_writable, int is_new, librdf_hash* options);
 
 
 /* prototypes for local functions */
-static int librdf_storage_hashes_init(librdf_storage* storage, char *name, librdf_hash* options);
+static int librdf_storage_hashes_init(librdf_storage* storage, const char *name, librdf_hash* options);
 static void librdf_storage_hashes_terminate(librdf_storage* storage);
 static int librdf_storage_hashes_clone(librdf_storage* new_storage, librdf_storage* old_storage);
 static int librdf_storage_hashes_open(librdf_storage* storage, librdf_model* model);
@@ -186,7 +186,7 @@ static librdf_iterator* librdf_storage_hashes_node_iterator_create(librdf_storag
 
 static int
 librdf_storage_hashes_register(librdf_storage *storage,
-                               char *name,
+                               const char *name,
                                const librdf_hash_descriptor *source_desc) 
 {
   librdf_storage_hashes_context *context=(librdf_storage_hashes_context*)storage->context;
@@ -230,7 +230,7 @@ librdf_storage_hashes_register(librdf_storage *storage,
 /* helper function for implementing init and clone methods */
 
 static int
-librdf_storage_hashes_init_common(librdf_storage* storage, char *name,
+librdf_storage_hashes_init_common(librdf_storage* storage, const char *name,
                                   char *hash_type, char *db_dir,
                                   char *indexes,
                                   int mode, int is_writable, int is_new,
@@ -339,19 +339,13 @@ librdf_storage_hashes_init_common(librdf_storage* storage, char *name,
     }
   }
 
-  /* on success or failure - don't need the passed in options */
-  if(context->options) {
-    librdf_free_hash(context->options);
-    context->options=NULL;
-  }
-  
   return status;
 }
 
 
 /* functions implementing storage api */
 static int
-librdf_storage_hashes_init(librdf_storage* storage, char *name,
+librdf_storage_hashes_init(librdf_storage* storage, const char *name,
                            librdf_hash* options)
 {
   char *hash_type, *db_dir, *indexes;
@@ -470,6 +464,7 @@ librdf_storage_hashes_open(librdf_storage* storage, librdf_model* model)
 {
   librdf_storage_hashes_context *context=(librdf_storage_hashes_context*)storage->context;
   int i;
+  int result=0;
   
   for(i=0; i<context->hash_count; i++) {
     librdf_hash *hash=context->hashes[i];
@@ -485,11 +480,20 @@ librdf_storage_hashes_open(librdf_storage* storage, librdf_model* model)
         context->hashes[j]=NULL;
       }
       
-      return 1;
+      result=1;
     }
+
+    if(result)
+      break;
   }
 
-  return 0;
+  /* on success or failure - don't need the passed in options any more */
+  if(context->options) {
+    librdf_free_hash(context->options);
+    context->options=NULL;
+  }
+  
+  return result;
 }
 
 
@@ -623,7 +627,7 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
 
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-    LIBRDF_DEBUG4("Using %s hash key %d bytes -> value %d bytes\n", context->hash_descriptions[i].name, key_len, value_len);
+    LIBRDF_DEBUG4("Using %s hash key %d bytes -> value %d bytes\n", context->hash_descriptions[i]->name, key_len, value_len);
 #endif
 
     /* Finally, store / remove the sucker */
@@ -751,7 +755,7 @@ librdf_storage_hashes_contains_statement(librdf_storage* storage, librdf_stateme
 
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  LIBRDF_DEBUG4("Using %s hash key %d bytes -> value %d bytes\n", context->hash_descriptions[hash_index].name, key_len, value_len);
+  LIBRDF_DEBUG4("Using %s hash key %d bytes -> value %d bytes\n", context->hash_descriptions[hash_index]->name, key_len, value_len);
 #endif
 
   hd_key.data=key_buffer; hd_key.size=key_len;
@@ -1877,6 +1881,12 @@ librdf_storage_hashes_register_factory(librdf_storage_factory *factory)
 }
 
 
+/**
+ * librdf_init_storage_hashes:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_hashes module.
+ **/
 void
 librdf_init_storage_hashes(librdf_world *world)
 {
