@@ -2,7 +2,7 @@
  *
  * rdf_storage_list.c - RDF Storage in memory as a list implementation
  *
- * $Id: rdf_storage_list.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage_list.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -63,7 +63,7 @@ typedef struct
 
 
 /* prototypes for local functions */
-static int librdf_storage_list_init(librdf_storage* storage, char *name, librdf_hash* options);
+static int librdf_storage_list_init(librdf_storage* storage, const char *name, librdf_hash* options);
 static int librdf_storage_list_open(librdf_storage* storage, librdf_model* model);
 static int librdf_storage_list_close(librdf_storage* storage);
 static int librdf_storage_list_size(librdf_storage* storage);
@@ -109,7 +109,7 @@ static void librdf_storage_list_register_factory(librdf_storage_factory *factory
 
 /* functions implementing storage api */
 static int
-librdf_storage_list_init(librdf_storage* storage, char *name,
+librdf_storage_list_init(librdf_storage* storage, const char *name,
                          librdf_hash* options)
 {
   librdf_storage_list_context *context=(librdf_storage_list_context*)storage->context;
@@ -498,8 +498,8 @@ librdf_storage_list_context_add_statement(librdf_storage* storage,
   if(context->index_contexts && context_node) {
     sln->context=librdf_new_node_from_node(context_node);
     if(!sln->context) {
-      LIBRDF_FREE(librdf_storage_list_node, sln);
       librdf_free_statement(sln->statement);
+      LIBRDF_FREE(librdf_storage_list_node, sln);
       return 1;
     }
   } else
@@ -507,10 +507,10 @@ librdf_storage_list_context_add_statement(librdf_storage* storage,
   
   status=librdf_list_add(context->list, sln);
   if(status) {
-    LIBRDF_FREE(librdf_storage_list_node, sln);
     if(context_node)
       librdf_free_node(sln->context);
     librdf_free_statement(sln->statement);
+    LIBRDF_FREE(librdf_storage_list_node, sln);
     return 1;
   }
 
@@ -962,6 +962,12 @@ librdf_storage_list_register_factory(librdf_storage_factory *factory)
 }
 
 
+/**
+ * librdf_init_storage_list:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_list module.
+ **/
 void
 librdf_init_storage_list(librdf_world *world)
 {
