@@ -2,7 +2,7 @@
  *
  * rdf_storage_postgresql.c - RDF Storage in PostgreSQL DB interface definition.
  *
- * $Id: rdf_storage_postgresql.c 10853 2006-04-25 06:40:43Z dajobe $
+ * $Id: rdf_storage_postgresql.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Based in part on rdf_storage_mysql.
  *
@@ -84,10 +84,12 @@ typedef struct {
   /* digest object for node hashes */
   librdf_digest *digest;
 
+  PGconn* transaction_handle;
+  
 } librdf_storage_postgresql_context;
 
 /* prototypes for local functions */
-static int librdf_storage_postgresql_init(librdf_storage* storage, char *name,
+static int librdf_storage_postgresql_init(librdf_storage* storage, const char *name,
                                           librdf_hash* options);
 static int librdf_storage_postgresql_merge(librdf_storage* storage);
 static void librdf_storage_postgresql_terminate(librdf_storage* storage);
@@ -161,8 +163,9 @@ typedef struct {
   char **row;
 } librdf_storage_postgresql_get_contexts_context;
 
-static u64 librdf_storage_postgresql_hash(librdf_storage* storage, char *type,
-                                          char *string, int length);
+static u64 librdf_storage_postgresql_hash(librdf_storage* storage, 
+                                          const char *type,
+                                          const char *string, int length);
 static u64 librdf_storage_postgresql_node_hash(librdf_storage* storage,
                                                librdf_node* node,int add);
 static int librdf_storage_postgresql_start_bulk(librdf_storage* storage);
@@ -170,7 +173,7 @@ static int librdf_storage_postgresql_stop_bulk(librdf_storage* storage);
 static int librdf_storage_postgresql_context_add_statement_helper(librdf_storage* storage,
                                                                   u64 ctxt,
                                                                   librdf_statement* statement);
-static int librdf_storage_postgresql_find_statements_in_context_augment_query(char **query, char *addition);
+static int librdf_storage_postgresql_find_statements_in_context_augment_query(char **query, const char *addition);
 
 /* methods for stream of statements */
 static int librdf_storage_postgresql_find_statements_in_context_end_of_stream(void* context);
@@ -186,6 +189,10 @@ static void librdf_storage_postgresql_get_contexts_finished(void* context);
 
 
 
+static int librdf_storage_postgresql_transaction_rollback(librdf_storage* storage);
+
+
+
 /* functions implementing storage api */
 
 /*
@@ -200,8 +207,8 @@ static void librdf_storage_postgresql_get_contexts_finished(void* context);
  * Return value: Non-zero on succes.
  **/
 static u64
-librdf_storage_postgresql_hash(librdf_storage* storage, char *type,
-                          char *string, int length)
+librdf_storage_postgresql_hash(librdf_storage* storage, const char *type,
+                               const char *string, int length)
 {
   librdf_storage_postgresql_context* context=(librdf_storage_postgresql_context*)storage->context;
   u64 hash;
@@ -291,6 +298,9 @@ librdf_storage_postgresql_get_handle(librdf_storage* storage)
   int i;
   char conninfo[256];
 
+  if(context->transaction_handle)
+    return context->transaction_handle;
+
   /* Look for an open connection handle to return */
   for(i=0; i < context->connections_count; i++) {
     if(LIBRDF_STORAGE_POSTGRESQL_CONNECTION_OPEN == context->connections[i].status) {
@@ -380,7 +390,7 @@ librdf_storage_postgresql_release_handle(librdf_storage* storage, PGconn *handle
     }
   }
   librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
-             "Unable to find busy connection (in pool of %i connections) to drop for postgresql server thread: %lu",
+             "Unable to find busy connection (in pool of %i connections) to drop for postgresql server thread: %d",
              context->connections_count, PQbackendPID(handle));
 }
 
@@ -389,7 +399,7 @@ librdf_storage_postgresql_release_handle(librdf_storage* storage, PGconn *handle
  * librdf_storage_postgresql_init:
  * @storage: the storage
  * @name: model name
- * @options: host, port, dbname, user, password [, new] [, bulk] [, merge].
+ * @options: host, port, database, user, password [, new] [, bulk] [, merge].
  *
  * Create connection to database.  Defaults to port 5432 if not given.
  *
@@ -403,8 +413,8 @@ librdf_storage_postgresql_release_handle(librdf_storage* storage, PGconn *handle
  * Return value: Non-zero on failure.
  **/
 static int
-librdf_storage_postgresql_init(librdf_storage* storage, char *name,
-                          librdf_hash* options)
+librdf_storage_postgresql_init(librdf_storage* storage, const char *name,
+                               librdf_hash* options)
 {
   librdf_storage_postgresql_context *context=(librdf_storage_postgresql_context*)storage->context;
   const char create_table_statements[]="\
@@ -467,10 +477,14 @@ librdf_storage_postgresql_init(librdf_storage* storage, char *name,
      context->port=(char*)LIBRDF_MALLOC(cstring, 10);
      strcpy(context->port,"5432"); /* default postgresql port */
   }
-  context->dbname=librdf_hash_get(options, "dbname");
+  context->dbname=librdf_hash_get(options, "database");
+  if(!context->dbname)
+    context->dbname=librdf_hash_get(options, "dbname");
   context->user=librdf_hash_get(options, "user");
-  if(context->user && !context->dbname)
-     strcpy(context->dbname,context->user); /* default dbname=user */
+  if(context->user && !context->dbname) {
+     context->dbname=(char*)LIBRDF_MALLOC(cstring, strlen(context->user)+1);
+     strcpy(context->dbname, context->user); /* default dbname=user */
+  }
 
   context->password=librdf_hash_get(options, "password");
 
@@ -687,6 +701,9 @@ librdf_storage_postgresql_terminate(librdf_storage* storage)
 
   if(context->digest)
     librdf_free_digest(context->digest);
+
+  if(context->transaction_handle)
+    librdf_storage_postgresql_transaction_rollback(storage);
 }
 
 /**
@@ -715,6 +732,8 @@ librdf_storage_postgresql_open(librdf_storage* storage, librdf_model* model)
 static int
 librdf_storage_postgresql_close(librdf_storage* storage)
 {
+  librdf_storage_postgresql_transaction_rollback(storage);
+
   return librdf_storage_postgresql_sync(storage);
 }
 
@@ -808,18 +827,8 @@ static int
 librdf_storage_postgresql_add_statements(librdf_storage* storage,
                                     librdf_stream* statement_stream)
 {
-  int helper=0;
-
-  while(!helper && !librdf_stream_end(statement_stream)) {
-    librdf_statement* statement=librdf_stream_get_object(statement_stream);
-    /* Do not add duplicate statements */
-    if(!librdf_storage_postgresql_contains_statement(storage, statement))
-      helper=librdf_storage_postgresql_context_add_statement_helper(storage, 0,
-                                                               statement);
-    librdf_stream_next(statement_stream);
-  };
-
-  return helper;
+  return librdf_storage_postgresql_context_add_statements(storage, NULL,
+                                                     statement_stream);
 }
 
 /*
@@ -1071,10 +1080,18 @@ librdf_storage_postgresql_context_add_statements(librdf_storage* storage,
 
   while(!helper && !librdf_stream_end(statement_stream)) {
     librdf_statement* statement=librdf_stream_get_object(statement_stream);
+    if(!context->bulk) {
+      /* Do not add duplicate statements 
+       * but do not check for this when in bulk mode.
+       */
+      if(librdf_storage_postgresql_contains_statement(storage, statement))
+        continue;
+    }
+    
     helper=librdf_storage_postgresql_context_add_statement_helper(storage, ctxt,
                                                              statement);
     librdf_stream_next(statement_stream);
-  };
+  }
 
   return helper;
 }
@@ -1100,7 +1117,7 @@ librdf_storage_postgresql_context_add_statement(librdf_storage* storage,
     ctxt=librdf_storage_postgresql_node_hash(storage,context_node,1);
     if(!ctxt)
       return 1;
-  };
+  }
 
   return librdf_storage_postgresql_context_add_statement_helper(storage, ctxt,
                                                            statement);
@@ -1667,7 +1684,7 @@ librdf_storage_postgresql_find_statements_with_options(librdf_storage* storage,
                PQresultErrorMessage(sos->results));
     librdf_storage_postgresql_find_statements_in_context_finished((void*)sos);
     return NULL;
-  };
+  }
   LIBRDF_FREE(cstring, query);
 
   sos->current_rowno=0;
@@ -1688,14 +1705,14 @@ librdf_storage_postgresql_find_statements_with_options(librdf_storage* storage,
   if(!stream) {
     librdf_storage_postgresql_find_statements_in_context_finished((void*)sos);
     return NULL;
-  };
+  }
 
   return stream;
 }
 
 
 static int
-librdf_storage_postgresql_find_statements_in_context_augment_query(char **query, char *addition)
+librdf_storage_postgresql_find_statements_in_context_augment_query(char **query, const char *addition)
 {
   char *newquery;
 
@@ -2176,6 +2193,147 @@ librdf_storage_postgresql_get_feature(librdf_storage* storage, librdf_uri* featu
 
 
 
+/**
+ * librdf_storage_postgresql_transaction_start:
+ * @storage: the storage object
+ * 
+ * Start a transaction
+ * 
+ * Return value: non-0 on failure
+ **/
+static int
+librdf_storage_postgresql_transaction_start(librdf_storage* storage) 
+{
+  librdf_storage_postgresql_context *context=(librdf_storage_postgresql_context *)storage->context;
+  const char query[]="START TRANSACTION";
+  
+  if(context->transaction_handle) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "Postgresql transaction already started");
+    return 1;
+  }
+  
+  context->transaction_handle=librdf_storage_postgresql_get_handle(storage);
+  if(!context->transaction_handle) 
+    return 1;
+
+  if(!PQexec(context->transaction_handle, query)) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "Postgresql query failed: %s", 
+               PQerrorMessage(context->transaction_handle));
+    librdf_storage_postgresql_release_handle(storage, context->transaction_handle);
+    context->transaction_handle=NULL;
+    return 1;
+  }
+
+  return 0;
+}
+
+
+/**
+ * librdf_storage_postgresql_transaction_start_with_handle:
+ * @storage: the storage object
+ * @handle: the transaction object
+ * 
+ * Start a transaction using an existing external transaction object.
+ * 
+ * Return value: non-0 on failure
+ **/
+static int
+librdf_storage_postgresql_transaction_start_with_handle(librdf_storage* storage,
+                                                   void* handle)
+{
+  return librdf_storage_postgresql_transaction_start(storage);
+}
+
+
+/**
+ * librdf_storage_postgresql_transaction_commit:
+ * @storage: the storage object
+ * 
+ * Commit a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static int
+librdf_storage_postgresql_transaction_commit(librdf_storage* storage) 
+{
+  librdf_storage_postgresql_context *context=(librdf_storage_postgresql_context *)storage->context;
+  const char query[]="COMMIT TRANSACTION";
+  PGconn* handle;
+  int status;
+  
+  if(!context->transaction_handle)
+    return 1;
+  
+  handle=context->transaction_handle;
+
+  status=(PQexec(context->transaction_handle, query) == NULL);
+  if(status) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "Postgresql query failed: %s", 
+               PQerrorMessage(context->transaction_handle));
+  }
+  
+  librdf_storage_postgresql_release_handle(storage, handle);
+  context->transaction_handle=NULL;
+  
+  return (status != 0);
+}
+
+
+/**
+ * librdf_storage_postgresql_transaction_rollback:
+ * @storage: the storage object
+ * 
+ * Rollback a transaction.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static int
+librdf_storage_postgresql_transaction_rollback(librdf_storage* storage)
+{
+  librdf_storage_postgresql_context *context=(librdf_storage_postgresql_context *)storage->context;
+  const char query[]="ROLLBACK TRANSACTION";
+  PGconn* handle;
+  int status;
+  
+  if(!context->transaction_handle)
+    return 1;
+  
+  handle=context->transaction_handle;
+
+  status=(PQexec(context->transaction_handle, query) == NULL);
+  if(status) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "Postgresql query failed: %s", 
+               PQerrorMessage(context->transaction_handle));
+  }
+
+  librdf_storage_postgresql_release_handle(storage, handle);
+  context->transaction_handle=NULL;
+  
+  return (status != 0);
+}
+
+
+/**
+ * librdf_storage_postgresql_transaction_get_handle:
+ * @storage: the storage object
+ * 
+ * Get the current transaction handle.
+ * 
+ * Return value: non-0 on failure 
+ **/
+static void*
+librdf_storage_postgresql_transaction_get_handle(librdf_storage* storage) 
+{
+  librdf_storage_postgresql_context *context=(librdf_storage_postgresql_context *)storage->context;
+
+  return context->transaction_handle;
+}
+
+
 /* local function to register postgresql storage functions */
 static void
 librdf_storage_postgresql_register_factory(librdf_storage_factory *factory)
@@ -2202,10 +2360,23 @@ librdf_storage_postgresql_register_factory(librdf_storage_factory *factory)
   factory->find_statements_in_context = librdf_storage_postgresql_find_statements_in_context;
   factory->get_contexts               = librdf_storage_postgresql_get_contexts;
   factory->get_feature                = librdf_storage_postgresql_get_feature;
+
+  factory->transaction_start             = librdf_storage_postgresql_transaction_start;
+  factory->transaction_start_with_handle = librdf_storage_postgresql_transaction_start_with_handle;
+  factory->transaction_commit            = librdf_storage_postgresql_transaction_commit;
+  factory->transaction_rollback          = librdf_storage_postgresql_transaction_rollback;
+  factory->transaction_get_handle        = librdf_storage_postgresql_transaction_get_handle;
 }
 
 
-void librdf_init_storage_postgresql(librdf_world *world)
+/**
+ * librdf_init_storage_postgresql:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_postgresql module.
+ **/
+void
+librdf_init_storage_postgresql(librdf_world *world)
 {
   librdf_storage_register_factory(world, "postgresql", "postgresql database store",
                                   &librdf_storage_postgresql_register_factory);
