@@ -2,7 +2,7 @@
  *
  * rdf_storage_sqlite.c - RDF Storage using SQLite implementation
  *
- * $Id: rdf_storage_sqlite.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage_sqlite.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -80,6 +80,18 @@
 #endif
 
 
+static const char* sqlite_synchronous_flags[4] = {
+  "off", "normal", "full", NULL
+};
+
+typedef struct librdf_storage_sqlite_query librdf_storage_sqlite_query;
+
+struct librdf_storage_sqlite_query
+{
+  unsigned char *query;
+  librdf_storage_sqlite_query *next;
+};
+
 typedef struct
 {
   librdf_storage *storage;
@@ -90,12 +102,18 @@ typedef struct
   
   char *name;
   size_t name_len;  
+
+  int synchronous; /* -1 (not set), 0+ index into sqlite_synchronous_flags */
+
+  int in_stream;
+  librdf_storage_sqlite_query *in_stream_queries;
+
 } librdf_storage_sqlite_context;
 
 
 
 /* prototypes for local functions */
-static int librdf_storage_sqlite_init(librdf_storage* storage, char *name, librdf_hash* options);
+static int librdf_storage_sqlite_init(librdf_storage* storage, const char *name, librdf_hash* options);
 static int librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model);
 static int librdf_storage_sqlite_close(librdf_storage* storage);
 static int librdf_storage_sqlite_size(librdf_storage* storage);
@@ -139,6 +157,7 @@ static int librdf_storage_sqlite_get_contexts_next_method(void* iterator);
 static void* librdf_storage_sqlite_get_contexts_get_method(void* iterator, int);
 static void librdf_storage_sqlite_get_contexts_finished(void* iterator);
 
+static void librdf_storage_sqlite_query_flush(librdf_storage *storage);
 
 static void librdf_storage_sqlite_register_factory(librdf_storage_factory *factory);
 
@@ -146,12 +165,13 @@ static void librdf_storage_sqlite_register_factory(librdf_storage_factory *facto
 
 /* functions implementing storage api */
 static int
-librdf_storage_sqlite_init(librdf_storage* storage, char *name,
+librdf_storage_sqlite_init(librdf_storage* storage, const char *name,
                            librdf_hash* options)
 {
   librdf_storage_sqlite_context *context=(librdf_storage_sqlite_context*)storage->context;
   char *name_copy;
   int is_new;
+  char* synchronous;
   
   if(!name)
     return 1;
@@ -168,6 +188,24 @@ librdf_storage_sqlite_init(librdf_storage* storage, char *name,
   if((is_new=librdf_hash_get_as_boolean(options, "new"))>0)
     context->is_new=1; /* default is NOT NEW */
 
+  /* Redland default is "PRAGMA synchronous normal" */
+  context->synchronous=1;
+
+  if((synchronous=librdf_hash_get(options, "synchronous"))) {
+    int i;
+    
+    for(i=0; sqlite_synchronous_flags[i]; i++) {
+      if(!strcmp(synchronous, sqlite_synchronous_flags[i])) {
+        context->synchronous=i;
+        break;
+      }
+    }
+    
+    LIBRDF_FREE(cstring, synchronous);
+
+  }
+  
+
   /* no more options, might as well free them now */
   if(options)
     librdf_free_hash(options);
@@ -311,11 +349,33 @@ librdf_storage_sqlite_exec(librdf_storage* storage,
     status=SQLITE_OK;
   
   if(status != SQLITE_OK) {
-    librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
-               "SQLite database %s SQL exec '%s' failed - %s (%d)", 
-               context->name, request, errmsg, status);
-    sqlite_CLOSE(context->db);
-    context->db=NULL;
+    if(status == SQLITE_LOCKED && !callback && context->in_stream) {
+       librdf_storage_sqlite_query *query;
+       query=(librdf_storage_sqlite_query*)LIBRDF_CALLOC(librdf_storage_sqlite_query, 1, sizeof(librdf_storage_sqlite_query));
+
+       query->query=(unsigned char*)LIBRDF_MALLOC(cstring, strlen((char *)request)+1);
+       strcpy((char*)query->query, (char *)request);
+
+       if(!context->in_stream_queries)
+         context->in_stream_queries=query;
+       else {
+         librdf_storage_sqlite_query *q=context->in_stream_queries;
+
+         while(q->next)
+           q=q->next;
+
+         q->next=query;
+       }
+
+       status = SQLITE_OK;
+
+    } else {
+      librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "SQLite database %s SQL exec '%s' failed - %s (%d)",
+                 context->name, request, errmsg, status);
+      sqlite_CLOSE(context->db);
+      context->db=NULL;
+    }
   }
 
   return (status != SQLITE_OK);
@@ -486,12 +546,12 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
   raptor_stringbuffer *sb;
   unsigned char *expression;
 
-  sb=raptor_new_stringbuffer();
-
   value_e=sqlite_string_escape(value, value_len, &value_e_len);
   if(!value_e)
     return -1;
 
+  sb=raptor_new_stringbuffer();
+
   raptor_stringbuffer_append_counted_string(sb,  
                                             (const unsigned char*)"text = ",
                                             7, 1);
@@ -504,13 +564,15 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
       return -1;
     raptor_stringbuffer_append_string(sb, (const unsigned char*)"AND language = ", 1);
     raptor_stringbuffer_append_counted_string(sb, language_e, language_e_len, 1);
-  }
+  } else
+    raptor_stringbuffer_append_string(sb, (const unsigned char*)"AND language IS NULL ", 1);
 
   if(datatype) {
     datatype_id=librdf_storage_sqlite_uri_helper(storage, datatype);
     raptor_stringbuffer_append_string(sb, (const unsigned char*)"AND datatype = ", 1);
     raptor_stringbuffer_append_decimal(sb, datatype_id);
-  }
+  } else
+    raptor_stringbuffer_append_string(sb, (const unsigned char*)"AND datatype IS NULL ", 1);
   
   expression=raptor_stringbuffer_as_string(sb);
   id=librdf_storage_sqlite_get_helper(storage, TABLE_LITERALS, expression);
@@ -680,16 +742,44 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s open failed - %s", 
                context->name, errmsg);
+#if SQLITE_API == 2
     sqlite_FREE(errmsg);
+#endif
 #if SQLITE_API == 3
     sqlite_CLOSE(context->db);
 #endif
     return 1;
   }
+
+  
+  if(context->synchronous >= 0) {
+    raptor_stringbuffer *sb;
+    unsigned char *request;
+
+    sb=raptor_new_stringbuffer();
+    raptor_stringbuffer_append_string(sb, 
+                                      (const unsigned char*)"PRAGMA synchronous=", 1);
+    raptor_stringbuffer_append_string(sb, 
+                                      (const unsigned char*)sqlite_synchronous_flags[context->synchronous], 1);
+    raptor_stringbuffer_append_counted_string(sb, (const unsigned char*)";", 1, 1);
+    
+    request=raptor_stringbuffer_as_string(sb);
+
+    librdf_storage_sqlite_exec(storage, 
+                               request,
+                               NULL, NULL, 0);
+    raptor_free_stringbuffer(sb);
+  }
+
   
   if(context->is_new) {
     int i;
     unsigned char request[200];
+    int begin;
+
+    begin=librdf_storage_sqlite_exec(storage,
+                                     (unsigned char *)"BEGIN TRANSACTION;",
+                                     NULL, NULL, 0);
 
     for(i=0; i < NTABLES; i++) {
 
@@ -709,20 +799,29 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
                                     request,
                                     NULL, /* no callback */
                                     NULL, /* arg */
-                                    0))
+                                    0)) {
+        if(!begin)
+          librdf_storage_sqlite_exec(storage,
+                                     (unsigned char *) "END TRANSACTION;",
+                                     NULL, NULL, 0);
         return 1;
+      }
 
     } /* end drop/create table loop */
 
-
     strcpy((char*)request, 
            "CREATE INDEX spindex ON triples (subjectUri, subjectBlank, predicateUri);");
     if(librdf_storage_sqlite_exec(storage,
                                   request,
                                   NULL, /* no callback */
                                   NULL, /* arg */
-                                  0))
+                                  0)) {
+      if(!begin)
+        librdf_storage_sqlite_exec(storage,
+                                   (unsigned char *) "END TRANSACTION;",
+                                   NULL, NULL, 0);
       return 1;
+    }
     
     strcpy((char*)request, 
            "CREATE INDEX uriindex ON uris (uri);");
@@ -730,12 +829,21 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
                                   request,
                                   NULL, /* no callback */
                                   NULL, /* arg */
-                                  0))
+                                  0)) {
+      if(!begin)
+        librdf_storage_sqlite_exec(storage,
+                                   (unsigned char *) "END TRANSACTION;",
+                                   NULL, NULL, 0);
       return 1;
+    }
+    
+    if(!begin)
+      librdf_storage_sqlite_exec(storage,
+                                 (unsigned char *) "END TRANSACTION;",
+                                 NULL, NULL, 0);
     
   } /* end if is new */
 
-  
   return 0;
 }
 
@@ -925,8 +1033,6 @@ librdf_storage_sqlite_statement_operator_helper(librdf_storage* storage,
     need_and=1;
   }
     
-  raptor_stringbuffer_append_counted_string(sb,  
-                                            (const unsigned char*)";", 1, 1);
   return 0;
 }
 
@@ -938,15 +1044,27 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
   raptor_stringbuffer *sb;
   unsigned char *request;
   int count=0;
-  int rc;
+  int rc, begin;
+
+  begin=librdf_storage_sqlite_exec(storage,
+                                   (unsigned char *) "BEGIN TRANSACTION;",
+                                   NULL, NULL, 0);
 
   sb=raptor_new_stringbuffer();
   raptor_stringbuffer_append_string(sb, 
-                                    (const unsigned char*)"SELECT COUNT(*)", 1);
+                                    (const unsigned char*)"SELECT subjectUri",
+                                    1);
   if(librdf_storage_sqlite_statement_operator_helper(storage, statement, 
-                                                     NULL, sb))
+                                                     NULL, sb)) {
+    if(!begin)
+      librdf_storage_sqlite_exec(storage,
+                                 (unsigned char *) "END TRANSACTION;",
+                                 NULL, NULL, 0);
+    raptor_free_stringbuffer(sb);
     return -1;
+  }
 
+  raptor_stringbuffer_append_string(sb, (const unsigned char*)" LIMIT 1;", 1);
   request=raptor_stringbuffer_as_string(sb);
   
   rc=librdf_storage_sqlite_exec(storage,
@@ -957,6 +1075,11 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
   
   raptor_free_stringbuffer(sb);
 
+  if(!begin)
+    librdf_storage_sqlite_exec(storage,
+                               (unsigned char *) "END TRANSACTION;",
+                               NULL, NULL, 0);
+
   if(rc)  
     return -1;
 
@@ -1037,6 +1160,7 @@ librdf_storage_sqlite_serialise(librdf_storage* storage)
     return NULL;
 
   scontext->sqlite_context=context;
+  context->in_stream++;
 
   sb=raptor_new_stringbuffer();
   sqlite_construct_select_helper(sb);
@@ -1243,7 +1367,9 @@ librdf_storage_sqlite_get_next_common(librdf_storage_sqlite_context* scontext,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  scontext->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
     result= -1;
   }
@@ -1345,7 +1471,9 @@ librdf_storage_sqlite_serialise_finished(void* context)
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  scontext->sqlite_context->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
   }
 
@@ -1358,6 +1486,10 @@ librdf_storage_sqlite_serialise_finished(void* context)
   if(scontext->context)
     librdf_free_node(scontext->context);
 
+  scontext->sqlite_context->in_stream--;
+  if(!scontext->sqlite_context->in_stream)
+    librdf_storage_sqlite_query_flush(scontext->storage);
+
   LIBRDF_FREE(librdf_storage_sqlite_serialise_stream_context, scontext);
 }
 
@@ -1416,17 +1548,22 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
     return NULL;
 
   scontext->sqlite_context=context;
+  context->in_stream++;
 
 
   scontext->query_statement=librdf_new_statement_from_statement(statement);
-  if(!scontext->query_statement)
+  if(!scontext->query_statement) {
+    librdf_storage_sqlite_find_statements_finished((void*)scontext);
     return NULL;
+  }
 
   if(librdf_storage_sqlite_statement_helper(storage,
                                             statement,
                                             NULL, 
-                                            node_types, node_ids, fields))
+                                            node_types, node_ids, fields)) {
+    librdf_storage_sqlite_find_statements_finished((void*)scontext);
     return NULL;
+  }
 
   sb=raptor_new_stringbuffer();
   sqlite_construct_select_helper(sb);
@@ -1482,7 +1619,7 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
                context->name, request, errmsg, status);
 
     raptor_free_stringbuffer(sb);
-    librdf_storage_sqlite_serialise_finished((void*)scontext);
+    librdf_storage_sqlite_find_statements_finished((void*)scontext);
     return NULL;
   }
   raptor_free_stringbuffer(sb);
@@ -1597,7 +1734,9 @@ librdf_storage_sqlite_find_statements_finished(void* context)
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  scontext->sqlite_context->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
   }
 
@@ -1613,6 +1752,10 @@ librdf_storage_sqlite_find_statements_finished(void* context)
   if(scontext->context)
     librdf_free_node(scontext->context);
 
+  scontext->sqlite_context->in_stream--;
+  if(!scontext->sqlite_context->in_stream)
+    librdf_storage_sqlite_query_flush(scontext->storage);
+
   LIBRDF_FREE(librdf_storage_sqlite_find_statements_stream_context, scontext);
 }
 
@@ -1639,14 +1782,24 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
   raptor_stringbuffer *sb;
   unsigned char* request;
   int i;
-  int rc;
+  int rc, begin;
   int max=3;
   
+  begin=librdf_storage_sqlite_exec(storage,
+                                   (unsigned char *) "BEGIN TRANSACTION;",
+                                   NULL, NULL, 0);
+
   if(librdf_storage_sqlite_statement_helper(storage,
                                             statement,
                                             context_node,
-                                            node_types, node_ids, fields))
+                                            node_types, node_ids, fields)) {
+
+    if(!begin)
+      librdf_storage_sqlite_exec(storage,
+                                 (unsigned char *) "END TRANSACTION;",
+                                 NULL, NULL, 0);
     return -1;
+  }
   
   if(context_node)
     max++;
@@ -1686,6 +1839,11 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
   
   raptor_free_stringbuffer(sb);
   
+  if(!begin)
+    librdf_storage_sqlite_exec(storage,
+                               (unsigned char *) "END TRANSACTION;",
+                               NULL, NULL, 0);
+
   if(rc)
     return 1;
 
@@ -1716,9 +1874,14 @@ librdf_storage_sqlite_context_remove_statement(librdf_storage* storage,
   sb=raptor_new_stringbuffer();
   raptor_stringbuffer_append_string(sb, (const unsigned char*)"DELETE", 1);
   if(librdf_storage_sqlite_statement_operator_helper(storage, statement,
-                                                     context_node, sb))
+                                                     context_node, sb)) {
+    raptor_free_stringbuffer(sb);
     return -1;
+  }
 
+  raptor_stringbuffer_append_counted_string(sb,
+                                            (const unsigned char*)";", 1, 1);
+ 
   request=raptor_stringbuffer_as_string(sb);
   
   rc=librdf_storage_sqlite_exec(storage,
@@ -1733,6 +1896,59 @@ librdf_storage_sqlite_context_remove_statement(librdf_storage* storage,
 }
 
 
+static  int
+librdf_storage_sqlite_context_remove_statements(librdf_storage* storage, 
+                                                librdf_node* context_node)
+{
+  triple_node_type node_types[4];
+  int node_ids[4];
+  const unsigned char* fields[4];
+  raptor_stringbuffer *sb;
+  unsigned char *request;
+  int rc=0;
+  
+  
+  if(librdf_storage_sqlite_statement_helper(storage,
+                                            NULL,
+                                            context_node,
+                                            node_types, node_ids, fields))
+    return -1;
+    
+  sb=raptor_new_stringbuffer();
+  raptor_stringbuffer_append_counted_string(sb,
+                                    (unsigned char*)"DELETE FROM ", 12, 1);
+  raptor_stringbuffer_append_string(sb, 
+                                    (unsigned char*)sqlite_tables[TABLE_TRIPLES].name, 1);
+
+  raptor_stringbuffer_append_counted_string(sb, 
+                                            (unsigned char*)" WHERE ", 7, 1);
+
+  raptor_stringbuffer_append_string(sb, fields[TRIPLE_CONTEXT], 1);
+  raptor_stringbuffer_append_counted_string(sb, 
+                                            (unsigned char*)"=", 1, 1);
+  raptor_stringbuffer_append_decimal(sb, node_ids[TRIPLE_CONTEXT]);
+  raptor_stringbuffer_append_counted_string(sb, 
+                                            (unsigned char*)"\n", 1, 1);
+  raptor_stringbuffer_append_counted_string(sb, 
+                                            (unsigned char*)";", 1, 1);
+  
+  request=raptor_stringbuffer_as_string(sb);
+
+  rc=librdf_storage_sqlite_exec(storage,
+                                request,
+                                NULL, /* no callback */
+                                NULL, /* arg */
+                                0);
+
+  raptor_free_stringbuffer(sb);
+
+  if(rc)
+    return -1;
+
+  return 0;
+}
+
+
 typedef struct {
   librdf_storage *storage;
   librdf_storage_sqlite_context* sqlite_context;
@@ -1779,14 +1995,17 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
     return NULL;
 
   scontext->sqlite_context=context;
+  context->in_stream++;
 
   scontext->context_node=librdf_new_node_from_node(context_node);
 
   if(librdf_storage_sqlite_statement_helper(storage,
                                             NULL,
                                             scontext->context_node,
-                                            node_types, node_ids, fields))
+                                            node_types, node_ids, fields)) {
+    librdf_storage_sqlite_context_serialise_finished((void*)scontext);
     return NULL;
+  }
 
   sb=raptor_new_stringbuffer();
   sqlite_construct_select_helper(sb);
@@ -1830,7 +2049,7 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
                context->name, errmsg, status);
 
     raptor_free_stringbuffer(sb);
-    librdf_storage_sqlite_serialise_finished((void*)scontext);
+    librdf_storage_sqlite_context_serialise_finished((void*)scontext);
     return NULL;
   }
   raptor_free_stringbuffer(sb);
@@ -1946,7 +2165,9 @@ librdf_storage_sqlite_context_serialise_finished(void* context)
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  scontext->sqlite_context->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
   }
 
@@ -1959,6 +2180,10 @@ librdf_storage_sqlite_context_serialise_finished(void* context)
   if(scontext->context)
     librdf_free_node(scontext->context);
 
+  scontext->sqlite_context->in_stream--;
+  if(!scontext->sqlite_context->in_stream)
+    librdf_storage_sqlite_query_flush(scontext->storage);
+
   LIBRDF_FREE(librdf_storage_sqlite_context_serialise_stream_context, scontext);
 }
 
@@ -2061,7 +2286,9 @@ librdf_storage_sqlite_get_next_context_common(librdf_storage_sqlite_context* sco
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  scontext->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
     result= -1;
   }
@@ -2171,7 +2398,9 @@ librdf_storage_sqlite_get_contexts_finished(void* iterator)
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s finalize failed - %s (%d)", 
                  icontext->sqlite_context->name, errmsg, status);
+#if SQLITE_API == 2
       sqlite_FREE(errmsg);
+#endif
     }
   }
 
@@ -2219,7 +2448,7 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
   raptor_stringbuffer_append_string(sb,  
                                     (const unsigned char*)sqlite_tables[TABLE_TRIPLES].name, 1);
   raptor_stringbuffer_append_string(sb,
-                                            (const unsigned char*)" LEFT JOIN uris ON uris.id = contextUri;", 1);
+                                            (const unsigned char*)" LEFT JOIN uris ON uris.id = contextUri WHERE contextUri NOT NULL;", 1);
 
   request=raptor_stringbuffer_as_string(sb);
   
@@ -2302,6 +2531,35 @@ librdf_storage_sqlite_get_feature(librdf_storage* storage, librdf_uri* feature)
   return NULL;
 }
 
+static void
+librdf_storage_sqlite_query_flush(librdf_storage *storage)
+{
+  librdf_storage_sqlite_query *query;
+  librdf_storage_sqlite_context* context=(librdf_storage_sqlite_context*)storage->context;
+  int begin;
+
+  if(!context->in_stream_queries)
+    return;
+
+  begin=librdf_storage_sqlite_exec(storage,
+                                   (unsigned char *) "BEGIN TRANSACTION;",
+                                   NULL, NULL, 0);
+
+  while(context->in_stream_queries) {
+    query=context->in_stream_queries;
+    context->in_stream_queries=query->next;
+
+    librdf_storage_sqlite_exec(storage, query->query, NULL, NULL, 0);
+
+    LIBRDF_FREE(cstring, query->query);
+    LIBRDF_FREE(librdf_storage_sqlite_query, query);
+  }
+
+  if(!begin)
+    librdf_storage_sqlite_exec(storage,
+                               (unsigned char *) "END TRANSACTION;",
+                               NULL, NULL, 0);
+}
 
 /* local function to register sqlite storage functions */
 
@@ -2323,12 +2581,19 @@ librdf_storage_sqlite_register_factory(librdf_storage_factory *factory)
   factory->find_statements    = librdf_storage_sqlite_find_statements;
   factory->context_add_statement    = librdf_storage_sqlite_context_add_statement;
   factory->context_remove_statement = librdf_storage_sqlite_context_remove_statement;
+  factory->context_remove_statements = librdf_storage_sqlite_context_remove_statements;
   factory->context_serialise        = librdf_storage_sqlite_context_serialise;
   factory->get_contexts             = librdf_storage_sqlite_get_contexts;
   factory->get_feature              = librdf_storage_sqlite_get_feature;
 }
 
 
+/**
+ * librdf_init_storage_sqlite:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_sqlite module.
+ **/
 void
 librdf_init_storage_sqlite(librdf_world *world)
 {
