@@ -2,7 +2,7 @@
  *
  * rdf_uri.c - RDF URI interface
  *
- * $Id: rdf_uri.c 10856 2006-04-25 06:53:48Z dajobe $
+ * $Id: rdf_uri.c 12180 2007-05-01 03:42:32Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -55,7 +55,7 @@
  * librdf_init_uri:
  * @world: redland world object
  *
- * INTERNAL - Initialise the librdf_uri class.
+ * INTERNAL - Initialise the uri module.
  *
  **/
 void
@@ -81,7 +81,7 @@ librdf_init_uri(librdf_world *world)
  * librdf_finish_uri:
  * @world: redland world object
  *
- * INTERNAL - Terminate the librdf_uri class.
+ * INTERNAL - Terminate the uri module.
  *
  **/
 void
@@ -117,6 +117,8 @@ librdf_new_uri (librdf_world *world,
   librdf_hash_datum key, value; /* on stack - not allocated */
   librdf_hash_datum *old_value;
 
+  librdf_world_open(world);
+
   if(!uri_string || !*uri_string)
     return NULL;
 
@@ -372,6 +374,8 @@ librdf_new_uri_from_filename(librdf_world* world, const char *filename) {
   librdf_uri* new_uri;
   unsigned char *uri_string;
 
+  librdf_world_open(world);
+
   if(!filename)
     return NULL;
   
@@ -663,7 +667,7 @@ main(int argc, char *argv[])
   const unsigned char *hp_string=(const unsigned char*)"http://purl.org/net/dajobe/";
   librdf_uri *uri1, *uri2, *uri3, *uri4, *uri5, *uri6, *uri7, *uri8, *uri9;
   librdf_digest *d;
-  char *program=argv[0];
+  const char *program=librdf_basename((const char*)argv[0]);
   const char *file_string="/big/long/directory/file";
   const unsigned char *file_uri_string=(const unsigned char*)"file:///big/long/directory/file";
   const unsigned char *uri_string=(const unsigned char*)"http://example.com/big/long/directory/blah#frag";
@@ -672,11 +676,7 @@ main(int argc, char *argv[])
   librdf_world *world;
   
   world=librdf_new_world();
-  librdf_world_init_mutex(world);
-
-  librdf_init_digest(world);
-  librdf_init_hash(world);
-  librdf_init_uri(world);
+  librdf_world_open(world);
 
   fprintf(stderr, "%s: Creating new URI from string\n", program);
   uri1=librdf_new_uri(world, hp_string);
@@ -760,11 +760,7 @@ main(int argc, char *argv[])
   librdf_free_uri(uri8);
   librdf_free_uri(uri9);
   
-  librdf_finish_uri(world);
-  librdf_finish_hash(world);
-  librdf_finish_digest(world);
-
-  LIBRDF_FREE(librdf_world, world);
+  librdf_free_world(world);
 
   /* keep gcc -Wall happy */
   return(0);
