@@ -2,9 +2,9 @@
  *
  * win32_rdf_config.h - redland WIN32 hard-coded config
  *
- * $Id: win32_rdf_config.h 10947 2006-05-05 14:20:56Z dajobe $
+ * $Id: win32_rdf_config.h 12150 2007-04-23 15:15:00Z dajobe $
  *
- * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -181,9 +181,6 @@ extern "C" {
 /* Define to 1 if you have the <pthread.h> header file. */
 /* #undef HAVE_PTHREAD_H */
 
-/* Have Raptor RDF parser */
-#define HAVE_RAPTOR_RDF_PARSER 1
-
 /* Define to 1 if you have the <stdint.h> header file. */
 #define HAVE_STDINT_H 1
 
@@ -224,7 +221,7 @@ extern "C" {
 /* #undef LIBRDF_DISABLE_ASSERT_MESSAGES */
 
 /* Release version as a decimal */
-#define LIBRDF_VERSION_DECIMAL 10004
+#define LIBRDF_VERSION_DECIMAL 10006
 
 /* Major version number */
 #define LIBRDF_VERSION_MAJOR 1
@@ -233,7 +230,7 @@ extern "C" {
 #define LIBRDF_VERSION_MINOR 0
 
 /* Release version number */
-#define LIBRDF_VERSION_RELEASE 4
+#define LIBRDF_VERSION_RELEASE 6
 
 /* The size of a `unsigned char', as computed by sizeof. */
 #define SIZEOF_UNSIGNED_CHAR 1
@@ -275,7 +272,7 @@ extern "C" {
 /* #define TIME_WITH_SYS_TIME*/
 
 /* Version number of package */
-#define VERSION "1.0.4"
+#define VERSION "1.0.6"
 
 /* Use POSIX threads */
 /* #undef WITH_THREADS */
