@@ -2,7 +2,7 @@
  *
  * raptor.h - Redland Parser Toolkit for RDF (Raptor) interfaces and definition
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -72,6 +72,13 @@ extern "C" {
 #define RAPTOR_DEPRECATED
 #endif
 
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define RAPTOR_PRINTF_FORMAT(string_index, first_to_check_index) \
+  __attribute__((__format__(__printf__, string_index, first_to_check_index)))
+#else
+#define RAPTOR_PRINTF_FORMAT(string_index, first_to_check_index)
+#endif
+
 /**
  * raptor_uri:
  *
@@ -95,6 +102,10 @@ RAPTOR_API
 extern const unsigned int raptor_version_release;
 RAPTOR_API
 extern const unsigned int raptor_version_decimal;
+RAPTOR_API
+extern const char * const raptor_license_string;
+RAPTOR_API
+extern const char * const raptor_home_url_string;
 
 RAPTOR_API
 extern const unsigned char * const raptor_xml_namespace_uri;
@@ -197,8 +208,8 @@ typedef raptor_parser raptor_ntriples_parser;
  * raptor_identifier_type:
  * @RAPTOR_IDENTIFIER_TYPE_RESOURCE:    Resource URI (e.g. <literal>rdf:about</literal>)
  * @RAPTOR_IDENTIFIER_TYPE_ANONYMOUS:   <literal>_:foo</literal> N-Triples, or generated
- * @RAPTOR_IDENTIFIER_TYPE_PREDICATE:   predicate URI.  WARNING: Will not be generated in in Raptor 1.4.9 or newer.  Instead a #RAPTOR_IDENTIFIER_RESOURCE will be returned.
- * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  May not be generated in Raptor 1.4.9 or newer, instead a #RAPTOR_IDENTIFIER_RESOURCE will be returned.
+ * @RAPTOR_IDENTIFIER_TYPE_PREDICATE:   predicate URI.  WARNING: Will not be generated in in Raptor 1.4.9 or newer.  Instead a #RAPTOR_IDENTIFIER_TYPE_RESOURCE will be returned.
+ * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  No longer generated in any parser in Raptor 1.4.10+, instead a #RAPTOR_IDENTIFIER_TYPE_RESOURCE is returned.
  * @RAPTOR_IDENTIFIER_TYPE_LITERAL:     regular literal
  * @RAPTOR_IDENTIFIER_TYPE_XML_LITERAL: <literal>rdf:parseType="Literal"</literal>.  No longer generated by any parser in Raptor 1.4.8+, instead a #RAPTOR_IDENTIFIER_TYPE_LITERAL is returned with a datatype of <literal>rdf:XMLLiteral</literal>.
  * @RAPTOR_IDENTIFIER_TYPE_UNKNOWN:     Internal
@@ -317,6 +328,19 @@ typedef struct {
  *   for each indent level when serializing with auto indent.
  * @RAPTOR_FEATURE_WRITER_XML_VERSION: Integer XML version XML 1.0 (10) or XML 1.1 (11)
  * @RAPTOR_FEATURE_WRITER_XML_DECLARATION: Write XML 1.0 or 1.1 declaration.
+ * @RAPTOR_FEATURE_NO_NET: Deny network requests
+ * @RAPTOR_FEATURE_RESOURCE_BORDER: Border color of resource
+ *   nodes for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_LITERAL_BORDER: Border color of literal nodes
+ *   for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_BNODE_BORDER: Border color of blank nodes for
+ *   GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_RESOURCE_FILL: Fill color of resource nodes
+ *   for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_LITERAL_FILL: Fill color of literal nodes for
+ *   GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_BNODE_FILL: Fill color of blank nodes for
+ *   GraphViz DOT serializer.
  * @RAPTOR_FEATURE_LAST: Internal
  *
  * Raptor parser, serializer or XML writer features.
@@ -339,7 +363,14 @@ typedef enum {
   RAPTOR_FEATURE_WRITER_INDENT_WIDTH,
   RAPTOR_FEATURE_WRITER_XML_VERSION,
   RAPTOR_FEATURE_WRITER_XML_DECLARATION,
-  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_WRITER_XML_DECLARATION
+  RAPTOR_FEATURE_NO_NET,
+  RAPTOR_FEATURE_RESOURCE_BORDER,
+  RAPTOR_FEATURE_LITERAL_BORDER,
+  RAPTOR_FEATURE_BNODE_BORDER,
+  RAPTOR_FEATURE_RESOURCE_FILL,
+  RAPTOR_FEATURE_LITERAL_FILL,
+  RAPTOR_FEATURE_BNODE_FILL,
+  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_BNODE_FILL
 } raptor_feature;
 
 
@@ -643,6 +674,17 @@ typedef void (*raptor_www_write_bytes_handler)(raptor_www* www, void *userdata,
  */
 typedef void (*raptor_www_content_type_handler)(raptor_www* www, void *userdata, const char *content_type);
 
+/**
+ * raptor_uri_filter_func:
+ * @user_data: user data
+ * @uri: #raptor_uri URI to check
+ *
+ * Callback function for #raptor_www_set_uri_filter
+ *
+ * Return value: non-0 to filter the URI
+ */
+typedef int (*raptor_uri_filter_func)(void *user_data, raptor_uri* uri);
+
 
 /* Public functions */
 
@@ -687,6 +729,8 @@ RAPTOR_API
 void raptor_set_generate_id_handler(raptor_parser* parser, void *user_data, raptor_generate_id_handler handler);
 RAPTOR_API
 void raptor_set_namespace_handler(raptor_parser* parser, void *user_data, raptor_namespace_handler handler);
+RAPTOR_API
+void raptor_parser_set_uri_filter(raptor_parser* parser, raptor_uri_filter_func filter, void* user_data);
 
 RAPTOR_API
 void raptor_print_statement(const raptor_statement * statement, FILE *stream);
@@ -742,6 +786,8 @@ RAPTOR_API
 const char* raptor_get_label(raptor_parser *rdf_parser);
 RAPTOR_API
 const char* raptor_get_mime_type(raptor_parser *rdf_parser);
+RAPTOR_API
+int raptor_get_need_base_uri(raptor_parser *rdf_parser);
 
 RAPTOR_API
 int raptor_features_enumerate(const raptor_feature feature, const char **name, raptor_uri **uri, const char **label);
@@ -754,10 +800,14 @@ int raptor_get_feature(raptor_parser *parser, raptor_feature feature);
 RAPTOR_API
 const unsigned char* raptor_parser_get_feature_string(raptor_parser *parser, raptor_feature feature);
 RAPTOR_API
+unsigned int raptor_get_feature_count(void);
+RAPTOR_API
 void raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict);
 RAPTOR_API
 const char* raptor_parser_get_accept_header(raptor_parser* rdf_parser);
 
+unsigned char* raptor_parser_generate_id(raptor_parser *rdf_parser, raptor_genid_type type);
+
 /* Get serializer names */
 RAPTOR_API
 int raptor_serializers_enumerate(const unsigned int counter, const char **name, const char **label, const char **mime_type, const unsigned char **uri_string);
@@ -860,6 +910,8 @@ const char* raptor_ntriples_term_as_string(raptor_ntriples_term_type term);
 RAPTOR_API
 int raptor_iostream_write_string_ntriples(raptor_iostream *iostr, const unsigned char *string, size_t len, const char delim);
 RAPTOR_API
+void raptor_iostream_write_string_turtle(raptor_iostream *iostr, const unsigned char *string, size_t len);
+RAPTOR_API
 void raptor_iostream_write_statement_ntriples(raptor_iostream* iostr, const raptor_statement *statement);
 RAPTOR_API
 int raptor_xml_any_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, int xml_version, raptor_simple_message_handler error_handler, void *error_data);
@@ -871,7 +923,7 @@ RAPTOR_API
 int raptor_iostream_write_xml_escaped_string(raptor_iostream* iostr, const unsigned char *string, size_t len, char quote, raptor_simple_message_handler error_handler, void *error_data);
 
 RAPTOR_API
-char* raptor_vsnprintf(const char *message, va_list arguments);
+char* raptor_vsnprintf(const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(1, 0);
 
 RAPTOR_API
 int raptor_xml_name_check(const unsigned char *string, size_t length, int xml_version);
@@ -972,6 +1024,8 @@ void raptor_www_set_content_type_handler(raptor_www *www, raptor_www_content_typ
 RAPTOR_API
 void raptor_www_set_error_handler(raptor_www *www, raptor_message_handler error_handler, void *error_data);
 RAPTOR_API
+void raptor_www_set_uri_filter(raptor_www* www, raptor_uri_filter_func filter, void* user_data);
+RAPTOR_API
 int raptor_www_fetch(raptor_www *www, raptor_uri *uri);
 RAPTOR_API
 int raptor_www_fetch_to_string(raptor_www *www, raptor_uri *uri, void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
@@ -1026,6 +1080,7 @@ RAPTOR_API
 raptor_namespace* raptor_namespaces_find_namespace_by_uri(raptor_namespace_stack *nstack, raptor_uri *ns_uri);
 RAPTOR_API
 int raptor_namespaces_namespace_in_scope(raptor_namespace_stack *nstack, const raptor_namespace *nspace);
+RAPTOR_API
 raptor_qname* raptor_namespaces_qname_from_uri(raptor_namespace_stack *nstack,  raptor_uri *uri, int xml_version);
 
 /* raptor_namespace - XML namespace */
