@@ -2,7 +2,7 @@
  *
  * raptor_internal.h - Redland Parser Toolkit for RDF (Raptor) internals
  *
- * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -167,8 +167,8 @@ void raptor_system_free(void *ptr);
 #define RAPTOR_FATAL2(msg,arg) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg); abort();} while(0)
 #define RAPTOR_FATAL3(msg,arg1,arg2) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg1, arg2); abort();} while(0)
 
-
-
+#define MAX_ASCII_INT_SIZE 13
+  
 /* XML parser includes */
 #ifdef RAPTOR_XML_EXPAT
 #ifdef HAVE_EXPAT_H
@@ -193,11 +193,12 @@ typedef struct raptor_sax2_s raptor_sax2;
 /* raptor_libxml.c exports */
 extern void raptor_libxml_init(raptor_sax2* sax2, raptor_uri *base_uri);
 extern void raptor_libxml_init_sax_error_handlers(xmlSAXHandler *sax);
-extern void raptor_libxml_init_generic_error_handlers(raptor_parser *rdf_parser);
+extern void raptor_libxml_init_generic_error_handlers(raptor_sax2 *sax2);
 
-extern void raptor_libxml_validation_error(void *context, const char *msg, ...);
-extern void raptor_libxml_validation_warning(void *context, const char *msg, ...);
+extern void raptor_libxml_validation_error(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+extern void raptor_libxml_validation_warning(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 void raptor_libxml_free(xmlParserCtxtPtr xc);
+void raptor_libxml_xmlStructuredErrorFunc(void *user_data, xmlErrorPtr err);
 
 /* raptor_parse.c - exported to libxml part */
 extern void raptor_libxml_update_document_locator(raptor_sax2* sax2, raptor_locator* locator);
@@ -269,10 +270,34 @@ struct raptor_namespace_s {
   int is_rdf_schema;
 };
 
+
+typedef struct {
+  void *fatal_error_user_data;
+  void *error_user_data;
+  void *warning_user_data;
+
+  raptor_message_handler fatal_error_handler;
+  raptor_message_handler error_handler;
+  raptor_message_handler warning_handler;
+
+  raptor_locator* locator;
+} raptor_error_handlers;
+
+
+typedef enum {
+  RAPTOR_LOG_LEVEL_NONE,
+  RAPTOR_LOG_LEVEL_FATAL,
+  RAPTOR_LOG_LEVEL_ERROR,
+  RAPTOR_LOG_LEVEL_WARNING,
+  RAPTOR_LOG_LEVEL_LAST=RAPTOR_LOG_LEVEL_WARNING
+} raptor_log_level;
+
+
 #ifdef RAPTOR_XML_LIBXML
 #define RAPTOR_LIBXML_MAGIC 0x8AF108
 #endif
 
+
 /*
  * Raptor parser object
  */
@@ -296,73 +321,13 @@ struct raptor_parser_s {
   /* static statement for use in passing to user code */
   raptor_statement statement;
 
-  /* FEATURE: 
-   * non 0 if scanning for <rdf:RDF> element inside the XML,
-   * else require rdf:RDF as first element. See also feature_assume_is_rdf
-   */
-  int feature_scanning_for_rdf_RDF;
-
-  /* FEATURE:
-   * non 0 to allow non-namespaced resource, ID etc attributes
-   * on RDF namespaced-elements
-   */
-  int feature_allow_non_ns_attributes;
-
-
-  /* FEATURE:
-   * non 0 to handle other rdf:parseType values that are not
-   * literal or resource
-   */
-  int feature_allow_other_parseTypes;
-
-
-  /* FEATURE: 
-   * non 0 if rdf:bagID is allowed.
-   * RDF/XML Syntax (Revised) 2003 forbids this.
-   */
-  int feature_allow_bagID;
-
-  /* FEATURE: 
-   * non 0 if rdf:Collection generates the <idList> rdf:type rdf:List triple.
-   * post-last call WD of RDF/XML Syntax (Revised) 2003 forbids amended
-   * the rules to not generate this.
-   */
-  int feature_allow_rdf_type_rdf_List;
-
-  /* FEATURE: 
-   * non 0 if normalizing xml:lang attribute values to lowercase.
-   * http://www.w3.org/TR/rdf-concepts/#dfn-language-identifier
-   * says this is done when language-tagged literals are in the graph.
-   */
-  int feature_normalize_language;
-
-  /* FEATURE
-   * non 0 if non-NFC literals cause a fatal error rather than
-   * a warning.
-   */
-  int feature_non_nfc_fatal;
-
-  /* FEATURE:
-   * non 0 to give a warning when other rdf:parseType values that are not
-   * Literal, Resource or Collection are seen.
-   */
-  int feature_warn_other_parseTypes;
-
-  /* FEATURE:
-   * non 0 to check rdf:ID values for duplicates
-   */
-  int feature_check_rdf_id;
+  /* Features */
+  int features[RAPTOR_FEATURE_LAST+1];
 
   /* stuff for our user */
   void *user_data;
 
-  void *fatal_error_user_data;
-  void *error_user_data;
-  void *warning_user_data;
-
-  raptor_message_handler fatal_error_handler;
-  raptor_message_handler error_handler;
-  raptor_message_handler warning_handler;
+  raptor_error_handlers error_handlers;
 
   void* unused1; /* UNUSED - re-use struct slot later needed */
 
@@ -376,6 +341,9 @@ struct raptor_parser_s {
   char *default_generate_id_handler_prefix;
   size_t default_generate_id_handler_prefix_length;
 
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
+
   /* parser specific stuff */
   void *context;
 
@@ -385,6 +353,8 @@ struct raptor_parser_s {
   raptor_namespace_handler namespace_handler;
 
   void* namespace_handler_user_data;
+
+  raptor_stringbuffer* sb;
 };
 
 
@@ -445,6 +415,9 @@ struct raptor_parser_factory_s {
 
   /* get the Accept header of a URI request (OPTIONAL) */
   const char* (*accept_header)(raptor_parser* rdf_parser);
+
+  /* non-0 if this parser needs a base URI */
+  int need_base_uri;
 };
 
 
@@ -465,6 +438,20 @@ struct raptor_serializer_s {
    */
   raptor_uri* feature_start_uri;
 
+  /* FEATURES:
+   * non NULL to override default border color
+   */
+  unsigned char *feature_resource_border;
+  unsigned char *feature_literal_border;
+  unsigned char *feature_bnode_border;
+
+  /* FEATURES:
+   * non NULL to fill with value
+   */
+  unsigned char *feature_resource_fill;
+  unsigned char *feature_literal_fill;
+  unsigned char *feature_bnode_fill;
+
   void *error_user_data;
   void *warning_user_data;
 
@@ -547,11 +534,13 @@ void raptor_serializer_register_factory(const char *name, const char *label, con
 
 /* raptor_general.c */
 
-raptor_parser_factory* raptor_parser_register_factory(const char *name, const char *label, const char *mime_type, const unsigned char *uri_string, void (*factory) (raptor_parser_factory*));
+raptor_parser_factory* raptor_parser_register_factory(const char *name, const char *label, void (*factory) (raptor_parser_factory*));
 void raptor_parser_factory_add_alias(raptor_parser_factory* factory, const char *alias);
 void raptor_parser_factory_add_mime_type(raptor_parser_factory* factory, const char* mime_type, int q);
+void raptor_parser_factory_add_uri(raptor_parser_factory* factory, const unsigned char *uri_string);
+
+unsigned char* raptor_parser_internal_generate_id(raptor_parser *rdf_parser, raptor_genid_type type, unsigned char *user_bnodeid);
 
-unsigned char* raptor_generate_id(raptor_parser *rdf_parser, const int id_for_bag, unsigned char* user_bnodeid);
 #ifdef RAPTOR_DEBUG
 void raptor_stats_print(raptor_parser *rdf_parser, FILE *stream);
 #endif
@@ -560,30 +549,33 @@ raptor_statement* raptor_statement_copy(const raptor_statement *statement);
 void raptor_free_statement(raptor_statement *statement);
 
 /* raptor_parse.c */
+raptor_parser_factory* raptor_get_parser_factory(const char *name);  
 void raptor_delete_parser_factories(void);
 void raptor_free_type_q(raptor_type_q* type_q);
 const char* raptor_parser_get_accept_header_all(void);
+int raptor_parse_uri_no_net_filter(void *user_data, raptor_uri* uri);
 
 /* raptor_general.c */
-void raptor_invoke_message_varargs(const char *type, raptor_message_handler handler, void* user_data, raptor_locator* locator, const char *message, va_list arguments);
-void raptor_invoke_simple_message_varargs(const char *type, raptor_simple_message_handler handler, void* user_data, const char *message, va_list arguments);
-
-extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...);
-extern void raptor_parser_error(raptor_parser* parser, const char *message, ...);
-extern void raptor_parser_simple_error(void* parser, const char *message, ...);
+extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+extern void raptor_parser_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+extern void raptor_parser_simple_error(void* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 extern void raptor_parser_error_message_handler(void *user_data, raptor_locator* locator, const char *message);
-extern void raptor_parser_warning(raptor_parser* parser, const char *message, ...);
-extern void raptor_parser_fatal_error_varargs(raptor_parser* parser, const char *message, va_list arguments);
+extern void raptor_parser_warning(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+extern void raptor_parser_fatal_error_varargs(raptor_parser* parser, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
 extern void raptor_parser_fatal_error_message_handler(void *user_data, raptor_locator* locator, const char *message);
-extern void raptor_parser_error_varargs(raptor_parser* parser, const char *message, va_list arguments);
-extern void raptor_parser_warning_varargs(raptor_parser* parser, const char *message, va_list arguments);
+extern void raptor_parser_error_varargs(raptor_parser* parser, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+extern void raptor_parser_warning_varargs(raptor_parser* parser, const char *message, va_list arguments)  RAPTOR_PRINTF_FORMAT(2, 0);
 void raptor_parser_warning_message_handler(void *user_data, raptor_locator* locator, const char *message);
 
+void raptor_error_handlers_init(raptor_error_handlers* error_handlers, void *fatal_error_user_data, raptor_message_handler fatal_error_handler, void *error_user_data, raptor_message_handler error_handler, void *warning_user_data, raptor_message_handler warning_handler, raptor_locator* locator);
+/* logging */
+void raptor_log_error_simple(raptor_log_level level, raptor_message_handler handler, void* handler_data, raptor_locator* locator, const char* message, ...) RAPTOR_PRINTF_FORMAT(5, 0);
+void raptor_log_error_varargs(raptor_log_level level, raptor_message_handler handler, void* handler_data, raptor_locator* locator, const char* message, va_list arguments) RAPTOR_PRINTF_FORMAT(5, 0);
+void raptor_log_error(raptor_log_level level, raptor_message_handler handler, void* handler_data, raptor_locator* locator, const char* message);
+
 
 /* raptor_parse.c */
 
-int raptor_parser_exec(raptor_parser* rdf_parser, const char *name);
-
 typedef struct raptor_rdfxml_parser_s raptor_rdfxml_parser;
 
 /* Prototypes for common expat/libxml parsing event-handling functions */
@@ -668,17 +660,28 @@ void raptor_qname_print(FILE *stream, raptor_qname* name);
 
 /* raptor_uri.c */
 void raptor_uri_init(void);
+raptor_uri* raptor_new_uri_from_rdf_ordinal(int ordinal);
 
-/* raptor_parse.c */
+/* parsers */
 void raptor_init_parser_rdfxml(void);
 void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
 void raptor_init_parser_n3(void);
 void raptor_init_parser_grddl(void);
 void raptor_init_parser_guess(void);
+void raptor_init_parser_rss(void);
+
+void raptor_terminate_parser_grddl(void);
+
+/* raptor_parse.c */
+void raptor_parsers_init(void);
+void raptor_parsers_finish(void);
+
+void raptor_parser_save_content(raptor_parser* rdf_parser, int save);
+const unsigned char* raptor_parser_get_content(raptor_parser* rdf_parser, size_t* length_p);
+
 
 /* raptor_rss.c */
-void raptor_init_parser_rss(void);
 void raptor_init_serializer_rss10(void);
 void raptor_init_serializer_atom(void);
 
@@ -689,19 +692,28 @@ raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
 void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
 unsigned char* raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p);
 
-/* raptor_serializer.c */
+/* serializers */
 void raptor_init_serializer_rdfxml(void);
 void raptor_init_serializer_ntriples(void);
+void raptor_init_serializer_dot(void);
 void raptor_init_serializer_simple(void);
 void raptor_delete_serializer_factories(void);
-void raptor_serializer_error(raptor_serializer* serializer, const char *message, ...);
-void raptor_serializer_simple_error(void* serializer, const char *message, ...);
-void raptor_serializer_error_varargs(raptor_serializer* serializer, const char *message,  va_list arguments);
-void raptor_serializer_warning(raptor_serializer* serializer, const char *message, ...);
-void raptor_serializer_warning_varargs(raptor_serializer* serializer, const char *message, va_list arguments);
 
-/* raptor_serialize_rdfxml.c */  
+/* raptor_serializer.c */
+void raptor_serializers_init(void);
+void raptor_serializers_finish(void);
+
+void raptor_serializer_error(raptor_serializer* serializer, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+void raptor_serializer_simple_error(void* serializer, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+void raptor_serializer_error_varargs(raptor_serializer* serializer, const char *message,  va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+void raptor_serializer_warning(raptor_serializer* serializer, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
+void raptor_serializer_warning_varargs(raptor_serializer* serializer, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+
+/* raptor_serialize_rdfxmla.c */  
 void raptor_init_serializer_rdfxmla(void);
+
+/* raptor_serialize_turtle.c */  
+void raptor_init_serializer_turtle(void);
   
 /* raptor_utf8.c */
 int raptor_unicode_is_namestartchar(long c);
@@ -764,6 +776,9 @@ struct  raptor_www_s {
   void *error_data;
   raptor_message_handler error_handler;
 
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
+
   /* can be filled with error location information */
   raptor_locator locator;
 
@@ -779,17 +794,13 @@ void raptor_www_libxml_init(raptor_www *www);
 void raptor_www_libxml_free(raptor_www *www);
 int raptor_www_libxml_fetch(raptor_www *www);
 
-void raptor_www_error_varargs(raptor_www *www, const char *message, va_list arguments);
-void raptor_www_error(raptor_www *www, const char *message, ...);
+void raptor_www_error_varargs(raptor_www *www, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+void raptor_www_error(raptor_www *www, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 void raptor_www_curl_init(raptor_www *www);
 void raptor_www_curl_free(raptor_www *www);
 int raptor_www_curl_fetch(raptor_www *www);
 
-void raptor_www_libwww_init(raptor_www *www);
-void raptor_www_libwww_free(raptor_www *www);
-int raptor_www_libwww_fetch(raptor_www *www);
-
 void raptor_www_libfetch_init(raptor_www *www);
 void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
@@ -820,7 +831,7 @@ struct raptor_xml_element_s {
   raptor_uri *base_uri;
 
   /* CDATA content of element and checks for mixed content */
-  unsigned char *content_cdata;
+  raptor_stringbuffer* content_cdata_sb;
   unsigned int content_cdata_length;
   /* how many cdata blocks seen */
   unsigned int content_cdata_seen;
@@ -899,14 +910,7 @@ struct raptor_sax2_s {
 
   raptor_locator *locator;
 
-  void *error_data;
-  raptor_message_handler error_handler;
-
-  void *fatal_error_data;
-  raptor_message_handler fatal_error_handler;
-
-  void *warning_data;
-  raptor_message_handler warning_handler;
+  raptor_error_handlers* error_handlers;
 
   /* New XML namespace callback */
   raptor_namespace_handler  namespace_handler;
@@ -916,6 +920,11 @@ struct raptor_sax2_s {
    */
   int feature_normalize_language;
 
+  /* FEATURE: 
+   * non 0 if network access is denied
+   */
+  int feature_no_net;
+
   /* stack of namespaces, most recently added at top */
   raptor_namespace_stack namespaces;
 
@@ -923,10 +932,10 @@ struct raptor_sax2_s {
   raptor_uri* base_uri;
 };
 
-void raptor_init_sax2(void);
-void raptor_finish_sax2(void);
+void raptor_sax2_init(void);
+void raptor_sax2_finish(void);
 
-raptor_sax2* raptor_new_sax2(void *user_data, void *error_data, raptor_message_handler error_handler, void *fatal_error_data, raptor_message_handler fatal_error_handler, void *warning_data, raptor_message_handler warning_handler);
+raptor_sax2* raptor_new_sax2(void *user_data, raptor_error_handlers* error_handlers);
 void raptor_free_sax2(raptor_sax2 *sax2);
 
 void raptor_sax2_set_start_element_handler(raptor_sax2* sax2, raptor_sax2_start_element_handler handler);
@@ -937,7 +946,6 @@ void raptor_sax2_set_comment_handler(raptor_sax2* sax2, raptor_sax2_comment_hand
 void raptor_sax2_set_unparsed_entity_decl_handler(raptor_sax2* sax2, raptor_sax2_unparsed_entity_decl_handler handler);
 void raptor_sax2_set_external_entity_ref_handler(raptor_sax2* sax2, raptor_sax2_external_entity_ref_handler handler);
 void raptor_sax2_set_namespace_handler(raptor_sax2* sax2, raptor_namespace_handler handler);
-void raptor_sax2_set_locator(raptor_sax2* sax2, raptor_locator* locator);
 void raptor_sax2_parse_start(raptor_sax2 *sax2, raptor_uri *base_uri);
 int raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer, size_t len, int is_end);
 void raptor_sax2_parse_handle_errors(raptor_sax2* sax2);
@@ -998,6 +1006,9 @@ struct raptor_uri_detail_s
   size_t path_len;
   size_t query_len;
   size_t fragment_len;
+
+  /* Flags */
+  int is_hierarchical;
 };
 
 
@@ -1036,6 +1047,96 @@ int raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer,
 /* raptor_xsd.c */
 raptor_identifier* raptor_new_identifier_from_double(double d);
 
+
+/* raptor_abbrev.c */
+
+typedef struct {
+  int ref_count;         /* count of references to this node */
+  int count_as_subject;  /* count of this blank/resource node as subject */
+  int count_as_object;   /* count of this blank/resource node as object */
+  
+  raptor_identifier_type type;  /* node type */
+  union {
+
+    struct {
+      raptor_uri *uri;
+    } resource;
+
+    struct {
+      unsigned char *string;
+      raptor_uri *datatype;
+      unsigned char *language;
+    } literal;
+
+    struct {
+      int ordinal;
+    } ordinal;
+
+    struct {
+      unsigned char *string;
+    } blank;
+    
+  } value;
+} raptor_abbrev_node;
+
+
+typedef struct {
+  raptor_abbrev_node* node;      /* node representing the subject of
+                                  * this resource */
+  raptor_abbrev_node* node_type; /* the rdf:type of this resource */
+  raptor_sequence *properties;   /* list of properties
+                                  * (predicate/object pair) of this
+                                  * subject */
+  raptor_sequence *list_items;   /* list of container elements if
+                                  * is rdf container */
+} raptor_abbrev_subject;
+
+
+raptor_abbrev_node* raptor_new_abbrev_node(raptor_identifier_type node_type, const void *node_data, raptor_uri *datatype, const unsigned char *language);
+void raptor_free_abbrev_node(raptor_abbrev_node* node);
+int raptor_abbrev_node_equals(raptor_abbrev_node* node1, raptor_abbrev_node* node2);
+int raptor_abbrev_node_matches(raptor_abbrev_node* node, raptor_identifier_type node_type, const void *node_data, raptor_uri *datatype, const unsigned char *language);
+raptor_abbrev_node* raptor_abbrev_node_lookup(raptor_sequence* nodes, raptor_identifier_type node_type, const void *node_value, raptor_uri *datatype, const unsigned char *language);
+
+raptor_abbrev_subject* raptor_new_abbrev_subject(raptor_abbrev_node* node);
+void raptor_free_abbrev_subject(raptor_abbrev_subject* subject);
+int raptor_abbrev_subject_add_property(raptor_abbrev_subject* subject, raptor_abbrev_node* predicate, raptor_abbrev_node* object);
+int raptor_abbrev_subject_add_list_element(raptor_abbrev_subject* subject, int ordinal, raptor_abbrev_node* object);
+raptor_abbrev_subject* raptor_abbrev_subject_find(raptor_sequence *sequence, raptor_identifier_type node_type, const void *node_data, int *idx);
+raptor_abbrev_subject* raptor_abbrev_subject_lookup(raptor_sequence* nodes, raptor_sequence* subjects, raptor_sequence* blanks, raptor_identifier_type node_type, const void *node_data);
+
+unsigned char *raptor_unique_id(unsigned char *base);
+
+raptor_qname* raptor_new_qname_from_resource(raptor_sequence* namespaces, raptor_namespace_stack* nstack, int* namespace_count, raptor_abbrev_node* node);
+
+
+/**
+ * raptor_turtle_writer:
+ *
+ * Raptor Turtle Writer class
+ */
+typedef struct raptor_turtle_writer_s raptor_turtle_writer;
+
+/* Turtle Writer Class (raptor_turtle_writer) */
+raptor_turtle_writer* raptor_new_turtle_writer(raptor_uri* base_uri, raptor_namespace_stack *nstack, raptor_uri_handler *uri_handler, void *uri_context, raptor_iostream* iostr, raptor_simple_message_handler error_handler, void *error_data);
+void raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer);
+void raptor_turtle_writer_raw(raptor_turtle_writer* turtle_writer, const unsigned char *s);
+void raptor_turtle_writer_raw_counted(raptor_turtle_writer* turtle_writer, const unsigned char *s, unsigned int len);
+void raptor_turtle_writer_namespace_prefix(raptor_turtle_writer* turtle_writer, raptor_namespace* ns);
+void raptor_turtle_writer_increase_indent(raptor_turtle_writer *turtle_writer);
+void raptor_turtle_writer_decrease_indent(raptor_turtle_writer *turtle_writer);
+void raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer);
+void raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer, raptor_uri* uri);
+void raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer, unsigned char *s, unsigned char* lang, raptor_uri* datatype);
+void raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer, raptor_qname* qname);
+void raptor_turtle_writer_quoted(raptor_turtle_writer* turtle_writer, unsigned char *s);
+void raptor_turtle_writer_comment(raptor_turtle_writer* turtle_writer, const unsigned char *s);
+int raptor_turtle_writer_features_enumerate(const raptor_feature feature, const char **name,  raptor_uri **uri, const char **label);
+int raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer, raptor_feature feature, int value);
+int raptor_turtle_writer_set_feature_string(raptor_turtle_writer *turtle_writer, raptor_feature feature, const unsigned char *value);
+int raptor_turtle_writer_get_feature(raptor_turtle_writer *turtle_writer, raptor_feature feature);
+const unsigned char *raptor_turtle_writer_get_feature_string(raptor_turtle_writer *turtle_writer, raptor_feature feature);
+
 /* end of RAPTOR_INTERNAL */
 #endif
 
