@@ -53,6 +53,9 @@ typedef struct {
   /* Namespace stack */
   raptor_namespace_stack *nstack;
 
+  /* the xml: namespace - this is destroyed when nstack above is deleted */
+  raptor_namespace *xml_nspace;
+
   /* the rdf: namespace - this is destroyed when nstack above is deleted */
   raptor_namespace *rdf_nspace;
 
@@ -87,9 +90,14 @@ raptor_rdfxml_serialize_init(raptor_serializer* serializer, const char *name)
   
   raptor_uri_get_handler(&uri_handler, &uri_context);
   context->nstack=raptor_new_namespaces(uri_handler, uri_context,
-                                        raptor_serializer_simple_error,
+                                        (raptor_simple_message_handler)raptor_serializer_simple_error,
                                         serializer,
                                         1);
+  context->xml_nspace=raptor_new_namespace(context->nstack,
+                                           (const unsigned char*)"xml",
+                                           (const unsigned char*)raptor_xml_namespace_uri,
+                                           0);
+
   context->rdf_nspace=raptor_new_namespace(context->nstack,
                                            (const unsigned char*)"rdf",
                                            (const unsigned char*)raptor_rdf_namespace_uri,
@@ -120,6 +128,9 @@ raptor_rdfxml_serialize_terminate(raptor_serializer* serializer)
   if(context->rdf_nspace)
     raptor_free_namespace(context->rdf_nspace);
 
+  if(context->xml_nspace)
+    raptor_free_namespace(context->xml_nspace);
+
   if(context->rdf_xml_literal_uri)
     raptor_free_uri(context->rdf_xml_literal_uri);
 
@@ -157,13 +168,17 @@ raptor_rdfxml_serialize_declare_namespace_from_namespace(raptor_serializer* seri
     raptor_namespace* ns;
     ns=(raptor_namespace*)raptor_sequence_get_at(context->namespaces, i);
 
-    /* If prefix is already declared, ignore ig */
+    /* If prefix is already declared, ignore it */
     if(!ns->prefix && !nspace->prefix)
       return 1;
     
     if(ns->prefix && nspace->prefix && 
        !strcmp((const char*)ns->prefix, (const char*)nspace->prefix))
       return 1;
+
+    if(ns->uri && nspace->uri &&
+       raptor_uri_equals(ns->uri, nspace->uri))
+      return 1;
   }
 
   nspace=raptor_new_namespace_from_uri(context->nstack,
@@ -209,16 +224,20 @@ raptor_rdfxml_serialize_start(raptor_serializer* serializer)
 
   raptor_uri_get_handler(&uri_handler, &uri_context);
 
+  if(context->xml_writer)
+    raptor_free_xml_writer(context->xml_writer);
+
   xml_writer=raptor_new_xml_writer(context->nstack,
                                    uri_handler, uri_context,
                                    serializer->iostream,
-                                   raptor_serializer_simple_error,
+                                   (raptor_simple_message_handler)raptor_serializer_simple_error,
                                    serializer,
                                    1);
   raptor_xml_writer_set_feature(xml_writer, RAPTOR_FEATURE_WRITER_XML_VERSION,
                                 serializer->xml_version);
   
   context->xml_writer=xml_writer;
+  context->written_header=0;
 
   return 0;
 }
@@ -232,6 +251,8 @@ raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
   raptor_qname *qname;
   raptor_uri *base_uri;
   int i;
+  raptor_qname **attrs;
+  int attrs_count=0;
 
   if(context->written_header)
     return;
@@ -253,6 +274,22 @@ raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
     raptor_xml_element_declare_namespace(context->rdf_RDF_element, ns);
   }
   
+  if(base_uri) {
+    const unsigned char* base_uri_string;
+
+    attrs=(raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname*));
+
+    base_uri_string=raptor_uri_as_string(base_uri);
+    attrs[attrs_count++]=raptor_new_qname_from_namespace_local_name(context->xml_nspace, (const unsigned char*)"base",  base_uri_string);
+  }
+
+  if(attrs_count)
+    raptor_xml_element_set_attributes(context->rdf_RDF_element, attrs, 
+                                      attrs_count);
+  else
+    raptor_xml_element_set_attributes(context->rdf_RDF_element, NULL, 0);
+
+
   raptor_xml_writer_start_element(xml_writer, context->rdf_RDF_element);
   raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
 
@@ -294,7 +331,8 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
     predicate_ns=context->rdf_nspace;
     sprintf((char*)ordinal_name, "_%d", *((int*)statement->predicate));
     name=ordinal_name;
-  } else {
+  } else if((statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE) ||
+     (statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE)) {
     unsigned char *p;
     size_t uri_len;
     size_t name_len=1;
@@ -331,6 +369,9 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
       free_predicate_ns=1;
     }
     raptor_free_uri(predicate_ns_uri);
+  } else {
+    raptor_serializer_error(serializer, "Cannot serialize a triple with subject node type %d\n", statement->predicate_type);
+    return 1;
   }
 
   
@@ -374,13 +415,15 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
       
       break;
       
-    case RAPTOR_IDENTIFIER_TYPE_PREDICATE:
     case RAPTOR_IDENTIFIER_TYPE_LITERAL:
     case RAPTOR_IDENTIFIER_TYPE_XML_LITERAL:
-
+      raptor_serializer_error(serializer, "Cannot serialize a triple with a literal subject\n");
+      break;
+      
+    case RAPTOR_IDENTIFIER_TYPE_PREDICATE:
     case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
     default:
-      raptor_serializer_error(serializer, "Do not know how to serialize node type %d\n", statement->subject_type);
+      raptor_serializer_error(serializer, "Cannot serialize a triple with subject node type %d\n", statement->subject_type);
   }
 
   if(attrs_count)
@@ -418,7 +461,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
         attrs[attrs_count++]=raptor_new_qname(context->nstack,
                                               (unsigned char*)"xml:lang",
                                               (unsigned char*)statement->object_literal_language,
-                                              raptor_serializer_simple_error,
+                                              (raptor_simple_message_handler)raptor_serializer_simple_error,
                                               serializer);
 
       len=strlen((const char*)statement->object);
@@ -499,7 +542,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
     case RAPTOR_IDENTIFIER_TYPE_PREDICATE:
     case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
     default:
-      raptor_serializer_error(serializer, "Do not know how to serialize node type %d\n", object_type);
+      raptor_serializer_error(serializer, "Cannot serialize a triple with object node type %d\n", object_type);
   }
 
   raptor_free_xml_element(predicate_element);
