@@ -483,11 +483,11 @@ raptor_uri_as_counted_string(raptor_uri *uri, size_t* len_p)
 unsigned char *
 raptor_uri_filename_to_uri_string(const char *filename) 
 {
-  unsigned char *buffer;
+  unsigned char *buffer=NULL;
   const char *from;
   char *to;
 #ifndef WIN32
-  char path[PATH_MAX];
+  char *path=NULL;
 #endif
   /*     "file://" ... \0 */
   size_t len=7 + sizeof(char*);
@@ -526,8 +526,29 @@ raptor_uri_filename_to_uri_string(const char *filename)
 /* others - unix: turn spaces into %20, '%' into %25 */
 
   if(*filename != '/') {
-    if(!getcwd(path, PATH_MAX))
-      return NULL;
+    size_t path_max;
+#ifdef PATH_MAX
+    path_max=PATH_MAX;
+#else
+    path_max=1024; /* an initial guess at the length */
+#endif
+    path=(char*)malloc(path_max);
+    while(1) {
+      /* malloc() failed or getcwd() succeeded */
+      if(!path || getcwd(path, path_max))
+        break;
+
+      /* failed */
+      if(errno != ERANGE)
+        break;
+
+      /* try again with a bigger buffer */
+      path_max >>= 2;
+      path=(char*)realloc(path, path_max);
+    }
+    if(!path)
+      goto path_done;
+
     strcat(path, "/");
     strcat(path, filename);
     filename=(const char*)path;
@@ -543,7 +564,7 @@ raptor_uri_filename_to_uri_string(const char *filename)
 
   buffer=(unsigned char*)RAPTOR_MALLOC(cstring, len);
   if(!buffer)
-    return NULL;
+    goto path_done;
 
   strcpy((char*)buffer, "file://");
   from=filename;
@@ -575,6 +596,12 @@ raptor_uri_filename_to_uri_string(const char *filename)
       *to++ =c;
   }
   *to='\0';
+
+  path_done:
+#ifndef WIN32
+  if(path)
+    free(path);
+#endif
   
   return buffer;
 }
@@ -713,12 +740,12 @@ raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string,
 #endif
     if(c == '%') {
       if(*from && from[1]) {
-        unsigned char hexbuf[3];
-        unsigned char *endptr=NULL;
-        hexbuf[0]=*from;
-        hexbuf[1]=from[1];
+        char hexbuf[3];
+        char *endptr=NULL;
+        hexbuf[0]=(char)*from;
+        hexbuf[1]=(char)from[1];
         hexbuf[2]='\0';
-        c=(char)strtol((const char*)hexbuf, (char**)&endptr, 16);
+        c=(char)strtol((const char*)hexbuf, &endptr, 16);
         if(endptr == &hexbuf[2])
           *to++ = c;
       }
@@ -1063,7 +1090,7 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
   raptor_uri_detail *base_detail=NULL, *reference_detail;
   const unsigned char *base, *reference_str, *base_file, *reference_file;
   unsigned char *suffix, *cur_ptr;
-  size_t base_len, reference_len, reference_file_len, prefix_len, suffix_len;
+  size_t base_len, reference_len, reference_file_len, suffix_len;
   unsigned char *result=NULL;
 
   if(!reference_uri)
@@ -1081,9 +1108,6 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
   base=raptor_uri_as_counted_string(base_uri, &base_len);
   base_detail=raptor_new_uri_detail(base);
   
-  /* The "prefix" is the scheme+authority part */
-  prefix_len = base_detail->scheme_len + base_detail->authority_len;
-  
   /* Check if the whole URIs are equal */
   if(raptor_uri_equals(base_uri, reference_uri)) {
     reference_len=0;
@@ -1091,12 +1115,14 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
   }
   
   /* Check if scheme and authority of the URIs are equal */
-  if(prefix_len != (reference_detail->scheme_len + reference_detail->authority_len))
-    goto buildresult;
-  
-
-  if(!strncmp((const char*)base_detail->buffer,
-              (const char*)reference_detail->buffer, prefix_len)) {
+  if(base_detail->scheme_len == reference_detail->scheme_len &&
+     base_detail->authority_len == reference_detail->authority_len &&
+     !strncmp((const char*)base_detail->scheme, 
+              (const char*)reference_detail->scheme,
+              base_detail->scheme_len) &&
+     !strncmp((const char*)base_detail->authority, 
+              (const char*)reference_detail->authority,
+              base_detail->authority_len)) {
     
     if(!base_detail->path)
       goto buildresult;
@@ -1285,6 +1311,27 @@ raptor_uri_to_string(raptor_uri *uri)
 }
 
 
+/**
+ * raptor_new_uri_from_rdf_ordinal:
+ * @ordinal: integer rdf:_n
+ * 
+ * Internal - convert an integer rdf:_n ordinal to the resource URI
+ * 
+ * Return value: new URI object or NULL on failure
+ **/
+raptor_uri*
+raptor_new_uri_from_rdf_ordinal(int ordinal)
+{
+  /* 55 = strlen(rdf namespace URI) + _ + 10-digit number + \0 */
+  unsigned char uri_string[55];
+  strncpy((char*)uri_string, (const char*)raptor_rdf_namespace_uri, 
+          raptor_rdf_namespace_uri_len);
+  sprintf((char*)uri_string+raptor_rdf_namespace_uri_len, "_%d",
+          ordinal);
+  return raptor_new_uri(uri_string);
+}
+
+
 #ifdef STANDALONE
 
 #include <stdio.h>
@@ -1512,6 +1559,7 @@ main(int argc, char *argv[])
   failures += assert_uri_to_relative("http://example.org/base#", "http://www.foo.org", "http://www.foo.org");
   failures += assert_uri_to_relative("http://example.org", "http://a.example.org/", "http://a.example.org/");
   failures += assert_uri_to_relative("http://example.org", "http://a.example.org", "http://a.example.org");
+  failures += assert_uri_to_relative("http://abcdefgh.example.org/foo/bar/", "http://ijklmnop.example.org/", "http://ijklmnop.example.org/");
 
   return failures ;
 }
