@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdfdump.c - Raptor RDF Parser example code 
+ * rapper.c - Raptor RDF Parsing and Serializing utility
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -60,7 +60,7 @@ extern int optind;
 extern char *optarg;
 #endif
 
-static void print_statements(void *user_data, const raptor_statement *statement);
+static void print_triples(void *user_data, const raptor_statement *statement);
 static void print_namespaces(void* user_data, raptor_namespace *nspace);
 static void relay_namespaces(void* user_data, raptor_namespace *nspace);
 int main(int argc, char *argv[]);
@@ -76,7 +76,7 @@ static int quiet=0;
 /* just count, no printing */
 static int count=0;
 
-static int statement_count=0;
+static int triple_count=0;
 
 static raptor_serializer* serializer=NULL;
 
@@ -87,13 +87,13 @@ static int reported_guess=0;
 static int report_namespace=0;
 
 static
-void print_statements(void *user_data, const raptor_statement *statement) 
+void print_triples(void *user_data, const raptor_statement *triple) 
 {
   raptor_parser* rdf_parser=(raptor_parser*)user_data;
-  statement_count++;
+  triple_count++;
 
   if(guess && !quiet && !reported_guess) {
-     fprintf(stdout, "%s: Guessed parser name '%s'\n", program,
+     fprintf(stderr, "%s: Guessed parser name '%s'\n", program,
              raptor_get_name(rdf_parser));
      reported_guess=1;
   }
@@ -104,14 +104,14 @@ void print_statements(void *user_data, const raptor_statement *statement)
 
   /* replace newlines with spaces if object is a literal string */
   if(replace_newlines && 
-     statement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL) {
+     triple->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL) {
     char *s;
-    for(s=(char*)statement->object; *s; s++)
+    for(s=(char*)triple->object; *s; s++)
       if(*s == '\n')
         *s=' ';
   }
 
-  raptor_serialize_statement(serializer, statement);
+  raptor_serialize_statement(serializer, triple);
   return;
 }
 
@@ -138,8 +138,10 @@ relay_namespaces(void* user_data, raptor_namespace *nspace)
 {
   raptor_serializer* rdf_serializer=(raptor_serializer*)user_data;
 
+  if(report_namespace)
+    print_namespaces(user_data, nspace);
+
   raptor_serialize_set_namespace_from_namespace(rdf_serializer, nspace);
-  return;
 }
 
 
@@ -157,7 +159,7 @@ relay_namespaces(void* user_data, raptor_namespace *nspace)
 #endif
 
 
-#define GETOPT_STRING "nsaf:ghrqo:wecm:i:v"
+#define GETOPT_STRING "nsaf:ghrqo:wecm:i:vt"
 
 #ifdef HAVE_GETOPT_LONG
 #define SHOW_NAMESPACES_FLAG 0x100
@@ -178,6 +180,7 @@ static struct option long_options[] =
   {"replace-newlines", 0, 0, 'r'},
   {"scan", 0, 0, 's'},
   {"show-namespaces", 0, 0, SHOW_NAMESPACES_FLAG},
+  {"trace", 0, 0, 't'},
   {"ignore-warnings", 0, 0, 'w'},
   {"version", 0, 0, 'v'},
   {NULL, 0, 0, 0}
@@ -191,7 +194,7 @@ static int warning_count=0;
 static int ignore_warnings=0;
 static int ignore_errors=0;
 
-static const char *title_format_string="Raptor RDF parsing and serializing utility %s\n";
+static const char *title_format_string="Raptor RDF syntax parsing and serializing utility %s\n";
 
 
 static void
@@ -241,6 +244,15 @@ rdfdump_free_namespace_decl(void* data) {
 }
 
 
+static int 
+rapper_uri_trace(void *user_data, raptor_uri* uri)
+{
+  fprintf(stderr, "%s: Processing URI %s\n", program,
+          raptor_uri_as_string(uri));
+  return 0;
+}
+
+
 int
 main(int argc, char *argv[]) 
 {
@@ -255,7 +267,8 @@ main(int argc, char *argv[])
   int strict_mode=0;
   int usage=0;
   int help=0;
-  raptor_uri *base_uri;
+  int trace=0;
+  raptor_uri *base_uri=NULL;
   raptor_uri *uri;
   char *p;
   char *filename=NULL;
@@ -308,27 +321,29 @@ main(int argc, char *argv[])
             int i;
             
             fprintf(stderr, "%s: Valid parser features are:\n", program);
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
               if(!raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
                 const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
-                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                fprintf(stderr, "  %-20s  %s%s\n", feature_name, feature_label, 
                        feature_type);
               }
             }
             fprintf(stderr, "%s: Valid serializer features are:\n", program);
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
               if(!raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
                 const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
-                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                fprintf(stderr, "  %-20s  %s%s\n", feature_name, feature_label, 
                        feature_type);
               }
             }
             fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
             fputs("\nA feature of the form xmlns:PREFIX=\"URI\" can be used to declare output\nnamespace prefixes and names for serializing using an XML-style syntax\nEither or both of PREFIX or URI can be omitted such as -f xmlns=\"URI\"\nThis form can be repeated for multiple declarations.\n", stderr);
+
+            raptor_finish();
             exit(0);
           } else if(!strncmp(optarg, "xmlns", 5)) {
             struct namespace_decl *nd;
@@ -339,6 +354,8 @@ main(int argc, char *argv[])
               fprintf(stderr, "%s: Bad xmlns syntax in '%s'\n", program, 
                       optarg);
               rdfdump_free_namespace_decl(nd);
+
+              raptor_finish();
               exit(0);
             }
 
@@ -350,7 +367,7 @@ main(int argc, char *argv[])
             int i;
             size_t arg_len=strlen(optarg);
             
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
@@ -374,7 +391,7 @@ main(int argc, char *argv[])
               }
             }
             
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
@@ -424,6 +441,10 @@ main(int argc, char *argv[])
         scanning=1;
         break;
 
+      case 't':
+        trace=1;
+        break;
+
       case 'q':
         quiet=1;
         break;
@@ -462,7 +483,7 @@ main(int argc, char *argv[])
               const char *help_label;
               if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
                 break;
-              printf("  %-12s for %s\n", help_name, help_label);
+              fprintf(stderr, "  %-12s for %s\n", help_name, help_label);
             }
             usage=1;
             break;
@@ -486,7 +507,7 @@ main(int argc, char *argv[])
               const char *help_label;
               if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
                 break;
-              printf("  %-12s for %s\n", help_name, help_label);
+              fprintf(stderr, "  %-12s for %s\n", help_name, help_label);
             }
             usage=1;
             break;
@@ -506,6 +527,8 @@ main(int argc, char *argv[])
       case 'v':
         fputs(raptor_version_string, stdout);
         fputc('\n', stdout);
+
+        raptor_finish();
         exit(0);
 
 #ifdef SHOW_NAMESPACES_FLAG
@@ -526,66 +549,83 @@ main(int argc, char *argv[])
   if(usage) {
     if(usage>1) {
       fprintf(stderr, title_format_string, raptor_version_string);
-      fputs(raptor_short_copyright_string, stderr);
+      fputs("Raptor home page: ", stderr);
+      fputs(raptor_home_url_string, stderr);
       fputc('\n', stderr);
+      fputs(raptor_copyright_string, stderr);
+      fputs("\nLicense: ", stderr);
+      fputs(raptor_license_string, stderr);
+      fputs("\n\n", stderr);
     }
     fprintf(stderr, "Try `%s " HELP_ARG(h, help) "' for more information.\n",
                     program);
+
+    raptor_finish();
     exit(1);
   }
 
   if(help) {
     int i;
     
-    printf("Usage: %s [OPTIONS] <source URI> [base URI]\n", program);
     printf(title_format_string, raptor_version_string);
-    puts(raptor_short_copyright_string);
-    puts("Parse RDF content at the source URI into RDF triples.");
-    puts("\nMain options:");
-    puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
+    puts("Parse RDF content at the source into RDF triples.");
+    printf("Usage: %s [OPTIONS] SOURCE-URI [BASE-URI]\n\n", program);
+
+    fputs(raptor_copyright_string, stdout);
+    fputs("\nLicense: ", stdout);
+    puts(raptor_license_string);
+    fputs("Raptor home page: ", stdout);
+    puts(raptor_home_url_string);
+
+    puts("\nArguments:");
+    puts("  SOURCE-URI is a filename, URI or '-' for standard input (stdin).");
+    puts("  BASE-URI   is the source URI if omitted or '-' for no base URI.");
+
+    puts("\nSyntax options:");
     puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
         break;
-      printf("    %-14s          %s", help_name, help_label);
+      printf("    %-14s  %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
         putchar('\n');
     }
+    putchar('\n');
     puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set the output format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
         break;
-      printf("    %-14s          %s", help_name, help_label);
+      printf("    %-14s  %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
-    puts("\nAdditional options:");
+    puts("\nGeneral options:");
     puts(HELP_TEXT("c", "count           ", "Count triples only - do not print them."));
     puts(HELP_TEXT("e", "ignore-errors   ", "Ignore error messages"));
     puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser or serializer features" HELP_PAD "Use `-f help' for a list of valid features"));
     puts(HELP_TEXT("g", "guess           ", "Guess the input syntax (same as -i guess)"));
+    puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
+    puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("r", "replace-newlines", "Replace newlines with spaces in literals"));
     puts(HELP_TEXT("s", "scan            ", "Scan for <rdf:RDF> element in source"));
+    puts(HELP_TEXT("t", "trace           ", "Trace URIs retrieved during parsing"));
 #ifdef SHOW_NAMESPACES_FLAG
     puts(HELP_TEXT_LONG("show-namespaces ", "Show namespaces as they are declared"));
 #endif
     puts(HELP_TEXT("w", "ignore-warnings ", "Ignore warning messages"));
     puts(HELP_TEXT("v", "version         ", "Print the Raptor version"));
     puts("\nReport bugs to http://bugs.librdf.org/");
-    puts("Raptor home page: http://librdf.org/raptor/");
 
     raptor_finish();
-
     exit(0);
   }
 
@@ -630,11 +670,13 @@ main(int argc, char *argv[])
   if(!base_uri_string) {
     base_uri=raptor_uri_copy(uri);
   } else {
-    base_uri=raptor_new_uri(base_uri_string);
-    if(!base_uri) {
-      fprintf(stderr, "%s: Failed to create URI for %s\n",
-              program, base_uri_string);
-      return(1);
+    if(strcmp((const char*)base_uri_string, "-")) {
+      base_uri=raptor_new_uri(base_uri_string);
+      if(!base_uri) {
+        fprintf(stderr, "%s: Failed to create URI for %s\n",
+                program, base_uri_string);
+        return(1);
+      }
     }
   }
 
@@ -664,24 +706,27 @@ main(int argc, char *argv[])
                                      parser_feature,
                                      parser_feature_string_value);
 
+  if(trace)
+    raptor_parser_set_uri_filter(rdf_parser, rapper_uri_trace, NULL);
+
 
   if(!quiet) {
     if (filename) {
       if(base_uri_string)
-        fprintf(stdout, "%s: Parsing file %s with base URI %s\n", program,
+        fprintf(stderr, "%s: Parsing file %s with base URI %s\n", program,
                 filename, base_uri_string);
       else
-        fprintf(stdout, "%s: Parsing file %s\n", program, filename);
+        fprintf(stderr, "%s: Parsing file %s\n", program, filename);
     } else {
       if(base_uri_string)
-        fprintf(stdout, "%s: Parsing URI %s with base URI %s\n", program,
+        fprintf(stderr, "%s: Parsing URI %s with base URI %s\n", program,
                 uri_string, base_uri_string);
       else
-        fprintf(stdout, "%s: Parsing URI %s\n", program, uri_string);
+        fprintf(stderr, "%s: Parsing URI %s\n", program, uri_string);
     }
   }
   
-  raptor_set_statement_handler(rdf_parser, rdf_parser, print_statements);
+  raptor_set_statement_handler(rdf_parser, rdf_parser, print_triples);
 
   if(report_namespace)
     raptor_set_namespace_handler(rdf_parser, rdf_parser, print_namespaces);
@@ -752,8 +797,8 @@ main(int argc, char *argv[])
   
 
   if(!quiet)
-    fprintf(stdout, "%s: Parsing returned %d statements\n", program,
-            statement_count);
+    fprintf(stderr, "%s: Parsing returned %d triple%s\n", program,
+            triple_count, (triple_count == 1 ? "" : "s"));
 
   raptor_free_uri(base_uri);
   if(uri)
