@@ -2,9 +2,9 @@
  *
  * rasqal.h - Rasqal RDF Query library interfaces and definition
  *
- * $Id: rasqal.h 10872 2006-04-28 05:49:16Z dajobe $
+ * $Id: rasqal.h 12095 2007-04-22 06:49:06Z dajobe $
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -41,10 +41,14 @@ extern "C" {
  *
  */
 #ifdef WIN32
-#  ifdef RASQAL_INTERNAL
-#    define RASQAL_API _declspec(dllexport)
+#  ifdef RASQAL_STATIC
+#    define RASQAL_API
 #  else
-#    define RASQAL_API _declspec(dllimport)
+#    ifdef RASQAL_INTERNAL
+#      define RASQAL_API _declspec(dllexport)
+#    else
+#      define RASQAL_API _declspec(dllimport)
+#    endif
 #  endif
 #else
 #  define RASQAL_API
@@ -86,6 +90,10 @@ RASQAL_API
 extern const unsigned int rasqal_version_release;
 RASQAL_API
 extern const unsigned int rasqal_version_decimal;
+RAPTOR_API
+extern const char * const rasqal_license_string;
+RAPTOR_API
+extern const char * const rasqal_home_url_string;
 
 
 /* Public structures */
@@ -104,6 +112,13 @@ typedef struct rasqal_query_s rasqal_query;
  */
 typedef struct rasqal_query_results_s rasqal_query_results;
 
+/**
+ * rasqal_query_results_formatter:
+ *
+ * Rasqal query results formatter class.
+ */
+typedef struct rasqal_query_results_formatter_s rasqal_query_results_formatter;
+
 typedef struct rasqal_literal_s rasqal_literal;
 
 /**
@@ -115,6 +130,7 @@ typedef struct rasqal_graph_pattern_s rasqal_graph_pattern;
 
 /**
  * rasqal_feature:
+ * @RASQAL_FEATURE_NO_NET: Deny network requests.
  * @RASQAL_FEATURE_LAST: Internal.
  *
  * Query features.
@@ -122,7 +138,8 @@ typedef struct rasqal_graph_pattern_s rasqal_graph_pattern;
  * None currently defined.
  */
 typedef enum {
-  RASQAL_FEATURE_LAST
+  RASQAL_FEATURE_NO_NET,
+  RASQAL_FEATURE_LAST = RASQAL_FEATURE_NO_NET
 } rasqal_feature;
 
 
@@ -164,12 +181,16 @@ typedef enum {
 } rasqal_variable_type;
 
 
+/* forward reference */
+struct rasqal_expression_s;
+
 /**
  * rasqal_variable:
  * @name: Variable name.
  * @value: Variable value or NULL if unbound.
  * @offset: Internal.
  * @type: Variable type.
+ * @expression: Expression when the variable is a computed SELECT expression 
  *
  * Binding between a variable name and a value.
  *
@@ -181,6 +202,7 @@ typedef struct {
   rasqal_literal* value;
   int offset;
   rasqal_variable_type type;
+  struct rasqal_expression_s* expression;
 } rasqal_variable;
 
 
@@ -365,6 +387,10 @@ struct rasqal_literal_s {
  * @RASQAL_EXPR_CAST: Expression for cast literal A to type B.
  * @RASQAL_EXPR_ORDER_COND_ASC: Expression for SPARQL order condition ascending.
  * @RASQAL_EXPR_ORDER_COND_DESC: Expression for SPARQL order condition descending.
+ * @RASQAL_EXPR_GROUP_COND_ASC: Expression for LAQRS group condition ascending.
+ * @RASQAL_EXPR_GROUP_COND_DESC: Expression for LAQRS group condition descending.
+ * @RASQAL_EXPR_COUNT: Expression for LAQRS select COUNT()
+ * @RASQAL_EXPR_VARSTAR: Expression for LAQRS select Variable *
  * @RASQAL_EXPR_UNKNOWN: Internal
  * @RASQAL_EXPR_LAST: Internal
  *
@@ -409,8 +435,12 @@ typedef enum {
   RASQAL_EXPR_ORDER_COND_DESC,
   RASQAL_EXPR_LANGMATCHES,
   RASQAL_EXPR_REGEX,
+  RASQAL_EXPR_GROUP_COND_ASC,
+  RASQAL_EXPR_GROUP_COND_DESC,
+  RASQAL_EXPR_COUNT,
+  RASQAL_EXPR_VARSTAR,
   /* internal */
-  RASQAL_EXPR_LAST= RASQAL_EXPR_REGEX
+  RASQAL_EXPR_LAST= RASQAL_EXPR_VARSTAR
 } rasqal_op;
 
 
@@ -428,7 +458,6 @@ struct rasqal_expression_s {
   struct rasqal_expression_s* arg2;
   struct rasqal_expression_s* arg3; /* optional 3rd arg for EXPR_REGEX */
   rasqal_literal* literal;
-  rasqal_variable* variable;
   unsigned char *value; /* UTF-8 value */
 
   /* for extension function qname(args...) and cast-to-uri */
@@ -503,6 +532,8 @@ typedef unsigned char* (*rasqal_generate_bnodeid_handler)(rasqal_query* query, v
  * @RASQAL_QUERY_VERB_CONSTRUCT: SPARQL query construct verb.
  * @RASQAL_QUERY_VERB_DESCRIBE: SPARQL query describe verb.
  * @RASQAL_QUERY_VERB_ASK: SPARQL query ask verb.
+ * @RASQAL_QUERY_VERB_DELETE: LAQRS query delete verb.
+ * @RASQAL_QUERY_VERB_INSERT: LAQRS query insert verb.
  * @RASQAL_QUERY_VERB_UNKNOWN: Internal
  * @RASQAL_QUERY_VERB_LAST: Internal
  *
@@ -516,9 +547,11 @@ typedef enum {
   RASQAL_QUERY_VERB_CONSTRUCT = 2,
   RASQAL_QUERY_VERB_DESCRIBE  = 3,
   RASQAL_QUERY_VERB_ASK       = 4,
+  RASQAL_QUERY_VERB_DELETE    = 5,
+  RASQAL_QUERY_VERB_INSERT    = 6,
 
   /* internal */
-  RASQAL_QUERY_VERB_LAST=RASQAL_QUERY_VERB_ASK
+  RASQAL_QUERY_VERB_LAST=RASQAL_QUERY_VERB_INSERT
 } rasqal_query_verb;
 
 
@@ -569,6 +602,15 @@ void rasqal_init(void);
 RASQAL_API
 void rasqal_finish(void);
 
+/* Features */
+RASQAL_API
+int rasqal_features_enumerate(const rasqal_feature feature, const char **name, raptor_uri **uri, const char **label);
+RASQAL_API
+unsigned int rasqal_get_feature_count(void);
+RASQAL_API
+rasqal_feature rasqal_feature_from_uri(raptor_uri *uri);
+RASQAL_API
+int rasqal_feature_value_type(const rasqal_feature feature);
 
 RASQAL_API
 int rasqal_languages_enumerate(const unsigned int counter, const char **name, const char **label, const unsigned char **uri_string);
@@ -597,7 +639,13 @@ void rasqal_query_set_error_handler(rasqal_query* query, void *user_data, raptor
 RASQAL_API
 void rasqal_query_set_warning_handler(rasqal_query* query, void *user_data, raptor_message_handler handler);
 RASQAL_API
-void rasqal_query_set_feature(rasqal_query* query, rasqal_feature feature, int value);
+int rasqal_query_set_feature(rasqal_query* query, rasqal_feature feature, int value);
+RASQAL_API
+int rasqal_query_set_feature_string(rasqal_query *query, rasqal_feature feature, const unsigned char *value);
+RASQAL_API
+int rasqal_query_get_feature(rasqal_query *query, rasqal_feature feature);
+RASQAL_API
+const unsigned char* rasqal_query_get_feature_string(rasqal_query *query, rasqal_feature feature);
 RASQAL_API
 void rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, char *prefix, int base);
 RASQAL_API
@@ -610,7 +658,11 @@ int rasqal_query_get_wildcard(rasqal_query* query);
 RASQAL_API
 int rasqal_query_get_distinct(rasqal_query* query);
 RASQAL_API
-void rasqal_query_set_distinct(rasqal_query* query, int is_distinct);
+void rasqal_query_set_distinct(rasqal_query* query, int distinct_mode);
+RASQAL_API
+int rasqal_query_get_explain(rasqal_query* query);
+RASQAL_API
+void rasqal_query_set_explain(rasqal_query* query, int is_explain);
 RASQAL_API
 int rasqal_query_get_limit(rasqal_query* query);
 RASQAL_API
@@ -656,6 +708,10 @@ raptor_sequence* rasqal_query_get_order_conditions_sequence(rasqal_query* query)
 RASQAL_API
 rasqal_expression* rasqal_query_get_order_condition(rasqal_query* query, int idx);
 RASQAL_API
+raptor_sequence* rasqal_query_get_group_conditions_sequence(rasqal_query* query);
+RASQAL_API
+rasqal_expression* rasqal_query_get_group_condition(rasqal_query* query, int idx);
+RASQAL_API
 raptor_sequence* rasqal_query_get_construct_triples_sequence(rasqal_query* query);
 RASQAL_API
 rasqal_triple* rasqal_query_get_construct_triple(rasqal_query* query, int idx);
@@ -753,9 +809,30 @@ raptor_statement* rasqal_query_results_get_triple(rasqal_query_results *query_re
 RASQAL_API
 int rasqal_query_results_next_triple(rasqal_query_results *query_results);
 
+/* Syntax result format */
+RASQAL_API
+int rasqal_query_results_is_syntax(rasqal_query_results* query_results);
+
 RASQAL_API
 int rasqal_query_results_write(raptor_iostream *iostr, rasqal_query_results *results, raptor_uri *format_uri, raptor_uri *base_uri);
 
+RASQAL_API
+int rasqal_query_results_formats_enumerate_full(const unsigned int counter, const char **name, const char **label, const unsigned char **uri_string, const char **mime_type);
+RASQAL_API
+int rasqal_query_results_formats_enumerate(const unsigned int counter, const char **name, const char **label, const unsigned char **uri_string);
+RASQAL_API
+int rasqal_query_results_formats_check(const char *name, raptor_uri* uri, const char *mime_type);
+RASQAL_API
+rasqal_query_results_formatter* rasqal_new_query_results_formatter(const char *name, raptor_uri* uri);
+RASQAL_API
+rasqal_query_results_formatter* rasqal_new_query_results_formatter_by_mime_type(const char *mime_type);
+RASQAL_API
+void rasqal_free_query_results_formatter(rasqal_query_results_formatter* formatter);
+RASQAL_API
+int rasqal_query_results_formatter_write(raptor_iostream *iostr, rasqal_query_results_formatter* formatter, rasqal_query_results* results, raptor_uri *base_uri);
+RASQAL_API
+const char* rasqal_query_results_formatter_get_mime_type(rasqal_query_results_formatter *formatter);
+
 RASQAL_API
 int rasqal_query_iostream_write_escaped_counted_string(rasqal_query* query, raptor_iostream* iostr, const unsigned char* string, size_t len);
 RASQAL_API
@@ -786,6 +863,8 @@ typedef enum {
 
 /* Expression class */
 RASQAL_API
+rasqal_expression* rasqal_new_0op_expression(rasqal_op op);
+RASQAL_API
 rasqal_expression* rasqal_new_1op_expression(rasqal_op op, rasqal_expression* arg);
 RASQAL_API
 rasqal_expression* rasqal_new_2op_expression(rasqal_op op, rasqal_expression* arg1, rasqal_expression* arg2);
@@ -993,7 +1072,11 @@ typedef struct
 
   rasqal_triple_parts parts;
 
+  /* non-0 if the associated triple pattern contains no variables */
   int is_exact;
+
+  /* non-0 if the triple pattern has been fully executed */
+  int executed;
 } rasqal_triple_meta;
 
 
@@ -1053,7 +1136,6 @@ void rasqal_set_triples_source_factory(void (*register_fn)(rasqal_triples_source
  * @arg2: Internal
  * @arg3: Internal
  * @literal: Internal
- * @variable: Internal
  * @value: Internal
  * @name: Internal
  * @args: Internal
