@@ -2,9 +2,9 @@
  *
  * rasqal_internal.h - Rasqal RDF Query library internals
  *
- * $Id: rasqal_internal.h 10899 2006-04-30 06:36:13Z dajobe $
+ * $Id: rasqal_internal.h 12086 2007-04-22 06:14:40Z dajobe $
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -43,6 +43,13 @@ extern "C" {
 #include <dmalloc.h>
 #endif
 
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index) \
+  __attribute__((__format__(__printf__, string_index, first_to_check_index)))
+#else
+#define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index)
+#endif
+
 /* Can be over-ridden or undefined in a config.h file or -Ddefine */
 #ifndef RASQAL_INLINE
 #define RASQAL_INLINE inline
@@ -131,37 +138,14 @@ struct rasqal_graph_pattern_s {
   raptor_sequence* triples;          /* ... rasqal_triple*         */
   raptor_sequence* graph_patterns;   /* ... rasqal_graph_pattern*  */
 
-  /* An array of items, one per triple in the pattern graph */
-  rasqal_triple_meta* triple_meta;
-
-  int column;
-
   int start_column;
   int end_column;
 
-  /* first graph_pattern in sequence with flags RASQAL_TRIPLE_FLAGS_OPTIONAL */
-  int optional_graph_pattern;
-
-  /* current position in the sequence */
-  int current_graph_pattern;
-
   /* Max optional graph pattern allowed so far to stop backtracking
    * going over old graph patterns
    */
   int max_optional_graph_pattern;
 
-  /* Count of all optional matches for the current mandatory matches */
-  int optional_graph_pattern_matches_count;
-
-  /* true when this graph pattern matched last time */
-  int matched;
-
-  /* true when an optional graph pattern finished last time round */
-  int finished;
-
-  /* Number of matches returned */
-  int matches_returned;
-
   raptor_sequence *constraints; /* ... rasqal_expression*          */
   /* the expression version of the sequence of constraints above - this is
    * where the constraints are freed
@@ -172,14 +156,11 @@ struct rasqal_graph_pattern_s {
   int gp_index;
 };
 
-rasqal_graph_pattern* rasqal_new_graph_pattern(rasqal_query* query);
-rasqal_graph_pattern* rasqal_new_graph_pattern_from_triples(rasqal_query* query, raptor_sequence* triples, int start_column, int end_column, rasqal_graph_pattern_operator op);
+rasqal_graph_pattern* rasqal_new_basic_graph_pattern(rasqal_query* query, raptor_sequence* triples, int start_column, int end_column);
 rasqal_graph_pattern* rasqal_new_graph_pattern_from_sequence(rasqal_query* query, raptor_sequence* graph_patterns, rasqal_graph_pattern_operator op);
 void rasqal_free_graph_pattern(rasqal_graph_pattern* gp);
-void rasqal_graph_pattern_init(rasqal_graph_pattern* gp);
 void rasqal_graph_pattern_adjust(rasqal_graph_pattern* gp, int offset);
 void rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern, rasqal_literal* origin);
-void rasqal_graph_pattern_add_triples(rasqal_graph_pattern* gp, raptor_sequence* triples, int start_column, int end_column, rasqal_graph_pattern_operator op);
 int rasqal_reset_triple_meta(rasqal_triple_meta* m);
 
 
@@ -208,7 +189,12 @@ struct rasqal_query_s {
   raptor_sequence* optional_triples; /* ... rasqal_triple*  SPARQL */
   raptor_sequence* describes;   /* ... rasqal_literal* (var or URIs) SPARQL */
 
-  /* non-0 if DISTINCT was seen in the query (SELECT or DESCRIBE) */
+  /* DISTINCT mode:
+   * 0 if not given
+   * 1 if DISTINCT: ensure solutions are unique
+   * 2 if SPARQL REDUCED: permit elimination of some non-unique solutions 
+   * otherwise undefined
+   */
   int distinct;
 
   /* result limit LIMIT (>=0) or <0 if not given */
@@ -226,8 +212,9 @@ struct rasqal_query_s {
    * in selects, triples, constraints and anonymous (no name, cannot
    * be selected or refered to).  An array of size variables_count
    *
-   * The first select_variables_count of this array are from the selects
-   * and are typically returned to the user.
+   * The first select_variables_count of this array are from the
+   * selects.  The selected variables only are typically returned to
+   * the user.
    *
    * Anonymous variables appear at the end of the 'variables' array but
    * are taken from the anon_variables_sequence.
@@ -250,13 +237,13 @@ struct rasqal_query_s {
   int anon_variables_count;
 
   /* array of variable names to bind or NULL if no variables wanted
-   * (size select_variables_count+1, last NULL)
+   * (size (select_variables_count OR construct_variables_count)+1, last NULL)
    * indexes into the names in variables_sequence above.
    */
   const unsigned char** variable_names;
 
   /* array of result binding values, per result or NULL if no variables wanted
-   * (size select_variables_count) 
+   * (size select_variables_count or construct_variables_count) 
    * indexes into the values in variables_sequence above, per-binding
    */
   rasqal_literal** binding_values;
@@ -308,11 +295,32 @@ struct rasqal_query_s {
   /* sequence of order condition expressions */
   raptor_sequence* order_conditions_sequence;
 
+  /* sequence of group by condition expressions */
+  raptor_sequence* group_conditions_sequence;
+
   /* INTERNAL rasqal_literal_compare / rasqal_expression_evaluate flags */
   int compare_flags;
 
   /* Number of graph patterns in this query */
   int graph_pattern_count;
+  
+  /* Graph pattern shared pointers by gp index (after prepare) */
+  raptor_sequence* graph_patterns_sequence;
+
+  /* Features */
+  int features[RASQAL_FEATURE_LAST+1];
+
+  /* Name of requested query results syntax.  If present, this
+   * is the name used for constructing a rasqal_query_formatter
+   * from the results.
+   */
+  const char* query_results_formatter_name;
+
+  /* EXPLAIN was given */
+  int explain;
+
+  /* generated counter - increments at every generation */
+  int genid_counter;
 };
 
 
@@ -385,12 +393,20 @@ typedef struct {
 } rasqal_query_result_row;
 
 
+typedef struct rasqal_map_s rasqal_map;
+
 /* The execution data here is a sequence of
  * rasqal_graph_pattern_data execution data of size
  * query->graph_pattern_count with each rasqal_graph_pattern_data
  */
 typedef struct {
   raptor_sequence* seq;
+
+  /* offset into stored results sequence */
+  int offset;
+
+  /* for ordering results during execution */
+  rasqal_map* map;
 } rasqal_engine_execution_data;
 
 
@@ -452,19 +468,50 @@ struct rasqal_query_results_s {
 };
     
 
+typedef int (*rasqal_query_results_writer)(raptor_iostream *iostr,
+                                           rasqal_query_results* results,
+                                           raptor_uri *base_uri);
+
+typedef struct {
+  /* query results format name */
+  const char* name;
+
+  /* query results format name */
+  const char* label;
+
+  /* query results format URI (or NULL) */
+  const unsigned char* uri_string;
+  
+  rasqal_query_results_writer writer;
+
+  /* MIME Type of the constructed syntax */
+  const char* mime_type;
+} rasqal_query_results_format_factory;
+
+
+/*
+ * A query results formatter for some query_results
+ */
+struct rasqal_query_results_formatter_s {
+  rasqal_query_results_format_factory* factory;
+
+  const char *mime_type;
+};
+
+
 /* rasqal_general.c */
 char* rasqal_vsnprintf(const char* message, va_list arguments);
 
 void rasqal_query_engine_register_factory(const char* name, const char* label, const char* alias, const unsigned char* uri_string, void (*factory) (rasqal_query_engine_factory*));
 rasqal_query_engine_factory* rasqal_get_query_engine_factory (const char* name, const unsigned char* uri);
 
-void rasqal_query_fatal_error(rasqal_query* query, const char* message, ...);
-void rasqal_query_fatal_error_varargs(rasqal_query* query, const char* message, va_list arguments);
-void rasqal_query_error(rasqal_query* query, const char* message, ...);
-void rasqal_query_simple_error(void* query, const char* message, ...);
-void rasqal_query_error_varargs(rasqal_query* query, const char* message, va_list arguments);
-void rasqal_query_warning(rasqal_query* query, const char* message, ...);
-void rasqal_query_warning_varargs(rasqal_query* query, const char* message, va_list arguments);
+void rasqal_query_fatal_error(rasqal_query* query, const char* message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_query_fatal_error_varargs(rasqal_query* query, const char* message, va_list arguments) RASQAL_PRINTF_FORMAT(2, 0);
+void rasqal_query_error(rasqal_query* query, const char* message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_query_simple_error(void* query, const char* message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_query_error_varargs(rasqal_query* query, const char* message, va_list arguments) RASQAL_PRINTF_FORMAT(2, 0);
+void rasqal_query_warning(rasqal_query* query, const char* message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_query_warning_varargs(rasqal_query* query, const char* message, va_list arguments) RASQAL_PRINTF_FORMAT(2, 0);
 
 const char* rasqal_basename(const char* name);
 
@@ -479,6 +526,7 @@ void rasqal_init_query_engine_rdql (void);
 
 /* sparql_parser.y */
 void rasqal_init_query_engine_sparql (void);
+void rasqal_init_query_engine_laqrs (void);
 
 /* rasqal_engine.c */
 int rasqal_engine_sequence_has_qname(raptor_sequence* seq);
@@ -493,7 +541,6 @@ int rasqal_engine_assign_variables(rasqal_query* rq);
 int rasqal_engine_prepare(rasqal_query* query);
 int rasqal_engine_execute_init(rasqal_query_results* query_results);
 int rasqal_engine_execute_finish(rasqal_query_results* query_results);
-int rasqal_engine_run(rasqal_query_results* query_results);
 void rasqal_engine_join_graph_patterns(rasqal_graph_pattern *dest_gp, rasqal_graph_pattern *src_gp);
 int rasqal_engine_check_limit_offset(rasqal_query_results* query_results);
 int rasqal_engine_merge_triples(rasqal_query* query, rasqal_graph_pattern* gp, void* data);
@@ -510,7 +557,7 @@ int rasqal_triples_source_next_source(rasqal_triples_source* rts);
 int rasqal_engine_get_next_result(rasqal_query_results* query_results);
 void rasqal_engine_assign_binding_values(rasqal_query* query);
 void rasqal_engine_move_constraints(rasqal_graph_pattern* dest_gp, rasqal_graph_pattern* src_gp);
-int rasqal_engine_execute_order(rasqal_query_results* query_results);
+int rasqal_engine_execute_run(rasqal_query_results* query_results);
 void rasqal_engine_free_query_result_row(rasqal_query_result_row* row);
 rasqal_literal** rasqal_engine_get_results_values(rasqal_query_results* query_results);
 rasqal_literal* rasqal_engine_get_result_value(rasqal_query_results* query_results, int offset);
@@ -531,16 +578,20 @@ int rasqal_literal_ebv(rasqal_literal* l);
 int rasqal_expression_is_constant(rasqal_expression* e);
 void rasqal_expression_clear(rasqal_expression* e);
 void rasqal_expression_convert_to_literal(rasqal_expression* e, rasqal_literal* l);
+int rasqal_expression_mentions_variable(rasqal_expression* e, rasqal_variable* v);
 
 /* strcasecmp.c */
 #ifdef HAVE_STRCASECMP
-#define rasqal_strcasecmp strcasecmp
-#define rasqal_strncasecmp strncasecmp
+#  define rasqal_strcasecmp strcasecmp
+#  define rasqal_strncasecmp strncasecmp
 #else
-#ifdef HAVE_STRICMP
-#define rasqal_strcasecmp stricmp
-#define rasqal_strncasecmp strnicmp
-#endif
+#  ifdef HAVE_STRICMP
+#    define rasqal_strcasecmp stricmp
+#    define rasqal_strncasecmp strnicmp
+#   else
+int rasqal_strcasecmp(const char* s1, const char* s2);
+int rasqal_strncasecmp(const char* s1, const char* s2, size_t n);
+#  endif
 #endif
 
 /* rasqal_raptor.c */
@@ -579,7 +630,7 @@ rasqal_formula* rasqal_new_formula(void);
 void rasqal_free_formula(rasqal_formula* formula);
 void rasqal_formula_print(rasqal_formula* formula, FILE *stream);
 rasqal_formula* rasqal_formula_join(rasqal_formula* first_formula, rasqal_formula* second_formula);
-rasqal_graph_pattern* rasqal_engine_new_graph_pattern_from_formula(rasqal_query* query, rasqal_formula* formula, rasqal_graph_pattern_operator op);
+rasqal_graph_pattern* rasqal_engine_new_basic_graph_pattern_from_formula(rasqal_query* query, rasqal_formula* formula);
 rasqal_graph_pattern* rasqal_engine_group_2_graph_patterns(rasqal_query* query, rasqal_graph_pattern* first_gp, rasqal_graph_pattern* second_gp);
 
 /* The following should be public eventually in rasqal.h or raptor.h or ...? */
@@ -593,8 +644,6 @@ typedef void (rasqal_kv_free_fn)(const void *key, const void *value);
 
 
 /* rasqal_map.c */
-typedef struct rasqal_map_s rasqal_map;
-
 typedef void (*rasqal_map_visit_fn)(void *key, void *value, void *user_data);
 
 rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, rasqal_kv_free_fn* free_fn, raptor_sequence_print_handler* print_key_fn, raptor_sequence_print_handler* print_value_fn, int flags);
@@ -608,10 +657,14 @@ void rasqal_query_remove_query_result(rasqal_query* query, rasqal_query_results*
 unsigned char* rasqal_prefix_id(int prefix_id, unsigned char *string);
 int rasqal_query_declare_prefix(rasqal_query* rq, rasqal_prefix* prefix);
 int rasqal_query_declare_prefixes(rasqal_query* rq);
+unsigned char* rasqal_query_get_genid(rasqal_query* query, const unsigned char* base, int counter);
 
 /* rasqal_query_results.c */
+void rasqal_init_query_results(void);
+void rasqal_finish_query_results(void);
+
 rasqal_query_results* rasqal_new_query_results(rasqal_query* query);
-void rasqal_query_results_init(rasqal_query_results* query_results);
+void rasqal_query_results_reset(rasqal_query_results* query_results);
 
 /* end of RASQAL_INTERNAL */
 #endif
