@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdfproc.c - Redland RDF processor
+ * rdfproc.c - Redland RDF command processor
  *
- * $Id: rdfproc.c 8536 2006-01-26 05:04:45Z cmdjb $
+ * $Id: rdfproc.c 12153 2007-04-24 06:24:09Z dajobe $
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -92,7 +92,7 @@ enum command_type {
 typedef struct
 {
   enum command_type type;
-  char *name;
+  const char *name;
   int min_args; /* min args needed? */
   int max_args; /* max args needed? */
   int write; /* write to db? */
@@ -111,7 +111,7 @@ static command commands[]={
   {CMD_ADD, "add", 3, 4, 1},
   {CMD_REMOVE, "remove", 3, 4, 1},
   {CMD_ADD_TYPED, "add-typed", 5, 6, 1},
-  {CMD_PARSE_MODEL, "parse", 1, 3, 1},
+  {CMD_PARSE_MODEL, "parse", 1, 4, 1},
   {CMD_PARSE_STREAM, "parse-stream", 1, 4, 1},
   {CMD_ARCS_IN, "arcs-in", 1, 1, 0},
   {CMD_ARCS_OUT, "arcs-out", 1, 1, 0},
@@ -138,7 +138,7 @@ static command commands[]={
 #endif
 
 
-#define GETOPT_STRING "chno:ps:t:v"
+#define GETOPT_STRING "chno:pr:s:t:Tv"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_options[] =
@@ -149,8 +149,10 @@ static struct option long_options[] =
   {"new", 0, 0, 'n'},
   {"output", 1, 0, 'o'},
   {"password", 0, 0, 'p'},
+  {"results", 1, 0, 'r'},
   {"storage", 1, 0, 's'},
   {"storage-options", 1, 0, 't'},
+  {"transactions", 0, 0, 'T'},
   {"version", 0, 0, 'v'},
   {NULL, 0, 0, 0}
 };
@@ -191,7 +193,54 @@ log_handler(void *user_data, librdf_log_message *message)
   return 1;
 }
 
-  
+
+
+static void
+print_nodes(FILE* fh, librdf_iterator* iterator) 
+{
+  int count;
+  librdf_node* context_node;
+  librdf_node* node;
+
+  /* (Common code) Print out nodes */
+  count=0;
+  while(!librdf_iterator_end(iterator)) {
+    context_node=(librdf_node*)librdf_iterator_get_context(iterator);
+    node=(librdf_node*)librdf_iterator_get_object(iterator);
+    if(!node) {
+      fprintf(stderr, "%s: librdf_iterator_get_object returned NULL\n",
+              program);
+      break;
+    }
+    
+    fputs("Matched node: ", fh);
+    librdf_node_print(node, fh);
+    if(context_node) {
+      fputs(" with context ", fh);
+      librdf_node_print(context_node, fh);
+    }
+    fputc('\n', fh);
+    
+    count++;
+    librdf_iterator_next(iterator);
+  }
+  librdf_free_iterator(iterator);
+  fprintf(stderr, "%s: matching nodes: %d\n", program, count);
+}
+
+
+static void
+print_node(FILE* fh, librdf_node* node) 
+{
+  if(node) {
+    fputs("Matched node: ", fh);
+    librdf_node_print(node, fh);
+    fputc('\n', fh);
+    
+    librdf_free_node(node);
+  }
+}
+
 
 int
 main(int argc, char *argv[]) 
@@ -206,7 +255,7 @@ main(int argc, char *argv[])
   librdf_stream* stream;
   librdf_iterator* iterator;
   librdf_uri *uri;
-  librdf_uri *base_uri;
+  librdf_uri *base_uri=NULL;
   librdf_query *query;
   librdf_query_results *results;
   librdf_hash *options;
@@ -224,6 +273,7 @@ main(int argc, char *argv[])
   char *p;
   int i;
   int rc;
+  int transactions=0;
   char *storage_name=(char*)default_storage_name;
   char *storage_options=(char*)default_storage_options;
   char *storage_password=NULL;
@@ -236,6 +286,7 @@ main(int argc, char *argv[])
   size_t capacity;
   size_t size;
   const char *query_graph_serializer_syntax_name="rdfxml";
+  char* results_format=NULL;
 
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -272,10 +323,10 @@ main(int argc, char *argv[])
 #else
     c = getopt (argc, argv, GETOPT_STRING);
 #endif
-    if (c == -1)
+    if(c == -1)
       break;
 
-    switch (c) {
+    switch(c) {
       case 0:
       case '?': /* getopt() - unknown option */
         usage=1;
@@ -365,14 +416,51 @@ main(int argc, char *argv[])
         }
         break;
 
+      case 'r':
+        if(optarg) {
+          if(!strcmp(optarg, "help")) {
+            fprintf(stderr, "%s: Valid query result formats are:\n", program);
+            for(i=0; 1; i++) {
+              const char *format_name;
+              const char *format_label;
+              if(rasqal_query_results_formats_enumerate_full(i,
+                                                             &format_name, 
+                                                             &format_label,
+                                                             NULL, NULL))
+                break;
+              printf("  %-20s  %s\n", format_name, format_label);
+            }
+            exit(0);
+          } else {
+            if(rasqal_query_results_formats_check(optarg, NULL, NULL))
+              results_format=optarg;
+            else {
+              fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(r, results) "'\nTry '%s " HELP_ARG(r, results) " help' for a list of valid formats\n",
+                      program, optarg, program);
+              usage=1;
+            }
+          }
+        }
+        break;
+        
       case 's':
-        storage_name=optarg;
+        if(optarg) {
+          if(!librdf_get_storage_factory(world, optarg)) {
+            fprintf(stderr, "%s: invalid storage `%s'\n", program, optarg);
+            usage=1;
+          } else
+            storage_name=optarg;
+        }
         break;
 
       case 't':
         storage_options=optarg;
         break;
 
+      case 'T':
+        transactions=1;
+        break;
+
       case 'v':
         fputs(librdf_version_string, stdout);
         fputc('\n', stdout);
@@ -453,11 +541,25 @@ main(int argc, char *argv[])
     puts(HELP_TEXT(o, "output FORMAT   ", "Set the triple output format to one of:"));
     puts("    simple                  A simple format (default)\n    ntriples                N-Triples\n    rdfxml                  RDF/XML");
     puts(HELP_TEXT(p, "password        ", "Read storage option 'password' from standard input"));
+    puts(HELP_TEXT(r, "results FORMAT  ", "Set the query results format"));
+    for(i=0; 1; i++) {
+      const char *help_name;
+      const char *help_label;
+      if(librdf_query_results_formats_enumerate(world, i, 
+                                                &help_name, &help_label, 
+                                                NULL, NULL))
+        break;
+      printf("    %-10s              %s", help_name, help_label);
+      if(!i)
+        puts(" (default)");
+      else
+        putchar('\n');
+    }
     puts(HELP_TEXT(s, "storage TYPE    ", "Set the graph storage type"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
-      if(librdf_storage_enumerate(i, &help_name, &help_label))
+      if(librdf_storage_enumerate(world, i, &help_name, &help_label))
         break;
       printf("    %-10s              %s", help_name, help_label);
       if(!i)
@@ -468,7 +570,7 @@ main(int argc, char *argv[])
     printf(HELP_TEXT(t, "storage-options OPTIONS\n                        ", "Storage options (default \"%s\")\n"), default_storage_options);
     puts(HELP_TEXT(v, "version         ", "Print the Redland version"));
     puts("\nCommands:");
-    puts("  parse FILE|URI [SYNTAX [BASEURI]]");
+    puts("  parse FILE|URI [SYNTAX [BASEURI [CONTEXT]]]");
     puts("  parse-stream FILE|URI [SYNTAX [BASEURI [CONTEXT]]]");
     puts("      Parse RDF syntax (default RDF/XML) in FILE or URI into the graph");
     puts("      with optional BASEURI, into the optional CONTEXT.");
@@ -509,7 +611,7 @@ main(int argc, char *argv[])
 
   if(commands[cmd_index].write) {
     librdf_hash_put_strings(options, "write", "yes");
-    if (is_new)
+    if(is_new)
       librdf_hash_put_strings(options, "new", "yes");
   }
 
@@ -531,6 +633,8 @@ main(int argc, char *argv[])
     return(1);
   }
 
+  if(transactions)
+    librdf_model_transaction_start(model);
 
   /* Do this or gcc moans */
   stream=NULL;
@@ -549,6 +653,8 @@ main(int argc, char *argv[])
     case CMD_PRINT:
       librdf_model_print(model, stdout);
       break;
+
+
     case CMD_PARSE_MODEL:
     case CMD_PARSE_STREAM:
       uri_string=(unsigned char *)argv[0];
@@ -564,7 +670,8 @@ main(int argc, char *argv[])
       
       parser=librdf_new_parser(world, ((argc > 1) ? argv[1] : NULL), NULL, NULL);
       if(!parser) {
-        fprintf(stderr, "%s: Failed to create new parser %s\n", program, argv[1]);
+        fprintf(stderr, "%s: Failed to create new parser %s\n", program,
+                argv[1]);
         librdf_free_uri(uri);
         break;
       }
@@ -573,57 +680,61 @@ main(int argc, char *argv[])
               (argc > 1) ? argv[1] : "default");
       
       if(argc >= 3 && argv[2]) {
-        base_uri=librdf_new_uri(world, (const unsigned char *)argv[2]);
-        if(!base_uri) {
-          fprintf(stderr, "%s: Failed to create base URI from %s\n", program, argv[2]);
-          break;
+        base_uri=NULL;
+        if(strcmp(argv[2], "-")) {
+          base_uri=librdf_new_uri(world, (const unsigned char *)argv[2]);
+          if(!base_uri) {
+            fprintf(stderr, "%s: Failed to create base URI from %s\n", program,
+                    argv[2]);
+            break;
+          }
         }
         fprintf(stderr, "%s: Using base URI %s\n", program,
-                librdf_uri_as_string(base_uri));
+                (base_uri ? (const char*)librdf_uri_as_string(base_uri) : "NULL"));
 
         target=NULL; /* context node */
         if(argc >= 4 && argv[3]) {
-          if (librdf_heuristic_is_blank_node(argv[3]))
+          if(librdf_heuristic_is_blank_node(argv[3]))
             target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
           else
             target=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[3]);
         }
         fprintf(stderr, "%s: Using context node %s\n", program,
-                argv[3]);
+                (argv[3] ? argv[3] : "NULL"));
       } else
         base_uri=librdf_new_uri_from_uri(uri);
 
       rc=0;
 
-      if (type == CMD_PARSE_MODEL) {
+      if(type == CMD_PARSE_MODEL && !target) {
         if(librdf_parser_parse_into_model(parser, uri, base_uri, model)) {
           fprintf(stderr, "%s: Failed to parse into the graph\n", program);
           rc=1;
         }
       } else {
-        /* must be CMD_PARSE_STREAM */
+        /* either CMD_PARSE_STREAM or it's a parse into context */
+        count=0;
         if(!(stream=librdf_parser_parse_as_stream(parser, uri, base_uri))) {
           fprintf(stderr, "%s: Failed to parse RDF as stream\n", program);
           rc=1;
-        }
-        
-        count=0;
-        while(!librdf_stream_end(stream)) {
-          librdf_statement *statement=librdf_stream_get_object(stream);
-          if(!statement) {
-            fprintf(stderr, "%s: librdf_stream_next returned NULL\n", program);
-            break;
-          }
+        } else {
+          while(!librdf_stream_end(stream)) {
+            librdf_statement *statement=librdf_stream_get_object(stream);
+            if(!statement) {
+              fprintf(stderr, "%s: librdf_stream_next returned NULL\n", program);
+              break;
+            }
           
-          if(target)  /* context node */
-            librdf_model_context_add_statement(model, target, statement);
-          else
-            librdf_model_add_statement(model, statement);
-          count++;
-          librdf_stream_next(stream);
+            if(target)  /* context node */
+              librdf_model_context_add_statement(model, target, statement);
+            else
+              librdf_model_add_statement(model, statement);
+            count++;
+            librdf_stream_next(stream);
+          }
+          librdf_free_stream(stream);  
         }
-        librdf_free_stream(stream);  
-        
+
         if(target) {
           librdf_free_node(target);
           target=NULL;
@@ -640,18 +751,18 @@ main(int argc, char *argv[])
         librdf_node* warning_count_node;
         int error_count, warning_count;
 
-        error_count_node =librdf_parser_get_feature(parser, error_count_uri);
+        error_count_node=librdf_parser_get_feature(parser, error_count_uri);
         if(error_count_node) {
-          error_count= atoi((const char*)librdf_node_get_literal_value(error_count_node));
+          error_count=atoi((const char*)librdf_node_get_literal_value(error_count_node));
           librdf_free_node(error_count_node);
         } else {
           fprintf(stderr, "%s: Could not get parsing error count\n", program);
           error_count= (-1);
         }
 
-        warning_count_node =librdf_parser_get_feature(parser, warning_count_uri);
+        warning_count_node=librdf_parser_get_feature(parser, warning_count_uri);
         if(warning_count_node) {
-          warning_count =atoi((const char*)librdf_node_get_literal_value(error_count_node));
+          warning_count=atoi((const char*)librdf_node_get_literal_value(error_count_node));
           librdf_free_node(warning_count_node);
         } else {
           fprintf(stderr, "%s: Could not get parsing warning count\n", program);
@@ -670,11 +781,15 @@ main(int argc, char *argv[])
       
       librdf_free_parser(parser);
       librdf_free_uri(uri);
-      librdf_free_uri(base_uri);
+      if(base_uri)
+        librdf_free_uri(base_uri);
       break;
 
+
     case CMD_SERIALIZE:
-        /* args are name (optional), uri (may be NULL), mime_type (optional), base URI (optional) */
+        /* args are name (optional), uri (may be NULL), mime_type
+         * (optional), base URI (optional)
+         */
 
         uri=NULL;
         name=NULL;
@@ -698,12 +813,12 @@ main(int argc, char *argv[])
           }
         }
 
-        if(argc ==3) {
+        if(argc == 3) {
           if(strcmp(argv[2], "-"))
             mime_type=argv[2];
         }
 
-	if(argc ==4) {
+	if(argc == 4) {
           if(strcmp(argv[3], "-")) {
             base_uri=librdf_new_uri(world, (const unsigned char *)argv[3]);
             if(!base_uri) {
@@ -722,7 +837,8 @@ main(int argc, char *argv[])
           break;
         }
 
-        librdf_serializer_serialize_model_to_file_handle(serializer, stdout, NULL, model);
+        librdf_serializer_serialize_model_to_file_handle(serializer, stdout,
+                                                         NULL, model);
 
         librdf_free_serializer(serializer);
         if(uri)
@@ -730,6 +846,7 @@ main(int argc, char *argv[])
 
         break;
       
+
     case CMD_QUERY:
       /* args are name, uri (may be NULL), query_string/mime_type */
 
@@ -742,15 +859,18 @@ main(int argc, char *argv[])
       else {
         uri=librdf_new_uri(world, (const unsigned char *)argv[1]);
         if(!uri) {
-          fprintf(stderr, "%s: Failed to create URI from %s\n", program, argv[1]);
+          fprintf(stderr, "%s: Failed to create URI from %s\n", program,
+                  argv[1]);
           break;
         }
       }
 
-      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2], NULL);
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2],
+                             NULL);
       goto printmatching;
       break;
 
+
     case CMD_QUERY_AS_BINDINGS:
       /* args are name (optional), uri (may be NULL), query_string */
       uri=NULL;
@@ -764,14 +884,16 @@ main(int argc, char *argv[])
       else {
         uri=librdf_new_uri(world, (const unsigned char *)argv[1]);
         if(!uri) {
-          fprintf(stderr, "%s: Failed to create URI from %s\n", program, argv[1]);
+          fprintf(stderr, "%s: Failed to create URI from %s\n", program,
+                  argv[1]);
           break;
         }
       }
 
       query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2], NULL);
       if(!query) {
-        fprintf(stderr, "%s: Failed to create new query %s\n", program, argv[2]);
+        fprintf(stderr, "%s: Failed to create new query %s\n", program,
+                argv[2]);
         if(uri)
           librdf_free_uri(uri);
         break;
@@ -787,7 +909,23 @@ main(int argc, char *argv[])
         break;
       }
 
-      if (librdf_query_results_is_bindings(results)) {
+      if(results_format) {
+        raptor_iostream *iostr;
+        librdf_query_results_formatter *formatter;
+
+        fprintf(stdout, "%s: Formatting query result as '%s':\n", program,
+                results_format);
+
+        iostr=raptor_new_iostream_to_file_handle(stdout);
+        formatter=librdf_new_query_results_formatter(results,
+                                                     results_format, NULL);
+
+        librdf_query_results_formatter_write(iostr, formatter, results, 
+                                             base_uri);
+
+        librdf_free_query_results_formatter(formatter);
+        raptor_free_iostream(iostr);
+      } else if(librdf_query_results_is_bindings(results)) {
         fprintf(stdout, "%s: Query returned bindings results:\n", program);
         
         while(!librdf_query_results_finished(results)) {
@@ -812,11 +950,11 @@ main(int argc, char *argv[])
 
         fprintf(stdout, "%s: Query returned %d results\n", program, 
                 librdf_query_results_get_count(results));
-     } else if (librdf_query_results_is_boolean(results)) {
+     } else if(librdf_query_results_is_boolean(results)) {
        fprintf(stdout, "%s: Query returned boolean result: %s\n",
                program,
                librdf_query_results_get_boolean(results) ? "true" : "false");
-     } else if (librdf_query_results_is_graph(results)) {
+     } else if(librdf_query_results_is_graph(results)) {
        librdf_storage* tmp_storage;
        librdf_model* tmp_model;
 
@@ -859,6 +997,7 @@ main(int argc, char *argv[])
       librdf_free_query_results(results);
       break;
       
+
     case CMD_CONTAINS:
     case CMD_FIND:
     case CMD_MATCH:
@@ -867,7 +1006,7 @@ main(int argc, char *argv[])
     case CMD_ADD_TYPED:
       if(!strcmp(argv[0], "-"))
         source=NULL;
-      else if (librdf_heuristic_is_blank_node(argv[0]))
+      else if(librdf_heuristic_is_blank_node(argv[0]))
         source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
       else
         source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -886,7 +1025,7 @@ main(int argc, char *argv[])
         if(!strcmp(argv[2], "-"))
           target=NULL;
         else {
-          if (!strncmp(argv[2], "_:", 2))
+          if(!strncmp(argv[2], "_:", 2))
             target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)argv[2]+2);
           else if(librdf_heuristic_object_is_literal(argv[2]))
             target=librdf_new_node_from_literal(world, (const unsigned char *)argv[2], NULL, 0);
@@ -925,7 +1064,7 @@ main(int argc, char *argv[])
           if(type==CMD_FIND || type==CMD_MATCH) {
             if(argc == 4 || type==CMD_MATCH) {
               if(argc == 4) {
-                if (librdf_heuristic_is_blank_node(argv[3]))
+                if(librdf_heuristic_is_blank_node(argv[3]))
                   context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
                 else
                   context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[3]);
@@ -996,7 +1135,7 @@ main(int argc, char *argv[])
         case CMD_ADD:
         case CMD_ADD_TYPED:
           if(argv[3]) {
-            if (librdf_heuristic_is_blank_node(argv[3]))
+            if(librdf_heuristic_is_blank_node(argv[3]))
               context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
             else
               context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[3]);
@@ -1017,7 +1156,7 @@ main(int argc, char *argv[])
           
         case CMD_REMOVE:
           if(argv[3]) {
-            if (librdf_heuristic_is_blank_node(argv[3]))
+            if(librdf_heuristic_is_blank_node(argv[3]))
               context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[3]));
             else
               context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[3]);
@@ -1042,9 +1181,10 @@ main(int argc, char *argv[])
       librdf_free_statement(partial_statement);
       break;
       
+
     case CMD_SOURCES:
       arc=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
-      if (librdf_heuristic_is_blank_node(argv[1]))
+      if(librdf_heuristic_is_blank_node(argv[1]))
         target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[1]));
       else if(librdf_heuristic_object_is_literal(argv[1]))
         target=librdf_new_node_from_literal(world, (const unsigned char *)argv[1], NULL, 0);
@@ -1057,14 +1197,20 @@ main(int argc, char *argv[])
         break;
       }
       
-      /* FALLTHROUGH */
+      print_nodes(stdout, iterator);
+
+      librdf_free_node(arc);
+      librdf_free_node(target);
+      break;
+      
+
     case CMD_ARCS:
       if(!iterator) {
-        if (librdf_heuristic_is_blank_node(argv[0]))
+        if(librdf_heuristic_is_blank_node(argv[0]))
           source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
         else
           source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
-        if (librdf_heuristic_is_blank_node(argv[1]))
+        if(librdf_heuristic_is_blank_node(argv[1]))
           target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[1]));
         else if(librdf_heuristic_object_is_literal(argv[1]))
           target=librdf_new_node_from_literal(world, (const unsigned char *)argv[1], NULL, 0);
@@ -1076,11 +1222,17 @@ main(int argc, char *argv[])
           break;
         }
       }
+
+      print_nodes(stdout, iterator);
+
+      librdf_free_node(source);
+      librdf_free_node(target);
+      break;
       
-      /* FALLTHROUGH */
+
     case CMD_TARGETS:
       if(!iterator) {
-        if (librdf_heuristic_is_blank_node(argv[0]))
+        if(librdf_heuristic_is_blank_node(argv[0]))
           source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
         else
           source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -1091,43 +1243,17 @@ main(int argc, char *argv[])
           break;
         }
       }
-      
-      /* (Common code) Print out nodes */
-      count=0;
-      while(!librdf_iterator_end(iterator)) {
-        context_node=(librdf_node*)librdf_iterator_get_context(iterator);
-        node=(librdf_node*)librdf_iterator_get_object(iterator);
-        if(!node) {
-          fprintf(stderr, "%s: librdf_iterator_get_object returned NULL\n",
-                  program);
-          break;
-        }
-        
-        fputs("Matched node: ", stdout);
-        librdf_node_print(node, stdout);
-        if(context_node) {
-          fputs(" with context ", stdout);
-          librdf_node_print(context_node, stdout);
-        }
-        fputc('\n', stdout);
-        
-        count++;
-        librdf_iterator_next(iterator);
-      }
-      librdf_free_iterator(iterator);
-      fprintf(stderr, "%s: matching nodes: %d\n", program, count);
-      
-      if(source)
-        librdf_free_node(source);
-      if(arc)
-        librdf_free_node(arc);
-      if(target)
-        librdf_free_node(target);
+
+      print_nodes(stdout, iterator);
+
+      librdf_free_node(source);
+      librdf_free_node(arc);
       break;
       
+
     case CMD_SOURCE:
       arc=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
-      if (librdf_heuristic_is_blank_node(argv[1]))
+      if(librdf_heuristic_is_blank_node(argv[1]))
         target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[1]));
       else if(librdf_heuristic_object_is_literal(argv[1]))
         target=librdf_new_node_from_literal(world, (const unsigned char *)argv[1], NULL, 0);
@@ -1142,14 +1268,19 @@ main(int argc, char *argv[])
         break;
       }
       
-      /* FALLTHROUGH */
+      print_node(stdout, node);
+      librdf_free_node(arc);
+      librdf_free_node(target);
+      break;
+
+
     case CMD_ARC:
       if(!node) {
-        if (librdf_heuristic_is_blank_node(argv[0]))
+        if(librdf_heuristic_is_blank_node(argv[0]))
           source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
         else
           source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
-      if (librdf_heuristic_is_blank_node(argv[1]))
+      if(librdf_heuristic_is_blank_node(argv[1]))
         target=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[1]));
       else if(librdf_heuristic_object_is_literal(argv[1]))
         target=librdf_new_node_from_literal(world, (const unsigned char *)argv[1], NULL, 0);
@@ -1164,10 +1295,15 @@ main(int argc, char *argv[])
         }
       }
       
-      /* FALLTHROUGH */
+      print_node(stdout, node);
+      librdf_free_node(source);
+      librdf_free_node(target);
+      break;
+
+
     case CMD_TARGET:
       if(!node) {
-        if (librdf_heuristic_is_blank_node(argv[0]))
+        if(librdf_heuristic_is_blank_node(argv[0]))
           source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
         else
           source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -1181,27 +1317,15 @@ main(int argc, char *argv[])
         }
       }
       
-      /* (Common code) Print out node */
-      if(node) {
-        fputs("Matched node: ", stdout);
-        librdf_node_print(node, stdout);
-        fputc('\n', stdout);
-        
-        librdf_free_node(node);
-      }
-      if(source)
-        librdf_free_node(source);
-      if(arc)
-        librdf_free_node(arc);
-      if(target)
-        librdf_free_node(target);
+      print_node(stdout, node);
+      librdf_free_node(source);
+      librdf_free_node(arc);
       break;
-
-
       
+
     case CMD_ARCS_IN:
     case CMD_ARCS_OUT:
-      if (librdf_heuristic_is_blank_node(argv[0]))
+      if(librdf_heuristic_is_blank_node(argv[0]))
         source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
       else
         source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -1244,7 +1368,7 @@ main(int argc, char *argv[])
       
     case CMD_HAS_ARC_IN:
     case CMD_HAS_ARC_OUT:
-      if (librdf_heuristic_is_blank_node(argv[0]))
+      if(librdf_heuristic_is_blank_node(argv[0]))
         source=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
       else
         source=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -1260,8 +1384,9 @@ main(int argc, char *argv[])
       librdf_free_node(arc);
       break;
 
+
     case CMD_REMOVE_CONTEXT:
-      if (librdf_heuristic_is_blank_node(argv[0]))
+      if(librdf_heuristic_is_blank_node(argv[0]))
         context_node=librdf_new_node_from_blank_identifier(world, (const unsigned char *)librdf_heuristic_get_blank_node(argv[0]));
       else
         context_node=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[0]);
@@ -1322,7 +1447,7 @@ main(int argc, char *argv[])
     librdf_free_query(query);
 
   if(free_uri_string)
-    SYSTEM_FREE(uri_string);
+    free(uri_string);
 
   if(output_model) {
     if(librdf_serializer_serialize_model_to_file_handle(output_serializer, 
@@ -1338,6 +1463,9 @@ main(int argc, char *argv[])
   
   librdf_free_hash(options);
 
+  if(transactions)
+    librdf_model_transaction_commit(model);
+
   librdf_free_model(model);
   librdf_free_storage(storage);
 
